/** AttributeCategory.js
* @author: HADW
*/
/**
* Structure of main object attributeCategory  {
*   attributes: array of attObj elements
*   cells: 2D array containing the layout and the attributes as cellObj objects of the current view
*   nRows: number of rows in the table model
*   nCols: number of columns in the table model
*	layout: layout of the current view, one of OneColumn, Grid
*	currentUser: instance of currentUser derived from global gCurrentUser object
*   valueClass: Class to be used for the values
*   labelClass: class to be used for the labels
*   errorMsg:  error message which will be displayed after ajax loading failsla
*   name: name of the attribute category
*   type: to what WCR object this attribute category is attached (one of: project, document, task)
*	projectID: ID of project this attr. cat. is attached to. Initialized for all types (if known)
*	documentID: ID of document this attr. cat. is attached to. Used only if type = document 
*	docVersionID: ID of document version this attr. cat. is attached to. Used only if type = document
*	taskID: ID of task this attr. cat. is attached to. Used only if type = task
*   views: array of viewObj. if views is null, no views are used (so only the default view)
* 	rules: array of ruleObj, each one representing a (visibility) rule used in the attribute category
*	styles: array of styleObj, each one representing a style used in the attribute category
*	keepSessionAlive: if true, will send keepSessionAlive ajax calls to the web server to keep the session alive. default off
* }
* Structure of attObj {
*   id: id of the attribute like in the database
*   name: name of the attribute
*   type: non-localized type of the attribute
*   localizedType: localized type of the attribute
*   value: value converted to the preference unit if applicable
*   unit: non-localized unit of the attribute
* 	localizedUnit: localized unite of the attribute
* }
* Structure of cellObj {
*   type:  type of the cell, one of Attribute, Title, Table, Ruler, Spacer, HeaderBar, Button
*   id: 	id of the attribute put on this position. Null if it's not an attribute
*   colSpan: 	Number of columns which this element will span. These subsequent cells should be null in the data model.
*   attObj: attribute object (an element of this.attributes) if type is "Attribute"
*   label:	Label which will be written in the first column, or text for a title
*   background: color value of the background for spacer cells
* }
* Structure of viewObj {
*    nRows: number of rows in the view
*    nCols: number of columns in the view
*    layout: layout of the view
*    name: name of the view
* }
* Structure of currentUser {
* 	userID: ID of the currently logged in user
* 	lang: language tag of the user (f.e. en, fr, cz, etc.)
* 	units: units set for the user (metric or imperial)
* 	tzOffset: time zone offset set for the user
* 	canEditAttributes: flag whether the user can edit attributes or just see them read-only.
* }
*/

//TO DO HADW: (these are minor problems)
// Styles on the multi-value list is not applied (it's applied to the label and to the background of the value but for example font is not applied
// Editing a JavaScript button: when you select another cell before defocusing the javaScript text area, you lose your changes
// Check working of buttons in non-edit mode (should not have any working at all). 


document.write('<link rel="stylesheet" type="text/css" href="styles/richtext.css"/>');  // for read only rich text attributes (in order not to inherit all webcenter styes). To avoid that this must be added to all places where AttributeCategory.js is used
document.write('<script type="text/javascript" src="./javascript/json-minified.js"></scr' + 'ipt>');  // for json parsing. To avoid that this js must be added to all places where AttributeCategory.js is used

var gCustomFormExtension = null;
document.write('<script type="text/javascript" src="./custom/javascript/CustomFormExtension.js"></scr' + 'ipt>');  // Allows extending the code with utilities for JavaScript scripting by setting gCustomFormExtension


var storedObj= null;  // purely for debugging
var debugMsg = "";   // also for debugging
var sessionLostMsg = "Your session is lost. You will not be able to save your work. Please log on again";

var cCollapseIcon = "./images/collapse.png";  // icon for tree view in search results
var cExpandIcon = "./images/expand.png";  // icon for tree view in search results
var cMaxButtonClick = 200;   // maximum number of times you are allowed to click a button - to detect endless loops
var cMaxResultsDBValues = 2000;  // maximum number of values you ever get from the database for cascading dropdowns.  Beyond this point users must use filtering
var gACatCache = new Array();
var gTextManagementEnabled = true;  // enable/disable text management rapidly

if (!tinyMceMgrObj){
	var tinyMceMgrObj = new tinyMceMgr();
	$(document).ready(function() {
		tinyMceMgrObj.initTinyMCEInstances();
	});
}

/**
* attributeCategory constructor
* @param aCatOptions: object with all the options. See the UDD for more details.
*/

function attributeCategory(aCatOptions) {
    try {

		this.aCatOptions = aCatOptions;
        this.displayId = aCatOptions.ACatPanel;
        this.id = aCatOptions.ACatObj;
        this.saveFieldId = aCatOptions.saveFieldId;
        var storeElem = document.getElementById(this.id);
        if (storeElem) storeElem.ACatObj = this;
        this.functionPrefix = 'javascript:document.getElementById(\'' + this.id + '\').ACatObj';        
        this.readURL = aCatOptions.readURL;
        this.currentUser = aCatOptions.currentUser;
        this.labelClass = "darktext_darklink_bold_11";
        this.valueClass = "darktext_darklink_11";       
		this.alternateBackground = true;  // lightgrey/white alternating background by default
        this.cViewOptions = new Array("Default","Creation","Header");  // possible view names
        this.cOperators = new Array("=",">",">=","<","<=","!=","One Of");
		this.alignmentOptions = new Array("Left","Center","Right");
		this.fontOptions = new Array("Verdana","Arial","ArialNarrow","Comic Sans MS","Courier New","Georgia","Times New Roman","Trebuchet, Trebuchet MS");
		this.fontStyleOptions = new Array("Normal","Italic","Oblique");
		this.fontWeightOptions = new Array("Normal","Bold");
		this.widthHeightUnitOptions = new Array("px","%");
		this.documentLinkOperations = ['Link','Copy','CopyLink','Move'];  // possible operations when adding a new document link
		this.documentLinkSources =['Browse','ProjectFolder','DocumentTemplates','SavedSearch'];	  // possible methods to find a browse/find a document to link via a document link	
		this.dlsProperties = ['sourceType','label', 'documentType','operation','targetFolder','sourceFolder','sourceProject','savedSearch'];
		this.dltProperties = ['type','typeDetail', 'label','visibility'];
		this.buttonFuncOptions = ['LookUpAttributes','CheckUniqueValue','JavaScript'];
		
		this.components = new Array(1, 2); //1:textfield, 2:dropdown
		this.showTypeAsUnit = true;  // show type of attributes and dropdowns if not a unit
		this.softwareVersion = 2;
		this.multiSep = ' | ';   // separator for multi-value attributes
		this.valueChangedListeners = new Array();
		this.loaded = false;
		
		// Setup of list reading via servlet for cascading dropdowns based on Excel
		if (aCatOptions.readListsURL) {
			this.readListsURL = aCatOptions.readListsURL;
		} else {
			this.readListsURL = "ReadExcelServletCombined";
		}
		if (aCatOptions.allListsURL) {
			this.allListsURL = aCatOptions.allListsURL;
		} else {
			this.allListsURL = "ReadExcelServletCombined?action=getlists";
		}

		this.isTaskTypePage = aCatOptions.isTaskTypePage;
		if(this.isTaskTypePage){
			this.canEditTaskType = aCatOptions.canEditTaskType;
			this.assignRolesByTaskSpecsCheckbox = aCatOptions.assignRolesByTaskSpecsCheckbox;
			this.autoUpdateProjAttrsCheckbox = aCatOptions.autoUpdateProjAttrsCheckbox;
		}
		this.projectAttributeValuesURL = aCatOptions.projectAttributeValuesURL;
		
		// adjust rendering for printing
		if (aCatOptions.printLayout) {
			this.printLayout = aCatOptions.printLayout;
		} else {
			this.printLayout = false;			
		}

		// set up for help windows
		var helpHeight = 500, helpWidth = 500; 
		var leftVal = (screen.width - helpWidth) / 2;
		var topVal = (screen.height - helpHeight) / 2;
		this.helpOptionsString = '&quot;scrollbars=1, width='+ helpWidth +' ,height='+ helpHeight +',top='+topVal+',left='+leftVal+' &quot';
		
		this.terms = this.addTerms();
		this.showEditTerminology = true;  // moved to default true from WCR 12 onwards
		if (this.isTaskTypePage) this.showEditTerminology = false;  // new in 14 but behind easter egg
		this.showInfoPage = true;	// moved to default true from WCR 12 onwards
		
        
        this.currentViewIdx = 0; // index in cViewOptions of the current view 
		this.currentViewFound = false;
		this.noCurrentViewDisplay = false;  // indicates that a task type view was asked but not found --> nothing will be displayed and lists will not be loaded

        this.styles = new Array(); // array containing standard and user defined styles
		// standard styles
        this.styles.push({
            name:"WebCenter Attribute",
            classLabel:"darktext_darklink_11",
            classValue:"darktext_darklink_11",
            standardStyle:true
        });
        this.styles.push({
            name:"WebCenter Title",
            classLabel:"darktext_darklink_bold_12",
            classValue:"darktext_darklink_bold_11",  // value not used for titles
            background:"#EDEDED" ,
            standardStyle:true
        });
		
		
        this.rules = new Array(); // array containing standard and user defined rules
        this.rules.push({
            name:"Editable",
            standardRule:true
        });
        this.isRequiredRule = {
            name:"Required",
            standardRule:true
        };
        this.rules.push(this.isRequiredRule);
        this.rules.push({
            name:"ReadOnly",
            standardRule:true
        });
        this.rules.push({
            name:"Hidden",
            standardRule:true
        });
		this.rules.push({
            name:"HiddenKeepValues",
            standardRule:true
        });

		this.layoutId = aCatOptions.layoutId;
		
        this.setDefaultLanguageStrings(); // strings (in English) which will be used if aCatOptions does not contain languageStrings
        if (this.layoutId) this.setDefaultLayoutLanguageStrings();
		if (aCatOptions.languageStrings) this.setLanguageStrings(aCatOptions.languageStrings);



        this.cells = new Array();  // array containing the cells of both onecolumn and grid layout. Always 2D even for OneColumn
        this.debug = 0;  // to debug ajax calls
        this.nRows = 2;
        this.nCols = 2;
        this.name = "";
		
        
        this.typeOptions = ['project','document','task'];   // possible options for attribute type
        if (aCatOptions.type) this.type = aCatOptions.type;   // even this might be empty (in searches)
        if (aCatOptions.projectID) this.projectID = aCatOptions.projectID; else this.projectID = "";
        if (aCatOptions.documentID) this.documentID = aCatOptions.documentID; else this.documentID = "";
        if (aCatOptions.docVersionID) this.docVersionID = aCatOptions.docVersionID; else this.docVersionID = "";
        if (aCatOptions.taskID) this.taskID = aCatOptions.taskID; else this.taskID = "";

        
        this.allAttributesURL = aCatOptions.allAttributesURL;

        if (aCatOptions.view) this.currentView = aCatOptions.view; else this.currentView = "Default";
		this.requestedView = this.currentView;  // remembers what was originally asked as view.  Needed for remembering that a creation view was asked to also submit hidden/readonly attributes

        // prefix to be put before names and id to make distinction between different panels on the same page
        if (aCatOptions.prefix) this.prefix = aCatOptions.prefix; else this.prefix="";
        if (aCatOptions.callbackOnload) this.callbackOnload = aCatOptions.callbackOnload;
        if (aCatOptions.callbackOnFail) this.callbackOnFail = aCatOptions.callbackOnFail;

		// default hour, given as a number between 0 and 47 (number of half hours)
		this.defaultHourAsHalfHours = 34;
		if (aCatOptions.defaultHourAsHalfHours) this.defaultHourAsHalfHours = aCatOptions.defaultHourAsHalfHours;

		// no validation for templates
		this.validationNeeded = true;
		if (aCatOptions.isTemplate != null) {			
			if (aCatOptions.isTemplate == 1 || aCatOptions.isTemplate == "1") this.validationNeeded = false;
		}
		
		// no validation needed if not editable
		if (this.currentUser && !this.currentUser.canEditAttributes){
			this.validationNeeded = false;
		}
		
		// check whether values of attributes need to be loaded
		this.copyValues = true;
		if (aCatOptions.copyValues != null) this.copyValues = aCatOptions.copyValues;
		
		// check whether grid layout is allowed (grid layout is a licensed option in WebCenter)
		this.allowGridLayout = true;
		if (aCatOptions.allowGridLayout != null) this.allowGridLayout = aCatOptions.allowGridLayout;
		
		this.checkIdenticalLists = false;  // remembers whether you already checked for identical lists

        this.valuesURL = aCatOptions.valuesURL;  // URL to load the values via Ajax
        this.valuesURLMerge = aCatOptions.valuesURLMerge;  // URL to load the values via Ajax to merge with first loaded values
		this.defaultPanelWidth = 270; // default width of the tree panel in search results
        if (this.currentView != 'Header') storedObj = this;  // easier to debug in firebug

        if (aCatOptions.readURL) {  // read the attribute category via ajax
            this.readACatFromXML(aCatOptions.readURL);
        };
		anAttributeChanged = false;  // global variable allowing the Export to JDF button to detect whether you can export or not  CR HADW: is this still trust-worthy?

		if(this.autoUpdateProjAttrsCheckbox) tinyMceMgrObj.attCatObj = this; // in this case, the onchange of the rich text attributes need the cell details
																			 // always adding it would lead to conflicts when there are multiple layouts on one page
		this.keepSessionAlive = false;
		
		// flag to see if this category is imported (ID's won't match in that case)
		this.imported = false;
		
		this.addOtherProperties(); // Only for the Easter Egg "Extended form builder" 
		if (gTextManagementEnabled && aCatOptions.projectID) {
			this.projectID = aCatOptions.projectID;			
		}
    } catch(err) {

        alert(this.langArr['ACatConstructorProblem'] + err.lineNumber + ': ' +  err.message );
    }
	return this;
}



///////////////////////////////////////////////////////////////////////////////////////////////////////////
// LOCALIZATION FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////

/** 
* Sets language strings for the UI
* Obligatory call except for unit testing
* @param langArr associative array with language values
* Overwrites the defaults. Whatever is not overwritten stays what it was and will therefore be in English
*/

attributeCategory.prototype.setLanguageStrings = function(langArr) {
  
  try {
	  for (var key in langArr) {
		this.langArr[key] = langArr[key];
	  } 
	} catch(err) {

        alert(this.langArr['ACatLangStringsProblem'] + err.lineNumber + ': ' +  err.message );
    }
};

/**
* Sets default language strings and help pages in English.  
* Should be overruled by a call to setLanguageStrings
*/

attributeCategory.prototype.setDefaultLanguageStrings = function() {
    var langArr = this.langArr = new Array();
	
    langArr['ACatProblemIn'] = 'Problem in';
    langArr['ACatPleaseSelect'] = 'Please Select';
    langArr['ACatConfirmPageExit'] = 'You have not yet saved your attributes. \nYour changes to the attributes will be lost.' ;
    langArr['ACatAttributesChanged'] = 'Attributes have been changed since last Save';
    langArr['ACatRequiredAttribute'] = 'Required attribute';
    langArr['ACatRequiredButNull'] = 'is not filled out';
    langArr['ACatCannotSave'] = 'Cannot Save attributes';
	langArr['ACatConfirmSave'] = 'One or more required attributes are not filled out. Save anyway?';
	langArr['ACatType'] = 'Type';
	langArr['ACatUnit'] = 'Unit';
	langArr['ACatName'] = 'Name';
	langArr['ACatValue'] = 'Value';
	langArr['ACatErrorLoadingACat'] = 'Error loading attribute category';
	langArr['ACatErrorReadingAttributes'] = ' Error reading attribute values' ;
	langArr['ACatDeleted'] = 'DELETED';
	langArr['ACatLoadingACat'] = 'Loading Attribute Category...';
	langArr['ACatReadingValues'] = 'Reading Values...';
	langArr['ACatReadingProjectAttributeValues'] = 'Reading Project Attribute Values...';
	// Added WCR 10.1
	langArr['ACatNotValid'] = '(NOT VALID)';
	// Added WCR 12.0
	langArr['ACatShowAdvancedControls'] = 'Show Rich Text Controls';
	langArr['ACatProjectAttributeOverwriteWarning'] = 'Existing Project Attribute value will be overwritten.';
	langArr['ACatLoadCurrentProjectAttributeValue'] = 'Click Here To Update Value From Project Attribute';
	langArr['ACatSearchLeftPanelOnly'] = 'Show Left Panel';
	langArr['ACatSearchImagePanelOnly'] = 'Show Top (Image) Panel';
	langArr['ACatSearchLeftAndImagePanel'] = 'Show Left and Top (Image) Panel';
	langArr['ACatSelectAll'] = 'Select All';
	langArr['ACatDeselectAll'] = 'Deselect All';
	langArr['ACatShowAll'] = 'Show All';
	langArr['ACatCurrentSelection'] = 'Current Selection:';
	langArr['ACatTreeProblem'] = 'Problem in attributeCategory.displayCascadingTrees at line ';
	langArr['ACatConstructorProblem'] = 'Problem in attributeCategory constructor at line number ';
	langArr['ACatLangStringsProblem'] = 'Problem in attributeCategory.setLanguageStrings  at line number ';
	langArr['ACatVersionProblem'] = 'Attribute category written by a more recent version of the software. Please check results';	
	langArr['ACatRowsColsProblem1'] = 'nRows or nCols of view ';
	langArr['ACatRowsColsProblem2'] = 'is 0 or null, skipping view';		
	langArr['ACatUnknownCellTypeProblem'] = 'unknown cell type: ';		
	langArr['ACatId'] = 'id: ';		
	langArr['ACatLayoutViewProblem'] = 'Layout does not contain a default view. A OneColumn default view is reconstructed from placed attributes';		
	langArr['ACatBrokenRuleProblem'] = 'Broken rule';		
	langArr['ACatListProblem1'] = 'List with name ';		
	langArr['ACatListProblem2'] = ' cannot be read. Only Excel lists are supported\n';		
	langArr['ACatListProblem3'] = ' cannot be read. No file name given\n';		
	langArr['ACatListProblem4'] = 'Cannot read list ';		
	langArr['ACatListProblem5'] = '\nerror:\n';			
	langArr['ACatJSONProblem1'] = 'cannot evaluate JSON content for list ';		
	langArr['ACatJSONProblem2'] = '\njsonContent: \n';		
	langArr['ACatAjaxProblem1'] = 'ajax call returned with error. \nError Message:';		
	langArr['ACatAjaxProblem2'] = '\ncalled URL: ';		
	langArr['ACatAjaxHandleProblem'] = 'Ajax call returned but could not retrieve the handle';		
	langArr['ACatServerXMLProblem1'] = 'The server does not return xml information. Server status: ';		
	langArr['ACatServerXMLProblem2'] = 'The server does not return xml information';		
	langArr['ACatServerXMLProblem3'] = 'XML of wrong type returned';		
	langArr['ACatServerXMLProblem4'] = 'category_XML contains no view(s)';		
	langArr['ACatAttributeProblem'] = 'no attribute values were returned';		
	langArr['ACatNotDefinedProblem'] = 'not defined';			
	langArr['ACatRenderAttrProblem'] = 'cannot render an attribute without ';		
	langArr['ACatAttrDateProblem'] = 'attribute value is not a Date:';		
	langArr['ACatColNotSelectedProblem'] = 'No column selected for list';		
	langArr['ACatOpNotSupportedProblem1'] = 'operator ';		
	langArr['ACatOpNotSupportedProblem2'] = ' not supported';		
	langArr['ACatDispHourProblem1'] = 'displayHourDropdown needs an id';	
	langArr['ACatDispHourProblem2'] = 'displayHourDropdown needs a name';	
	langArr['ACatListIndexProblem'] = 'not found. List index out of bounds? ';		
	langArr['ACatServerStatus'] = '\nServer status: ';		
	langArr['ACatSearchPanelFieldProblem'] = 'no search panel field exists for attribute ';		
	langArr['ACatAtLine'] = ' at line: ';	
	langArr['ACatMoreEntries'] = ' more ... ';	
	langArr['ACatDeselect'] = 'Deselect';
	langArr['ACatShowAllowed'] = 'Show Allowed';
	langArr['ACatSessionLostMessage'] = 'Your session is lost. You will not be able to save your work. Please log on again';
	langArr['ACatNoFittingRow'] = 'No fitting data found';
	langArr['ACatResetListValues'] = 'Reset All Related Values';
	langArr['ACatNothingValid'] = 'No valid entries found for already made selections';
	langArr['ACatValuesRequiringChanges'] = 'Values requiring changes to other selections:';
	langArr['ACatOnly'] = 'Only ' ;
	langArr['ACatValuesShownOf'] = ' values shown of ' ;
	langArr['ACatTypePartInInputField'] = 'Please type a part of the value in the input field';
	langArr['ACatHighlightRelatedValues'] = 'Highlight Related Values';
	langArr['ACatResetHighlight'] = 'Reset Highlight';
	langArr['ACatSearchMultiple'] = 'Search Multiple';
	langArr['ACatSearchSingle'] = 'Search Single';
	langArr['ACatUniqueValueCheckNoValue'] = 'Cannot check empty value.\nPlease, enter a value and try again.';
	langArr['ACatUniqueValueCheckNoType'] = 'Attribute type (project, document or task) is not known.';
	langArr['ACatUniqueValueCheckResultOK'] = 'The attribute value is unique.';
	langArr['ACatUniqueValueCheckResultOK_spec'] = 'The specification value is unique.';
	langArr['ACatUniqueValueCheckResult1'] = 'The value of ';
	langArr['ACatUniqueValueCheckResult2'] = ' attribute is not unique!';
	langArr['ACatUniqueValueCheckResult2_spec'] = ' specification is not unique!';
	langArr['ACatUniqueValueCheckOnSubmitResult'] = 'The following attributes have to be unique in the system:';
	langArr['ACatUniqueValueCheckOnSubmitResult_spec'] = 'The following specifications have to be unique in the system:';
	langArr['ACatNoInputValues'] = 'No input values given';
	langArr['ACatShowAdvancedDBFunctions'] = 'Advanced...';
	langArr['ACatShowUnfiltered'] = 'Show All (remove cascading filter)';
	langArr['ACatListProblemDriver64'] = '\nIf this problem occurs after upgrade to WebCenter 14, you may need to install the 64-bit driver on your web server. \nSee Knowlege base article 79695044 \nhttps://wiki.esko.com/pages/viewpage.action?pageId=79697090';
	langArr['ACatNotLoaded'] = 'The Attribute Category is not fully loaded yet.';
	
	sessionLostMsg = langArr['ACatSessionLostMessage'];

};

attributeCategory.prototype.addTerms = function() {
	try {
		var terms = new Array();

		terms.push({id:"PROJ_CREATION_PROJ_DETAILS",name:"Details",defaultValue:""});
		terms.push({id:"PROJ_CREATION_NAME",name:"Name",defaultValue:""});
		terms.push({id:"PROJ_CREATION_REQUESTOR",name:"Requestor",defaultValue:""});
		terms.push({id:"PROJ_CREATION_PROJMANAGER",name:"Project Manager",defaultValue:""});	
		terms.push({id:"PROJ_CREATION_STATUS",name:"Status",defaultValue:""});
		terms.push({id:"PROJ_CREATION_CUSTOMER",name:"Customer",defaultValue:""});
		terms.push({id:"PROJ_CREATION_LOCATION",name:"Location",defaultValue:""});
		terms.push({id:"PROJ_CREATION_DESCRIPTION",name:"Description",defaultValue:""});
		terms.push({id:"PROJ_CREATION_PROJECT_DUE_DATE",name:"Due Date",defaultValue:""});
		terms.push({id:"PROJDETAILSBOM_TAG",name:"Bill of Materials",defaultValue:""});
		terms.push({id:"PROJ_CREATION_DOCS",name:"Documents",defaultValue:""});
		terms.push({id:"PROJ_CREATION_DOCUMENT_TEMPLATE",name:"Document Template",defaultValue:""});
		terms.push({id:"PROJ_CREATION_MEMBERS",name:"Members",defaultValue:""});
		terms.push({id:"PROJECTROLE",name:"Role",defaultValue:""});  // Could be removed since no longer any need (sine we don't call it placeholder groups anymore) and it also does not work anymore (since the dialog is replaced)
		terms.push({id:"PROJ_CREATION_INVITE_MEMBERS",name:"Invite other members",defaultValue:""});
		terms.push({id:"PROJ_CREATION_ATTRIBUTES",name:"Attributes",defaultValue:""});
		terms.push({id:"PROJ_TASK_SPECS",name:"Specifications",defaultValue:"",TEP:true});
		terms.push({id:"PROJ_TASK_DISCUSSION",name:"Discussions",defaultValue:"",TEP:true});
		terms.push({id:"PROJ_TASK_DOCS",name:"Documents",defaultValue:"",TEP:true});
		terms.push({id:"PROJ_TASK_CHECKLIST",name:"Checklist",defaultValue:"",TEP:true});
	
		return terms;
	} catch (err) {
		return '';
	}
};
	
///////////////////////////////////////////////////////////////////////////////////////////////////////////
// I/O FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
* Calls a web service via Ajax to read the attribute category XML (if there is one). 
* @param readURL URL of the attribute category, typically the ... sdk call
* The Ajax call is cached in gACatCache. Multiple calls with the same readURL will only do 1 call.
*/
attributeCategory.prototype.readACatFromXML = function(readURL) {
    try {
        if (!readURL) return;        
		var aCatCacheObj;
        // show that you are busy
        var displayElement = document.getElementById(this.displayId);
        if (displayElement && this.currentView != 'Header' ) {
            var htmlcode = "<span class=\"darktext_darklink_bold_12\">" + this.langArr['ACatLoadingACat'] + "</span>";
            displayElement.innerHTML = htmlcode;
        }
		
		// check whether a call is already done for this attribute category
		var cacheIdx = 0;		
		for (cacheIdx=0; cacheIdx < gACatCache.length; cacheIdx++) {
			if (readURL == gACatCache[cacheIdx].readURL) break; 
		}			

		if (cacheIdx < gACatCache.length) {
			aCatCacheObj = gACatCache[cacheIdx];
			aCatCacheObj.aCatObjects.push(this);
			if (aCatCacheObj.status != 'Called') { // the ajax call is still underway, when it finishes, it will call readReturnXML for all attached copies
				this.readReturnXML(aCatCacheObj.returnXML,aCatCacheObj, true);				
			}
			
		} else {
			// make a new entry
			
			var aCatCacheObj = new Object();
			gACatCache.push(aCatCacheObj);
			aCatCacheObj.readURL = readURL;
			aCatCacheObj.aCatId = readURL;
			aCatCacheObj.aCatObjects = new Array();
			aCatCacheObj.aCatObjects.push(this);
			aCatCacheObj.returnXML = null;
			
			var aCatObj = this;  // closure
			
			// go fetching it        
			ajax({
				url: readURL,				
				type: "GET",
				data: "xml",
				debug: this.debug,
				synchronous: false,
				timeout: 120000,
				onSuccess: function(returnXML,objHdl2) {					
					aCatObj.readReturnXML(returnXML,aCatCacheObj,false);
				},
				
				//onSuccess: this.readReturnXML,
				onError: this.readError,				
				objHdl: this
			});
			aCatCacheObj.status = 'Called';
		}


    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " readACatFromXML: " + err);
    }
};

/**
* Gets values for the attributes, typically via a call to getAttributes.jsp
*/
attributeCategory.prototype.getAttributeValues = function(secondrun) {
    try {
		
        var url = null;
        if(secondrun==true){
        	url = this.valuesURLMerge;
        }else{
        	url = this.valuesURL;
        }

		if (!url) return;
		
        var displayElement = document.getElementById(this.displayId);
        if (displayElement && this.currentView == 'Default') {
            var htmlcode = "<span class=\"darktext_darklink_bold_12\">" + this.langArr['ACatReadingValues'] + "</span>";
            displayElement.innerHTML = htmlcode;
        }

        ajax({
            url: url,
            type: "GET",
            data: "xml",
            debug: this.debug,
            synchronous: false,
            onSuccess: this.readValuesFromXML,
            onError: this.readError,
            timeout: 120000,
            objHdl: this
        });


    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " getAttributeValues: " + err);
    }
};

/**
* Gets project attribute values for the specifications, typically via a call to getAttributes.jsp
*/
attributeCategory.prototype.getProjectAttributeValues = function() {
    try {		
		var url = this.projectAttributeValuesURL;
		if (!url) return;
		
        var displayElement = document.getElementById(this.displayId);
        if (displayElement && this.currentView == 'Default') {
            var htmlcode = "<span class=\"darktext_darklink_bold_12\">" + this.langArr['ACatReadingProjectAttributeValues'] + "</span>";
            displayElement.innerHTML = htmlcode;
        }

        ajax({
            url: url,
            type: "GET",
            data: "xml",
            debug: this.debug,
            synchronous: false,
            onSuccess: this.readProjectAttributeValuesFromXML,
            onError: this.readError,
            timeout: 120000,
            objHdl: this
        });


    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " getProjectAttributeValues: " + err);
    }
};

/**
* Reads the xml returned by the call to readURL (typically GetAttributeCategoryXML.jsp)
* and converts the xml information into javascript structures in the attribute category object
*/
attributeCategory.prototype.readReturnXML = function(returnxml,aCatCacheObj, fromCache) {
	
	if(!fromCache){
		// aCacheObj contains all categories for which it XML must be loaded, including current categoryObj
		// If the xml is ready, call this for all categories that need it
		if (aCatCacheObj) {
			aCatCacheObj.returnXML = returnxml;
			aCatCacheObj.status = 'XMLReady';
			if (aCatCacheObj.aCatObjects) {
				// call readReturnXML on all other copies
				for (var copy = 1; copy < aCatCacheObj.aCatObjects.length; copy++) {
					var attCatObj2 = aCatCacheObj.aCatObjects[copy];
					if (attCatObj2) attCatObj2.readReturnXML(aCatCacheObj.returnXML,aCatCacheObj,true);
				}
			}
		}
	}

    var attCatObj = null;
    try {
        //attCatObj = this.objHdl;  // retrieve the attribute category object
        attCatObj = this;  // TODO: if this works, just use "this" instead (refactoring)
		
		var showLayout = attCatObj.layoutId != null;	
		if (attCatObj.layoutId == "") showLayout = false;
		
        var attCatElem = document.getElementById(attCatObj.id);
		if (attCatElem == null || attCatElem.ACatObj != attCatObj) {
			//logger +="\n" + new Date() + ": mismatch between id and object of attribute category. Overwritten --> Ajax call is outdated. Only the most recent one will be filled";		
			return;
		}

        if (!attCatObj) throw(this.langArr['ACatAjaxHandleProblem']);

		var noDefaultViewException = false;  // flag raised if an xml is returned without a default view
		var xmlLoadFailed = false;  // flag raised if parsing/interpreting the XML throws an exception 
		
        var errorElem = WCXML.GetFirstFirstLevelChild(returnxml,"Error");
        if (errorElem) {
            var errorMsg = WCXML.GetElementText(errorElem,"ErrorMessage");
            alert(attCatObj.langArr['ACatErrorLoadingACat'] + ":\n" + errorMsg);
            attCatObj.errorMsg = errorMsg;
            
            attCatObj.displayAttributeCategory(true);
            return;
        } else {
            attCatObj.errorMsg = null; // otherwise nothing will be displayed
        }


        var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(returnxml,"attribute_category");
        if (!attributeCategoryElem) throw(this.langArr['ACatServerXMLProblem3']);

        // read name and category id
        attCatObj.name = WCXML.GetElementText(attributeCategoryElem,"name");
        
        // read attribute list if one is provided
		if (!attCatObj.importEaster) {  // if importing via easter Egg, don't read the attribute list since you already have the only good one
			var attListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"attribute_list");
			if (attListElem) attCatObj.readAttributeList(attListElem);
		}

        // read layout if one is provided
        var layoutXMLElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"category_XML");
        var attributeCategoryElem = null;
        if (layoutXMLElem) attributeCategoryElem=WCXML.GetFirstFirstLevelChild(layoutXMLElem,"attributeCategory");

        if (attributeCategoryElem) {  // read the actual layout
			try {
				attCatObj.version = WCXML.GetElementText(attributeCategoryElem,"version");
				if (attCatObj.version && attCatObj.version > attCatObj.softwareVersion) alert(this.langArr['ACatVersionProblem']);
				attCatObj.modificationTimeStamp = WCXML.GetElementText(attributeCategoryElem,"modificationTimeStamp");
				attCatObj.currentViewAtLastSave = WCXML.GetElementText(attributeCategoryElem,"currentViewAtLastSave");
								// set imported flag if needed
		        var importedElem = WCXML.GetElementText(attributeCategoryElem,"migrated");
		        if (importedElem && importedElem == "1"){
		        	attCatObj.imported = true;
		        }
				
				// read terminology if there is any
				if (attCatObj.requestedView == 'Creation' || attCatObj.layoutId || attCatObj.isTaskTypePage) {  
					// DT121899: edit terminology is not supposed to do anything for header or default view or document's task type view or BOM view. 
					// Remark that currentView can be 'Default' also in project creation if there is no creation view, therefore the check is on requestedView					
					// Also remark that you need to read terminology in the admin edit page, otherwise it's lost at next save
					// Through EasterEgg, also taskTypePage can have terminology.
					var terminologyElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"terminology");
					if (terminologyElem) {
						attCatObj.readTerminologyFromXML(terminologyElem);
						attCatObj.showEditTerminology = true;
					}
				}
				
				
				//read styles if there are any
				var styleListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"styles");
				if (styleListElem) attCatObj.readStylesFromXML(styleListElem);

				//read rules if there are any
				var ruleListElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"rules");
				if (ruleListElem) attCatObj.readRulesFromXML(ruleListElem);

				// read views
				//////////////////////////////////////
				var viewArr=WCXML.GetFirstLevelChilds(attributeCategoryElem,"view");
				if (!viewArr || viewArr.length < 1) throw(this.langArr['ACatServerXMLProblem4']);

				attCatObj.views = new Array();
				

				for (var view = 0 ; view < viewArr.length; view++) {
					
					var viewElem = viewArr[view];
					if (!viewElem) continue; // very unlikely to happen

					viewObj = new Object();
					viewObj.layout = WCXML.GetElementText(viewElem,"layout");
					viewObj.name = WCXML.GetElementText(viewElem,"name");
					var viewIdx = 0;
					for (viewIdx=0; viewIdx<attCatObj.cViewOptions.length; viewIdx++) {
						if (viewObj.name == attCatObj.cViewOptions[viewIdx]) break;
					}
					if (viewIdx >= attCatObj.cViewOptions.length) attCatObj.cViewOptions.push(viewObj.name);
					// viewIdx is now the index in cViewOptions of the current view
					
					viewObj.showTypeAsUnit = WCXML.GetElementBoolean(viewElem,"showTypeAsUnit");
					if (viewObj.showTypeAsUnit.constructor != Boolean) {  // xml from alpha period
						if (viewObj.name == 'Header') viewObj.showTypeAsUnit=false; else viewObj.showTypeAsUnit=true;
					}
					viewObj.addToGeneralSection = WCXML.GetElementBoolean(viewElem,"addToGeneralSection");	// Experimental feature	
					viewObj.keepSessionAlive = WCXML.GetElementBoolean(viewElem,"keepSessionAlive");
				
					viewObj.alternateBackground = WCXML.GetElementBoolean(viewElem,"alternateBackground");
					if (viewObj.alternateBackground.constructor != Boolean) {  // xml from alpha period
						if (viewObj.name == 'Header')  viewObj.alternateBackground=false; else viewObj.alternateBackground=true;
					}
					
					viewObj.nRows = parseInt(WCXML.GetElementText(viewElem,"nRows"));
					viewObj.nCols = parseInt(WCXML.GetElementText(viewElem,"nCols"));
					if (!viewObj.nRows || !viewObj.nCols) {

						alert(this.langArr['ACatRowsColsProblem1'] + view + this.langArr['ACatRowsColsProblem2']);
						continue;
					}
					viewObj.cells = new Array(viewObj.nRows);
					for (var row=0; row < viewObj.nRows;row++) viewObj.cells[row] = new Array(viewObj.nCols);
					cellArr = WCXML.GetFirstLevelChilds(viewElem,"cell");
					if (cellArr && cellArr.length>0) {
						for (var cell = 0; cell < cellArr.length; cell++) {
							var cellElem = cellArr[cell];
							if (!cellElem) continue;
							var row =  WCXML.GetElementText(cellElem,"row");
							var col =  WCXML.GetElementText(cellElem,"col");
							if (row > viewObj.nRows) viewObj.nRows = row +1;  // safety, should not happen
							if (col > viewObj.nCols) viewObj.nCols = col +1;  // safety, should not happen
							if (row!="" && col!="") {
								var cellObj = new Object();
								cellObj.type = WCXML.GetElementText(cellElem,"type");
								cellObj.label = WCXML.GetElementText(cellElem,"label", null, true); // extra parameters to distinguish null and ''
								cellObj.labelImage = WCXML.GetElementText(cellElem,"labelImage");
								cellObj.labelPosition = WCXML.GetElementText(cellElem,"labelPosition");
								if (!cellObj.labelPosition) cellObj.labelPosition = 'Left';
								
								// AttValueImages
								cellObj.renderOption = WCXML.GetElementText(cellElem,"renderOption");
								cellObj.nImageColumns =  WCXML.GetElementText(cellElem,"nImageColumns");
								cellObj.imageAlignment =  WCXML.GetElementText(cellElem,"imageAlignment");
								if (cellObj.renderOption == 'Images' && !cellObj.imageAlignment) cellObj.imageAlignment = 'DistributeOverWidth'; 
								cellObj.restrictedValueLabelPosition =  WCXML.GetElementText(cellElem,"restrictedValueLabelPosition");
								
								
								cellObj.renderHeight =  WCXML.GetElementText(cellElem,"renderHeight");
								cellObj.imageHeight =  WCXML.GetElementText(cellElem,"imageHeight");
								cellObj.renderImageType =  WCXML.GetElementText(cellElem,"renderImageType");
								if (viewObj.name == 'Default'){
									cellObj.allowMultipleValues = WCXML.GetElementBoolean(cellElem,"allowMultipleValues");
									cellObj.fractionalIfImperial = WCXML.GetElementBoolean(cellElem,"fractionalIfImperial");
								}
								var styleName = WCXML.GetElementText(cellElem,"style");
								if (styleName) {
									try {
										// find the styleObj
										for (var style=0; style<attCatObj.styles.length; style++) {
											if (attCatObj.styles[style].name == styleName) {
												cellObj.styleObj = attCatObj.styles[style];
												break;
											}
										}
									} catch(err) {
										// do nothing -->  will get standard style
									}
								}
								if (!cellObj.styleObj && viewObj.name != 'Header') { // !styleName or not found in the list --> assign standard style
									if (cellObj.type == "Attribute") {
										cellObj.styleObj = attCatObj.styles[0]; 
									} else if (cellObj.type == "Title") {
										cellObj.styleObj=attCatObj.styles[1];
									}  // ruler, headerbar and spacer don't need a style object
								}


								var ruleName = WCXML.GetElementText(cellElem,"rule");
								if (ruleName) {
									// find the ruleObj
									for (var rule=0; rule<attCatObj.rules.length; rule++) {
										if (attCatObj.rules[rule].name == ruleName) {
											cellObj.ruleObj = attCatObj.rules[rule];
											break;
										}
									}
								}
								if (!cellObj.ruleObj) cellObj.ruleObj = attCatObj.rules[0]; // Editable

								cellObj.colSpan = WCXML.GetElementText(cellElem,"colSpan");
								cellObj.rowSize = WCXML.GetElementText(cellElem,"rowSize");
								
								cellObj.infoPage = WCXML.GetElementText(cellElem,"infoPage");
								if (cellObj.infoPage) attCatObj.showInfoPage = true;
								
								switch (cellObj.type) {
									case 'Attribute':
									
										cellObj.id = WCXML.GetElementText(cellElem,"id");
										cellObj.name = WCXML.GetElementText(cellElem,"name");

										// go searching for the attribute in the list
										// first search on name (for export/import reasons)
										var attObj = null; 
										if (attCatObj.imported) {
											attObj = attCatObj.findAttribute(null, cellObj.name);
											// make sure id is stored correctly so it loads correctly once the imported flag is removed
											if(attObj) cellObj.id = attObj.id;
											else cellObj.id = '';
										} else {
											attObj = attCatObj.findAttribute(cellObj.id);
											if (!attObj && cellObj.name) {
												attObj = attCatObj.findAttribute(null, cellObj.name);
												if (attObj) {
													cellObj.id = attObj.id;
												} else {
													cellObj.id = '';
												}
											}
										}
										
										// if label was never filled out, fill it out now 
										if(attCatObj.version < 2){
											cellObj.label = WCXML.GetElementText(cellElem,"name");
											// if name was empty, make the label empty string, this was done on purpose to show nothing
											if(!cellObj.label) cellObj.label = '';
										}
										
										cellObj.attObj = attObj;
										if (!attObj) {											
											cellObj = null;  // skip references to attributes which are taken out of the system
										} else {
											cellObj.name = attObj.name; // if there is an attribute, always make sure the name of the cell is updated
											if (!attObj.placedArr) attObj.placedArr = new Array(attCatObj.cViewOptions.length);
											if (attObj.placedArr[viewIdx]) {
												attObj.placedArr[viewIdx]++;
											} else {
												attObj.placedArr[viewIdx] = 1;
											}
											attObj.placedOriginally = true;
											if (attObj.placed) attObj.placed++; else attObj.placed=1;
											if (viewObj.name == 'Default') {
												attObj.allowMultipleValues = cellObj.allowMultipleValues;
												attObj.fractionalIfImperial = cellObj.fractionalIfImperial;
											}
											
											// pick up restrictions on this attribute (other than normal restricted sets and lists)
											if (view == 0) {
												var restrictType = WCXML.GetElementText(cellElem,"restrictType");
												if (restrictType == 'Attribute') { 
													var restrictAttributeId =  WCXML.GetElementText(cellElem,"restrictAttributeId");
													var restrictAttributeName =  WCXML.GetElementText(cellElem,"restrictAttributeName");
													attObj.restrictToAttObj = attCatObj.findAttribute(restrictAttributeId,restrictAttributeName);
													if (attObj.restrictToAttObj) attObj.restrictToAttObj.isInRule = true;
													
												}
												attObj.restrictType = restrictType;
												if (!attObj.restrictType) attObj.restrictType = 'None';  // default restrictType
											}
											if (cellObj.restrictType != 'None' &&  cellObj.renderOption != 'Dropdown' && !cellObj.restrictedValueLabelPosition) cellObj.restrictedValueLabelPosition = 'LabelBelow'; 
											
											// read document link sources, if any exist
											cellObj.docLinkSources = this.readDocLinkSources(cellElem, row, col);

											if (cellObj.docLinkSources && this.currentView != 'Header' && this.currentUser.canEditAttributes) this.loadDocLinkSources(cellObj);											
											
											if (cellObj.docLinkSources) cellObj.docLinkTableOptions = this.readDocLinkTableOptions(cellElem);
											
										}
										


										break;
									
									case 'Property':
										cellObj.name = WCXML.GetElementText(cellElem,"name");
										this.easterEgg = true; // switch on extended functionality as soon as you get properties in the input

										// go searching for it in the extended list										
										var attObj = null; 										
																				
										for (var prop=0; prop<this.otherProperties.length; prop++) {
											if (cellObj.name == this.otherProperties[prop].name) break;
										}
										if (prop <this.otherProperties.length) {
											attObj = cellObj.attObj = this.otherProperties[prop];
										
											if (!attObj.placedArr) attObj.placedArr = new Array(attCatObj.cViewOptions.length);
											if (attObj.placedArr[viewIdx]) {
												attObj.placedArr[viewIdx]++;
											} else {
												attObj.placedArr[viewIdx] = 1;
											}
											attObj.placedOriginally = true;
											if (attObj.placed) attObj.placed++; else attObj.placed=1;
										} else {
											cellObj = null;  // throw out what no longer exists - shoudl probably not happen
										}
											

										break;
									case 'Ruler':
										cellObj.rulerThickness = WCXML.GetElementText(cellElem,"rulerThickness");
										cellObj.rulerColor = WCXML.GetElementText(cellElem,"rulerColor");									
										break;
									case 'Spacer':
										cellObj.height = WCXML.GetElementText(cellElem,"height");
										cellObj.background = WCXML.GetElementText(cellElem,"background");	
										break;
									case 'Title':															
									case 'HeaderBar':
									case 'Button':  
										break;	
									
										
									default:
										alert(this.langArr['ACatUnknownCellTypeProblem'] + cellObj.type + this.langArr['ACatId'] + cellObj.id);
										continue;
										break;

								}  // end switch cellObj.type
								
								this.readButtonsFromXML(cellElem,cellObj, viewObj);  // read buttons (can be part of many cell types)
								
								// from here on, cellObj can be null --> in the case an attribute has been taken out of the system

								viewObj.cells[row][col]=cellObj;

							}
						} // end for cells
					} // end if cellArr
					// check if this is the current view
					
					if (viewObj.name && attCatObj.currentView && viewObj.name.toLowerCase() == attCatObj.currentView.toLowerCase()) {
						attCatObj.cells = viewObj.cells;
						attCatObj.layout = viewObj.layout;
						attCatObj.nRows = viewObj.nRows;
						attCatObj.nCols = viewObj.nCols;
						attCatObj.showTypeAsUnit = viewObj.showTypeAsUnit;
						attCatObj.alternateBackground = viewObj.alternateBackground;
						attCatObj.addToGeneralSection = viewObj.addToGeneralSection;
						attCatObj.keepSessionAlive = viewObj.keepSessionAlive;
						attCatObj.currentViewFound = true;
					}
					
					attCatObj.views[viewIdx] = viewObj;
					
                
				} // end for view
				// check if at least one view fits this.currentView
				if (!attCatObj.currentViewFound) {
					if (attCatObj.currentView ==  'Header') {
						attCatObj.noHeader = true;  // remember that there is no header view at all when asked for display in normal projects or documents
					} else if (attCatObj.currentView ==  'Creation' || attCatObj.currentView ==  'Default' || attCatObj.currentView ==  'BOM') {   // Default will normally never happen, but it is surely possible to ask a Creation view if there is none
						if (attCatObj.views[0]) {
							attCatObj.cells = attCatObj.views[0].cells;
							attCatObj.layout = attCatObj.views[0].layout;
							attCatObj.nRows = attCatObj.views[0].nRows;
							attCatObj.nCols = attCatObj.views[0].nCols;
							attCatObj.showTypeAsUnit = attCatObj.views[0].showTypeAsUnit;
							attCatObj.alternateBackground = attCatObj.views[0].alternateBackground;
							attCatObj.currentView = 'Default';   // This is where currentView becomed different from requestedView.
						} else {
							noDefaultViewException = true;							
						}
					} else {
						// a task type view is asked and there is none --> display nothing and don't ask to read the cascading dropdowns
						attCatObj.noCurrentViewDisplay = true;
						
					}
				}
				if (viewArr.length <2) attCatObj.views=null;  // no views used
				
				// read lists if there are any
				if (attCatObj.currentView !=  'Header' && !attCatObj.noCurrentViewDisplay) {
					var listsElem = WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"lists");
					if (listsElem) attCatObj.readListsFromXML(listsElem,attCatObj.readListsURL,aCatCacheObj,fromCache);
				}
				
			} catch (err) {
				// problem reading the XML --> fall back on OneColumn while ignoring the XML
//				alert("error reading XML on line " + err.lineNumber + ": " +  err.message );
				xmlLoadFailed = true;
			}

        } 
		this.initManageTEP();
		if (!attributeCategoryElem || xmlLoadFailed || noDefaultViewException)  { 
			// no layout exists yet or it failed loading or no default view --> OneColumn layout for the default view
            attCatObj.layout = 'OneColumn';
            attCatObj.nCols = 1;
			if (!noDefaultViewException) attCatObj.noHeader = true; // if there is no layoutXML, there cannot be views and also not a header view
            if (attCatObj.attributes) {
                attCatObj.nRows = attCatObj.attributes.length;
                if (attCatObj.nRows > 0) {
                    attCatObj.cells = new Array(attCatObj.nRows);
                } else {
                    attCatObj.cells = new Array();
                }
                for (var row=0;row<attCatObj.nRows;row++) attCatObj.cells[row] = new Array(attCatObj.nCols);
                for (var row=0;row<attCatObj.attributes.length;row++) {
                    var cellObj = new Object();
                    attCatObj.cells[row][0] = cellObj;
                    cellObj.attObj = attCatObj.attributes[row];
                    cellObj.type = 'Attribute';
                    // TO DO: decide whether you want to have id and name doubled in the data structure.
                    cellObj.id = cellObj.attObj.id;
                    cellObj.name = cellObj.attObj.name;
                    if (cellObj.attObj.placed) cellObj.attObj.placed++; else cellObj.attObj.placed=1;
                    cellObj.attObj.placedArr[0] = 1;
                    // set required flag here if required task specification
                    if(attCatObj.isTaskTypePage && cellObj.attObj.isRequired) {
                    	cellObj.ruleObj = attCatObj.isRequiredRule;
                	}
                }
            } else {
                attCatObj.attributes = new Array();
                attCatObj.nRows = 0;
            }
			if (noDefaultViewException) {

				alert(this.langArr['ACatLayoutViewProblem'] ); // should not happen
				if (attCatObj.views) {  // put the default view also in views[0];
					attCatObj.views[0].cells = attCatObj.cells;
					attCatObj.views[0].layout = attCatObj.layout;
					attCatObj.views[0].nRows = attCatObj.nRows;
					attCatObj.views[0].nCols = attCatObj.nCols;
					// showTypeAsUnit and alternateBackground don't have to be copied for OneColumn layout
				}
			}
        }
		
		// load values if necessary	
		var loadvalues = false; 
        if (attCatObj.valuesURL && !attCatObj.noCurrentViewDisplay && !(attCatObj.currentView ==  'Header' && attCatObj.noHeader)) {
        	attCatObj.getAttributeValues();		
        	loadvalues = true;
		}

        // current view

        if (attCatObj.views) {
            // set current view index
            for (var view = 0; view< attCatObj.views.length; view++) {
                if (attCatObj.views[view] && attCatObj.views[view].name == attCatObj.currentView) attCatObj.currentViewIdx = view;
            }
        }
		
        if (attCatObj) {
            //var showLayout = ((attCatObj.layoutId != null) && (attCatObj.layoutId != ""));  // this line seems to be wrong but I don't see why

            
            

			if (attCatObj.addToGeneralSection) {   // experimental feature
				var generalSectionId = attCatObj.displayId + '2';
				if (document.getElementById(generalSectionId)) {
					attCatObj.displayId=generalSectionId;
					var attributesSection = document.getElementById('mainAttributesRowId');
					if (attributesSection) attributesSection.style.display='none';					
				}
			}
            if (showLayout) {
                attCatObj.startLayout(attCatObj.layoutId);
            } else {
            	if (!loadvalues) this.loaded = true;
                attCatObj.displayAttributeCategory(true);
                if (attCatObj.callbackOnload) attCatObj.callbackOnload();
				if (attCatObj.keepSessionAlive &&  !showLayout  && (this.currentUser && this.currentUser.canEditAttributes == true)) window.setInterval("keepSessionAlive(alertSessionLost)", 1200000);  // to avoid that you lose your session after a number of minutes while editing				
					
            }
			attCatObj.loadingDone = true;  // indicate that the category is loaded
        }
    } catch (err) {
        var errormsg = "XML data could not be interpreted. Error message: " + err;
        var displayElement = document.getElementById(attCatObj.displayId);
        if (displayElement) {
            displayElement.innerHTML = errormsg;
        } else {
            alert(errormsg);
        }
        if (attCatObj) attCatObj.errorStatus = errormsg;
        if (attCatObj.callbackOnFail){
        	try{
        		attCatObj.callbackOnFail();
        		if (aCatCacheObj) {
        			if (aCatCacheObj &&  aCatCacheObj.aCatObjects && !fromCache) {
        				// call readReturnXML on all other copies
        				for (var copy = 1; copy < aCatCacheObj.aCatObjects.length; copy++) {
        					var attCatObj2 = aCatCacheObj.aCatObjects[copy];
        					if (attCatObj2) {
        						var displayElement2 = document.getElementById(attCatObj2.displayId);
        				        if (displayElement2) {
        				            displayElement2.innerHTML = errormsg;
        				        } else {
        				            alert(errormsg);
        				        }
        						attCatObj2.errorStatus = errormsg;
        						if(attCatObj2.callbackOnFail){
            						attCatObj2.callbackOnFail();        							
        						}
        					}
        				}
        				
        			}
        		}
        	}catch(err2){}
        }
    } finally {
		this.importEaster = false;
	}
};

/**
* Constructor for buttonObj. All new creations of buttonObj should pass through this constructor
* @param {String} [func] - function to be called when clicking the button. Allowed values are defined in aCatObj.buttonFuncOptions: LookUpAttributes, JSP, JavaScript, CheckUniqueValue, Print
* @param {String} [label] - label to be shown on the button (or the text for the green link). Defaults to ''
* @param {String] [style] - style of the button: allowed are buttonprimary, buttonsecondary, plain and greenlink
* @param {String} [javaScript] - javascript code which will be executed if func == 'JavaScript'
* @param {String} [viewName] - view in which the button will be added. one of 'Default', 'Creation', ...
*/
attributeCategory.prototype.buttonObj = function (func, label, style,javaScript, viewName) {
	var buttonObj = {};	// CR: is this needed?
	this.func = func;	  // since buttonObj is called with new buttonObj, this refers to the just created object
	if (!this.func) this.func = 'LookUpAttributes';
	this.label = label;
	if (!this.label) this.label = '';
	this.style = style;
	if (!this.style) this.style = 'buttonprimary';
	this.inputAtts = [];
	this.outputAtts = [];
	this.changeAtts = [];
	this.javaScript = javaScript;
	this.listObj = null;
	this.execOnLoad = false;  // execute the button as soon as all data is loaded
	this.execOnSave = false;	// execute the button at validation time right before saving
	this.execOnChange = false;   // execute the button when another attribute changes
	this.clicked = 0;  // count how many times the button is clicked
	this.viewName =  viewName ? viewName : 'Default';  // view in which this button is placed
	
	
}



/**
* reads button xml and constructs cellObj.buttons
* xml looks like 
* <buttons>
* 	<button>
* 		<label>Click</label>
*		<func>LookUpAttributes</func>
*		<inputAtt>
*			<id>00002_0000001078</id>
*			<name>Brand</name>
*			<columnName>CountryCode</columnName>
*			<exact>true</exact>
*		</inputAtt>
*		<outputAtt>
*			<id>00002_0000001078</id>
*			<name>Brand</name>
*			<columnName>CountryCode</columnName>
*		</outputAtt>
*		<style>buttonprimary</style>
*		<listName>Countries and Managers</listName>
*	</button>
*</buttons>
*/
attributeCategory.prototype.readButtonsFromXML = function(cellElem,cellObj, viewObj) {
    "use strict";
	try {
		if (!cellElem) return;
		if (this.currentView == 'Header') return;   // never have buttons in Header view, so don't bother reading them, let alone fetch necessary information.
		var buttonsElem = WCXML.GetFirstFirstLevelChild(cellElem,"buttons");
		if (buttonsElem) {
			var buttonArr = WCXML.GetFirstLevelChilds(buttonsElem,"button");
			
			if (buttonArr && buttonArr.length>0) {
				cellObj.buttons = [];
				for (var button=0; button < buttonArr.length; button++) {
					var buttonElem = buttonArr[button];
					var buttonObj = new this.buttonObj();
					cellObj.buttons.push(buttonObj);	
					
					buttonObj.label = WCXML.GetElementText(buttonElem,'label');
					buttonObj.func = WCXML.GetElementText(buttonElem,'func');
					buttonObj.style = WCXML.GetElementText(buttonElem,'style');
					buttonObj.execOnLoad = WCXML.GetElementBoolean(buttonElem,'execOnLoad');
					buttonObj.execOnSave = WCXML.GetElementBoolean(buttonElem,'execOnSave');
					buttonObj.execOnChange = WCXML.GetElementBoolean(buttonElem,'execOnChange');
					buttonObj.viewName = viewObj? viewObj.name: 'Default';					
					
					if (buttonObj.execOnChange) {
						this.readReferencedAttributeList(buttonObj, buttonElem,'changeAtts','changeAtt');						
					}
					
					if (buttonObj.func == 'JavaScript') {
						buttonObj.javaScript = WCXML.GetElementText(buttonElem,'javaScript');						
					}
					
					if (buttonObj.func == 'LookUpAttributes') {
						// retrieve listName. listObj cannot be found yet because lists are not read at this moment
						buttonObj.listName = WCXML.GetElementText(buttonElem,'listName');	
						buttonObj.listCounter = WCXML.GetElementText(buttonElem,'listCounter');	
						
						// input attributes
						this.readReferencedAttributeList(buttonObj, buttonElem,'inputAtts','inputAtt');
						
						// output attributes		
						this.readReferencedAttributeList(buttonObj, buttonElem,'outputAtts','outputAtt');						
						
					}
					
							
				}
			}
			
			
		}
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " readStylesFromXML: " + err);
    }

}

/**
* Reads inputAtts, outputAtts and changeAtts from XML for a button represented by buttonObj
* @param {Obj} buttonObj - buttonObj for which to read the references attributes
* @param {XMLDOMElem} buttonElem - element in the XML representing the button
* @param {String} type - one of 'changeAtts', 'inputAtts', 'outputAtts'
* @param {String} tagName - the XML tag containing the attributes ('changeAtt' etc).
*/
attributeCategory.prototype.readReferencedAttributeList = function(buttonObj, buttonElem, type, tagName) {
	'use strict';
	try {
		var attsArr = [];
		if (!buttonObj) throw new Error('buttonObj does not exist'); 
		buttonObj[type] = attsArr;  // means f.e. buttonObj.changeAtts = [];

		var attArr =  WCXML.GetFirstLevelChilds(buttonElem,tagName);
		if (attArr && attArr.length>0) {
			for (var att=0; att < attArr.length; att++) {
				var attElem = attArr[att];
				// find attObj
				var id = WCXML.GetElementText(attElem,'id');
				var name = WCXML.GetElementText(attElem,'name');
				var attObj;
				if (!this.imported) {
					attObj = this.findAttribute(id,name);  // find by id
				} else {
					attObj = this.findAttribute(null,name);  // find by name when imported from a foreign system
				}
				if (attObj) {
				
					var attObj2 = {};
					attsArr.push(attObj2);
					attObj2.attObj = attObj;    
					
					if (type != 'changeAtts') {
						attObj2.columnName =  WCXML.GetElementText(attElem,'columnName');
						// columnObj cannot be retrieved yet because lists are not yet read at this point
					}
					if (type == 'inputAtts') {
						attObj2.exact = WCXML.GetElementBoolean(attElem,'exact');
					}
					if (type == 'changeAtts') {
						if (!attObj.buttonDependency) attObj.buttonDependency = [];
						attObj.buttonDependency.push(buttonObj);
					}
					
				} else {
					//alert('Attribute ' + name + ' is referenced by button ' + buttonObj.label + ' but is no longer in the category' );
				}
			}
		}
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " readReferencedAttributeList: " + err);
    }
}

attributeCategory.prototype.readStylesFromXML = function(stylesElem) {
    try {
        if (!stylesElem) return;
        if (!this.styles) this.styles = new Array();
        var styleArr = WCXML.GetFirstLevelChilds(stylesElem,"style");
        for (var style=0; style<styleArr.length; style++) {
            var styleElem = styleArr[style];
            var styleObj = new Object();
            styleObj.standardStyle = WCXML.GetElementBoolean(styleElem,"standardStyle");
            styleObj.name = WCXML.GetElementText(styleElem,"name");
            if (1) {
                // look whether you already have it, typical for standard styles
                var found = false;
                for (var style2=0; style2<this.styles.length; style2++) {
                    if (styleObj.name == this.styles[style2].name) {
                        found=true; break;
                    };
                }
                if (found) {
                    continue;  // don't add an existing (standard) style
                } else {
                    // declared as standard style but no longer a standard style --> degrade
                    styleObj.standardStyle = false;
                }
            }
            this.styles.push(styleObj);  // if the style is not known yet, add to the list

            styleObj.classLabel = WCXML.GetElementText(styleElem,"classLabel");
            styleObj.classValue = WCXML.GetElementText(styleElem,"classValue");
            styleObj.background = WCXML.GetElementText(styleElem,"background");
			styleObj.alignment = WCXML.GetElementText(styleElem,"alignment");
            styleObj.fontColor = WCXML.GetElementText(styleElem,"fontColor");
            styleObj.fontStyle = WCXML.GetElementText(styleElem,"fontStyle");
            styleObj.fontWeight = WCXML.GetElementText(styleElem,"fontWeight");
            styleObj.fontFamily = WCXML.GetElementText(styleElem,"fontFamily");
            styleObj.fontSize = WCXML.GetElementText(styleElem,"fontSize");
            styleObj.width = WCXML.GetElementText(styleElem,"width");
			styleObj.widthunit = WCXML.GetElementText(styleElem,"widthunit");
            styleObj.height = WCXML.GetElementText(styleElem,"height");
			styleObj.columnWidth = WCXML.GetElementText(styleElem,"columnWidth");
			styleObj.columnWidthUnit = WCXML.GetElementText(styleElem,"columnWidthUnit");
			
			
			
			
			var valueDifferent = WCXML.GetElementBoolean(styleElem,"valueDifferent");
			if (valueDifferent) {
				styleObj.isDifferentStyles = true;
				var valueStyleObj = new Object();
				styleObj.valueStyleObj = valueStyleObj;
				valueStyleObj.background = WCXML.GetElementText(styleElem,"valueBackground");
				valueStyleObj.alignment = WCXML.GetElementText(styleElem,"valueAlignment");
				valueStyleObj.fontColor = WCXML.GetElementText(styleElem,"valueFontColor");
				valueStyleObj.fontStyle = WCXML.GetElementText(styleElem,"valueFontStyle");
				valueStyleObj.fontWeight = WCXML.GetElementText(styleElem,"valueFontWeight");
				valueStyleObj.fontFamily = WCXML.GetElementText(styleElem,"valueFontFamily");
				valueStyleObj.fontSize = WCXML.GetElementText(styleElem,"valueFontSize");
				valueStyleObj.width = WCXML.GetElementText(styleElem,"valueWidth");
				valueStyleObj.widthunit = WCXML.GetElementText(styleElem,"valueWidthunit");
				valueStyleObj.height = WCXML.GetElementText(styleElem,"valueHeight");
				valueStyleObj.columnWidth = WCXML.GetElementText(styleElem,"valueColumnWidth");
				valueStyleObj.columnWidthUnit = WCXML.GetElementText(styleElem,"valueColumnWidthUnit");
				
			} else {
				styleObj.isDifferentStyles = false;
			}

        }
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " readStylesFromXML: " + err);
    }
};

/**
* Reads terminology from xml
* The terminology looks as follows
* <terminology>
* 	<term>
*		<id>PROJ_CREATION_PROJECT_DUE_DATE</id>
*		<default>Samples to be delivered by</default>
*		<fr>echantillons a delivrer vers</fr>
*	</term>
*   ... more terms
* </terminology>
 The code goes looking on the page to fix-up for elements with the <id> provided and replaces it with what is found in <default> or in the language provided 
* Language is however on other places not yet supported
*/
attributeCategory.prototype.readTerminologyFromXML = function(terminologyElem) {
    try {
        if (!terminologyElem) return;

        var terminologyArr = WCXML.GetFirstLevelChilds(terminologyElem,"term");

		if (!terminologyArr) return;
        for (var term=0; term<terminologyArr.length; term++) {
            var termElem = terminologyArr[term]; 			
            var id = WCXML.GetElementText(termElem,"id");
			if (id ) {
				for (var term2=0; term2<this.terms.length; term2++) {
					if (this.terms[term2] && this.terms[term2].id == id) {
						var termObj = this.terms[term2];
						var elem = WCXML.GetFirstFirstLevelChild(termElem,"default");
						if (elem) {
							var value = WCXML.GetElementText(termElem,"default");
							
							if (this.language) {  // only overrule if there is actually a language string defined for this langauge
								var elem= WCXML.GetFirstFirstLevelChild(termElem,this.language);
								if (elem) value = WCXML.GetElementText(termElem,this.language);					
							}
							
							termObj.defaultValue = value;
							
							// do the actual change of the term in the project creation page
							value = WCXML.xmlencode(value);
							if (document.getElementById(id)) {
								// try rescuing the * but only if it's not yet in the new name (otherwise you will get 2 *s. Done for historical reasons: setups exist with the * in there already)
								// only add a * when there was a *-span in the old value and it was not hidden (meaning it was required)
								var oldvalue = document.getElementById(id).innerHTML;
								if (oldvalue && oldvalue.toLowerCase().indexOf('<span')!=-1 && oldvalue.toLowerCase().indexOf('display: none;') == -1 
										&& value.indexOf('*') == -1) 
									value += '&nbsp;<span class="esko-green" id="projectNameRowReqId" style="display: inline;">*</span>';
								document.getElementById(id).innerHTML = value;							
							}
						}
					}
				}
				
			}
           
        }
		// now look whether to switch sequence of the actual boxes
		var sequenceString = WCXML.GetElementText(terminologyElem,"sequence");
		this.projectCreationSequence = sequenceString;
		
		if (sequenceString && sequenceString.length >0) {		
			var sectionNames = {G:'mainProjectDetailsRowId',B:'mainBomDetailsRowId',D:'mainDocumentDetailsRowId', M: 'mainMembersRowId', A: 'mainAttributesRowId'};
			var item;
			
		
			// Code changed for DT115720.  Moving the G section around causes its calendars (due date, last task date, first task date) to break as a consequence of jQuery miser
			
			// find where G is in the sequenceString
			for (var itemG=0; itemG < sequenceString.length; itemG++) {
				if (sequenceString.substr(itemG,1) == 'G') break;
			}
			// until item G, put it before item G.  If itemG is 0, this does nothing
			for (var item=0; item<itemG; item++) {
				$("#mainProjectDetailsRowId").before($("#"+sectionNames[sequenceString.substr(item,1)]));
			}
			// after item G, put it after the last item
			for (var item = itemG+1; item < sequenceString.length; item++) {
				$("#"+sectionNames[sequenceString.substr(item-1,1)]).after($("#"+sectionNames[sequenceString.substr(item,1)]));
			}
			// CHECKLIST
			// Protect against sections which don't exist --> OK: sections are always placed but hidden as needed. So you can still shuffle them around
			// Protect against wrong strings --> The section name will not be found and jQuery will simply not do the statement. It will NOT go to the catch statement
			// Protect against G not appearing in the sequenceString: itemG will be equal to sequenceString.length. The first for statement will do all shuffling (so all will appear before G). 
			
		}
		
		// width of the project creation sections
		var sectionWidth = WCXML.GetElementText(terminologyElem,"width");
		if (sectionWidth) sectionWidth = parseInt(sectionWidth);
		this.projectCreationWidth = sectionWidth;
		if (sectionWidth) {
			$("#mainTableId").width(sectionWidth);
		}
		//$('#mainAttributesDetailsRowId').html($('#mainAttributesDetailsTd').html());
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " readTerminologyFromXML: " + err);
    }
};

attributeCategory.prototype.readRulesFromXML = function(rulesElem) {
    try {
        if (!rulesElem) return;
        if (!this.rules) this.rules = new Array();
        var ruleArr = WCXML.GetFirstLevelChilds(rulesElem,"rule");
        for (var rule=0; rule< ruleArr.length; rule++) {
            var ruleElem = ruleArr[rule];
            var ruleObj = new Object();

            ruleObj.standardRule = WCXML.GetElementBoolean(rulesElem,"standardRule");
            ruleObj.name = WCXML.GetElementText(ruleElem,"name");

            // look whether you already have it, typical for standard rules
            var found = false;
            for (var rule2=0; rule2<this.rules.length;rule2++) {
                if (ruleObj.name == this.rules[rule2].name) {
                    found=true; break;
                };
            }
            if (found) {
                continue;  // don't add an existing (standard) rule
            } else {
                // declared as standard rule but no longer a standard rule --> degrade --> unlikely to happen
                ruleObj.standardRule = false;
            }

            this.rules.push(ruleObj);  // if the rule is not known yet, add to the list

            this.readConditionFromXML(ruleElem,ruleObj);
			ruleObj.depth = parseInt(WCXML.GetElementText(ruleElem,"depth"));
            ruleObj.visibility = WCXML.GetElementText(ruleElem,"visibility");
            ruleObj.visibilityRule = WCXML.GetElementText(ruleElem,"visibilityRule");            
            ruleObj.visibilityElse = WCXML.GetElementText(ruleElem,"visibilityElse");
            ruleObj.visibilityElseRule = WCXML.GetElementText(ruleElem,"visibilityElseRule");
            


        }
        // replace rule names with rule objects
		// TO DO Check this code: looks more advanced than what we actually use.
        for (var rule=0; rule<this.rules.length; rule++) {
            var ruleObj = this.rules[rule];
            if (ruleObj.visibilityRule) {
                for (var rule2=0; rule2<this.rules.length; rule2++) {
                    if (this.rules[rule2].ruleName == this.rules.visibilityRule) ruleObj.visibilityRule = this.rules[rule2];
                    break;
                }
            }
            if (ruleObj.visibilityElseRule) {
                for (var rule2=0; rule2< this.rules.length; rule2++) {
                    if (this.rules[rule2].ruleName == this.rules.visibilityElseRule) ruleObj.visibilityElseRule = this.rules[rule2];
                    break;
                }
            }
        }
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + "readRulesFromXML: " + err);
    }
};

attributeCategory.prototype.readConditionFromXML = function(ruleElem,ruleObj) {
	"use strict;"
    try {
        if (!ruleElem || !ruleObj) return;    		
 
		ruleObj.condition = WCXML.GetElementText(ruleElem,"condition");		
		ruleObj.compareValue = WCXML.GetElementText(ruleElem,"compareValue");
		ruleObj.operator = WCXML.GetElementText(ruleElem,"operator");		
		var attributeName =  WCXML.GetElementText(ruleElem,"attributeName");
		var attributeId =  WCXML.GetElementText(ruleElem,"attributeId");
		ruleObj.attObj = this.findAttribute(attributeId,attributeName);
		if (ruleObj.attObj) ruleObj.attObj.isInRule = true;
		if (ruleObj.condition == 'AND' || ruleObj.condition == 'OR') {
			var subRules = WCXML.GetFirstLevelChilds(ruleElem,"rule"); // DT120572: var added. (omitted by mistake before use strict was there... consequence: bug caused by recursive call)
			if (subRules && subRules.length>1) {
				ruleObj.ruleObj1 = new Object();
				ruleObj.ruleObj1.parentObj = ruleObj;
				this.readConditionFromXML(subRules[0],ruleObj.ruleObj1);
				
				ruleObj.ruleObj2 = new Object();
				ruleObj.ruleObj2.parentObj = ruleObj;
				this.readConditionFromXML(subRules[1],ruleObj.ruleObj2);
			} else {
				// this rule is broken  
				alert(this.langArr['ACatBrokenRuleProblem']);
			}

        }
 
 
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.readConditionFromXML: " + err);
    }
};

/**
* Loads lists into this.lists and then calls for each an ajax function to fill them
* @param {String} listsXML Content of the <lists> element. (without the <lists> and </lists> tage
* @param {String} baseURL. Ajax URL to be called
* @param {Boolean} fromCache. Says whether the XML is loaded from cache AND it has or will load the lists.  This indicates that there is another attribute category object with the same attribute category and therefore using the same lists
*/
attributeCategory.prototype.readListsFromXML = function(listsXML,baseURL,aCatCacheObj,fromCache) {
    try {
		if (!listsXML) return;
        if (!this.lists) this.lists = new Array();
        var listsArr = WCXML.GetFirstLevelChilds(listsXML,"list");
				
        for (var list=0; list<listsArr.length; list++) {
            var listElem = listsArr[list];
            var listObj = new Object();
			this.lists.push(listObj);
			
			listObj.name =  WCXML.GetElementText(listElem,"name");	
			listObj.type =  WCXML.GetElementText(listElem,"type");	
			listObj.counter = WCXML.GetElementText(listElem,"counter");
			if (!listObj.counter) listObj.counter = null; // normalize to null, not empty string
			
			
			if (listObj.type == 'database') {
				listObj.loadingType = 'OnTheFly'; 
				// if there are other image or checkbox/radio renderings, should load immediate
				
			} else { 
				listObj.loadingType = 'Immediate';
			}
			if (listObj.type != "Excel" && listObj.type != 'database') {

				alert(this.langArr['ACatListProblem1'] + listObj.name + this.langArr['ACatListProblem2'] );
				continue;
			}
			
			if (listObj.type == 'Excel') {
				listObj.file = WCXML.GetElementText(listElem,"file");
				if (!listObj.file) {

					alert(this.langArr['ACatListProblem1'] + listObj.name + this.langArr['ACatListProblem3']);
					continue;
				}
				
				listObj.sheet = WCXML.GetElementText(listElem,"sheet");	
				if (!listObj.sheet) listObj.sheet = "Sheet1";  // Excel defaults the first (and often only) sheet to the name Sheet1
			}
			listObj.hierarchical = WCXML.GetElementText(listElem,"hierarchical");
			if (listObj.hierarchical) listObj.hierarchical = parseInt(listObj.hierarchical,10);
			listObj.invalidateAutomatically = WCXML.GetElementBoolean(listElem,"invalidateAutomatically");
			
			listObj.uniqueIdentifyerColumn	= WCXML.GetElementText(listElem,"uniqueIdentifyerColumn");  // experimental: 1 column is assigned as unique identifyer for the rest
			if (listObj.uniqueIdentifyerColumn === "") listObj.uniqueIdentifyerColumn=-1;
			
			listObj.columns = new Array();
			var columnArr = WCXML.GetFirstLevelChilds(listElem,"column");
			for (var col=0; col< columnArr.length; col++) {
				var columnElem = columnArr[col];
				if (!columnElem) continue;
				var columnObj = new Object();
				
				

				columnObj.columnName = WCXML.GetElementText(columnElem,"columnName");				
				columnObj.attributeId = WCXML.GetElementText(columnElem,"attributeId");	
				columnObj.attributeName = WCXML.GetElementText(columnElem,"attributeName");
				columnObj.uniqueIdentifyer = WCXML.GetElementBoolean(columnElem,"uniqueIdentifyer");
				columnObj.maxValuesInDropdown = WCXML.GetElementBoolean(columnElem,"maxValuesInDropdown");
				
				/*
				columnObj.restrictToCombinations = WCXML.GetElementBoolean(columnElem,"restrictToCombinations");
				columnObj.autoSelect = WCXML.GetElementBoolean(columnElem,"autoSelect");
				*/
				
				var attObj = this.findAttribute(columnObj.attributeId,columnObj.attributeName);
				if (attObj) {
					attObj.listObj = listObj;
					attObj.columnObj = columnObj;				
					columnObj.attObj = attObj;
					attObj.restrictType = 'List';
					var cellObjObj = this.findCell(attObj);
					if (cellObjObj) {
						var cellObj = cellObjObj.cellObj;
						if (cellObj && cellObj.renderOption == 'RadioCheckboxes' || cellObj.renderOption == 'Images') listObj.loadingType = 'Immediate';
					}
				} else {
					// this attribute no longer exists whereas it was part of the cascading dropdown
					//if (this.layoutId) alert("Attribute " + columnObj.attributeName + " was part of a cascading dropdown but no longer exists. Please correct the Attribute Category"); 
					columnObj.attributeId = null; columnObj.attributeName = null;
				}
				
				columnObj.currentIndex = -1;
				
				listObj.columns.push(columnObj);
			}
			
			
		} // end for list
		
		
		
		// do back-end call to fill the list with values and combinations
		// However, in case of the admin layout page, we delay reading until having the entire list of lists
		// same logic for reconnecting the buttons to lists
		if (!this.layoutId && this.currentView != "Header") {	
			this.connectAllButtonsToLists();  
			var loadListNeeded = true;
			if (fromCache) {
				loadListNeeded = false;
				if  (!aCatCacheObj.aCatObjects[0].lists || !aCatCacheObj.aCatObjects[0].lists[list]) loadListNeeded=true;
			}
			if (loadListNeeded) {
				for (var list=0; list<listsArr.length; list++) {
					var listObj = this.lists[list];
					if (listObj) this.readListAjax(listObj,baseURL);  
				}
			} else {
			
				for (var list=0; list<listsArr.length; list++) {
					var listObj2 = this.lists[list];
					var listObj = aCatCacheObj.aCatObjects[0].lists[list];
					if (listObj && listObj2 && listObj.combinations) {  // only copy when the list is already loaded. In the other case, the copy will be done when the load returns - in onSuccess of readListAjax
						this.copyListData(listObj,listObj2);	
					}
				}
			}
		}
		
	
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ " attributeCategory.readListsFromXML" + this.langArr['ACatAtLine'] + err.lineNumber + '  : ' + err.message);
    }
};

/**
* connects all buttons in all views to lists by calling per view connectViewButtonsToLists
* must be called after this.lists is complete (remark it's loaded in 2 times when on the management page
*/
attributeCategory.prototype.connectAllButtonsToLists = function() {
    "use strict";
	try {
		var view, row, col, viewObj;
		
		if (this.views) {
            for (var view=0; view<this.views.length; view++) {
                var viewObj = this.views[view];
                if (viewObj) this.connectViewButtonsToLists(viewObj.cells);
            }
        } else {
            this.connectViewButtonsToLists(this.cells);
        }
	
	
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ " attributeCategory.connectAllButtonsToLists" + this.langArr['ACatAtLine'] + err.lineNumber + '  : ' + err.message);
    }

}


/**
* re-connects buttons in a view to lists 
* @param cells can either be ACatObj.views.cells or ACatObj.cells 
**/
attributeCategory.prototype.connectViewButtonsToLists = function(cells) {
    "use strict";
	try {
		var row, col, button, cellObj, buttonObj,list, att, attObj2, column, listObj;
		
        if (cells == null) return;
		for (row=0; row < cells.length; row++) {
			if ( cells[row] ) {
				for (col=0; col < cells[row].length; col++) {
					cellObj = cells[row][col];
					if (cellObj && cellObj.buttons) {
						for (button=0; button < cellObj.buttons.length; button++) {
							buttonObj = cellObj.buttons[button];
							
							if (buttonObj.listName) {
								listObj = this.findList(buttonObj.listName, null, buttonObj.listCounter);  // convert listName to listObj
								if (listObj) {								
									buttonObj.listObj = listObj;
									if (!listObj.buttons) listObj.buttons = [];
									listObj.buttons.push(buttonObj);
									buttonObj.listName = null; // to not give the impression that this is the correct way of using the list, and to avoid saving to XML
														
									// convert columnName to columnObj for all input attributes
									var objArr = [buttonObj.inputAtts,buttonObj.outputAtts];
									for (var arrIdx=0; arrIdx< objArr.length; arrIdx++) {
										var arr = objArr[arrIdx];  // so this is bottonObj.inputAtts or buttonObj.outputAtts
										if (arr) {
											for (att=0; att < arr.length; att++) {
												attObj2 = arr[att];
												for (column=0; column < listObj.columns.length; column++)  {
													if (listObj.columns[column].columnName == attObj2.columnName) {
														attObj2.columnObj = listObj.columns[column]; break;
														attObj2.columnName = null; // to not give the impression that this is the correct way of using the list, and to avoid saving to XML
													}
												}
											}
										}
									}
								
								} else {
									alert('Cannot find list named ' + buttonObj.listName + ' which is used for a button in cell [' + row + '][' + col + ']'); // TODO decide whether it needs translation - depends on how likely it is that it happens, no translation also better for KnowledgeBase
								}
							}
						   
						}
					}
				}
			}
		}
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ " attributeCategory.connectViewButtonsToLists" + this.langArr['ACatAtLine'] + err.lineNumber + '  : ' + err.message);
    }

}

/**
* Reads a single list from the back-end via an Ajax call
* @param {Object} listObj object containing the spec of the list and receiving the result
* @description For webserver based lists, uses listObj.file Filename of an Excel file to read, listObj.sheet Name of the worksheet to be read. If omitted, Sheet1 will be read, {Array} listObj.columns array of column names
* @param {String} baseURL. URL of the JSP or Servlet to be called for webserver based lists. Not used for database lists
* @return Returns nothing. Results come in listObj.columns (which will be overwritten) and listObj.combinations
*/
attributeCategory.prototype.readListAjax = function(listObj, baseURL) {
	try {
		var url;
				

		if (listObj.loadingType == 'OnTheFly') return;	 // OnTheFly lists are never loaded in advance
		if (this.checkIdenticalLists === false ) {
			// check for identical lists, only with the first call to readListAjax. From then onwards it knows
			// it is important to do this at the very beginning since the rest is asynchronous so you don't know in which sequence things arrive
			// algorithm: take the first list and find all lists which are the same.
			// then take the second list and do the same, but only if that list is different from the first
			for (var list1=0; list1< this.lists.length; list1++) {
				var listObj1 =  this.lists[list1];
				if ((listObj1.type != 'Excel') || listObj1.identicalList) continue;  
				for (var list2=list1+1; list2< this.lists.length; list2++) {
					var listObj2 = this.lists[list2];
					if (listObj2.type != 'Excel') continue;
					
					if (listObj1.file != listObj2.file) continue;
					if (listObj1.sheet != listObj2.sheet) continue;
					if (listObj1.type != listObj2.type) continue;
					if (listObj1.columns.length != listObj2.columns.length) continue;
					for (var col=0; col<listObj1.columns.length; col++) {
						if (listObj1.columns[col].columnName != listObj2.columns[col].columnName) break;			
					}
					if (col != listObj1.columns.length) continue;
					
					// if you get here, you have 2 identical lists
					listObj2.identicalList = listObj1;					
				}
			}
			this.checkIdenticalLists = true;
		}
		
		if (!listObj.identicalList || listObj.identicalList.loadingType == 'OnTheFly') {
			if ( listObj.type == 'Excel') {   // only load the first occurance of identical lists
			
			// create the Ajax url
				url = baseURL;
			url += '?fileName=' + WCXML.urlencode(listObj.file);
			url += '&sheetName=' + WCXML.urlencode(listObj.sheet);
			url += '&listName=' + WCXML.urlencode(listObj.name);
			

			// read columns and append to Ajax url
			

			for (var col=0; col< listObj.columns.length; col++) {				
				url += '&column=' + WCXML.urlencode(listObj.columns[col].columnName);
			}

			} else {
				// database list
				var nColumns = listObj.columns.length;
				
				var url = 'GetRowsFromList.jsp?listType=WCR_DB_List&outputType=JSON&indexed=1';
				url += '&list=' + WCXML.urlencode(listObj.name);		
				url += '&maxrows=10000';  // To do: make this configurable
				url += '&maxcolrows=1000';
				
				
				for (var column=0; column< nColumns; column++ ) {
					var columnObj = listObj.columns[column];
					url += '&output=' +  WCXML.urlencode(columnObj.columnName);
				}
				
			}
			var attCatObj = this;  // closure
			
			ajax({
				url: url,
				type: "GET",
				data: "text",
				debug: this.debug,
				synchronous: false,	
				timeout: 120000,
				onSuccess: function(jsonContent) {
					try {	
						var jsonObj;
						try {						
							jsonObj = jsonParse(jsonContent);
						} catch(err) {   // if jsonParse fails, fall back on eval (which is slower)
							jsonObj = eval('(' + jsonContent + ')');
						}
												
						if (jsonObj.error) {
							alert(attCatObj.langArr['ACatListProblem4'] + listObj.name + attCatObj.langArr['ACatListProblem5'] + jsonObj.error + attCatObj.langArr['ACatListProblemDriver64']);
						} else {
							for (var col=0; col<listObj.columns.length; col++) {
								var columnObj = listObj.columns[col];
								columnObj.values = jsonObj.columns[col].values;

								columnObj.currentIndex = attCatObj.findColumnIndex(columnObj);	
								
								//columnObj.dataProvider = new SDropDataProvider({'dataSource': "ReadExcelServletCombined.jsp"});
							}
							listObj.combinations = jsonObj.combinations;
							
							// look whether there are identical lists
							// share the heavy lifting data (which is read-only anyway and therefore identical)
							for (list2=0; list2 < attCatObj.lists.length ;list2++) {
								var listObj2 = attCatObj.lists[list2];
								if (listObj2.identicalList == listObj) {
									attCatObj.copyListData(listObj,listObj2);									
								}
							}
							try {
								// determine list index
								for (var listIdx=0 ;listIdx< attCatObj.lists.length;listIdx++) {
									if (attCatObj.lists[listIdx] == listObj) break;
								}
								
								// look whether there are other attribute categories to update
								for (var cacheIdx=0; cacheIdx< gACatCache.length; cacheIdx++) {
									if (gACatCache[cacheIdx].aCatObjects[0] == attCatObj) {
										for (var otherCat= 1; otherCat < gACatCache[cacheIdx].aCatObjects.length; otherCat++) {
											var otherACatObj = gACatCache[cacheIdx].aCatObjects[otherCat];
											if (otherACatObj.lists && otherACatObj.lists[listIdx]) {  // check needed because header view does not have lists
												attCatObj.copyListData(listObj,otherACatObj.lists[listIdx]);	
												otherACatObj.displayAttributeCategory();		// repaint this attribute category, otherwise the dropdowns remain on Loading...		
											}
										}
										break;
									}
								
								}
							} catch(err) {
								//alert("readListAjax onSuccess problem: " + err);
							}
							
						
							attCatObj.displayAttributeCategory();
							

						}
					} catch (err) {

						alert(attCatObj.langArr['ACatJSONProblem1'] + listObj.name + attCatObj.langArr['ACatJSONProblem2'] + jsonContent);
						//Typically the excel sheet is still open. Try again later.
						/*
						if (!listObj.failures) listObj.failures=1; else listObj.failures++;
						if (listObj.failures && listObj.failures<10) {							
							setTimeout(function(){attCatObj.readListAjax(listObj,baseURL)},1000);
						}
						*/
					}
				}, // end onSuccess
				onError: function(status,ObjHdl,description) {

					if (status && status != '0') alert(attCatObj.langArr['ACatAjaxProblem1'] + description + '  status ' + status + '\n' + attCatObj.langArr['ACatAjaxProblem2'] + url);
				}
			}); // end ajax
		}

	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.readListAjax:\n"+ err);
    }
};

/** 
* copies column possible values and combinations from listObj1 to listObj2 (by reference)
*/
attributeCategory.prototype.copyListData = function(listObj1, listObj2) {
	if (listObj1 && listObj2 && listObj1.combinations) {  
		for (var col=0; col<listObj1.columns.length; col++) {
			var columnObj2 = listObj2.columns[col]
			if (columnObj2) {
				columnObj2.values = listObj1.columns[col].values;	
				columnObj2.currentIndex = this.findColumnIndex(columnObj2);  // don't copy the currentIndex!!!	
			}
		}
		listObj2.combinations = listObj1.combinations;
	}
}

/**
* reads document link sources from an XML Object
*@return {Array}  array with sources or null when no document link sources are found
* XML looks like
* <docSources>
* <dls><sourceType>Browse</sourceType>... </dls>
* </docSources>
*/
attributeCategory.prototype.readDocLinkSources= function(cellElem, row, col) {
	"use strict";
	var docLinkSources = null;
	try {
		var docLinkSourcesElem = WCXML.GetFirstFirstLevelChild(cellElem,"docSources");
		if (!docLinkSourcesElem) docLinkSourcesElem = WCXML.GetFirstFirstLevelChild(cellElem,"docLinkSources");
		/*
		if (docLinkSourcesElem) {
			var dlsArr =  WCXML.GetFirstLevelChilds(docLinkSourcesElem,"dls");
			if (dlsArr && dlsArr.length>0) {
				docLinkSources = [];
				
				for (var dls=0; dls < dlsArr.length; dls++) {
					var dlsElem = dlsArr[dls];
					var dlsObj = {};
					for (var idx in this.dlsProperties) {
						dlsObj[this.dlsProperties[idx]] = WCXML.GetElementText(dlsElem,this.dlsProperties[idx]);
					}
					docLinkSources.push(dlsObj);
				}
			}
		}

		*/
		if (docLinkSourcesElem) {
			var options = {
							docSourceXML : cellElem ,
							docSourceTableId : 'documentSourceTable',
							functionPrefix : this.functionPrefix + '.cells['+row+']['+col+'].docLinkSources',    
							// no langArr--> use default;
							langArr: this.langArr,											
							textClass : 'darktext_greenlink_11',
							//updateCallBack : function() {alert(' callback')},
							prefix: this.prefix,
							linksId: null
							}
			docLinkSources = new DocumentSourceHandler( options);
		}
	} catch(err) {
		alert(attCatObj.langArr['ACatProblemIn']+ ' attributeCategory.readValuesFromXML:\n' + err);
	}
	return docLinkSources;  

}
/**
* reads document link sources from an XML Object
*@return {Array}  array with sources or null when no document link sources are found
* XML looks like
* <cocLinkTableOptions>
* <dlt><type>DocumentProperty</type>... </dlt>
* </docLinkTableOptions>
*/
attributeCategory.prototype.readDocLinkTableOptions= function(cellElem) {
	var docLinkTableOptions = null;
	
	var docLinkTableOptionsElem = WCXML.GetFirstFirstLevelChild(cellElem,"docLinkTableOptions");
	if (docLinkTableOptionsElem) {
		var dltArr =  WCXML.GetFirstLevelChilds(docLinkTableOptionsElem,"dlt");
		if (dltArr && dltArr.length>0) {
			docLinkTableOptions = [];
			
			for (var dlt=0; dlt < dltArr.length; dlt++) {
				var dltElem = dltArr[dlt];
				var dltObj = {};
				for (var idx in this.dltProperties) {
					dltObj[this.dltProperties[idx]] = WCXML.GetElementText(dltElem,this.dltProperties[idx]);
				}
				docLinkTableOptions.push(dltObj);
			}
		}
	}
	
	return docLinkTableOptions;  

}


// To do: document and get rid of attCatObj in favor of this
attributeCategory.prototype.readValuesFromXML= function(valuesXML) {
	var attCatObj = null;
    try {
        attCatObj = this.objHdl;


        if (!attCatObj) throw(this.langArr['ACatAjaxHandleProblem']);
        if (!valuesXML) throw(this.langArr['ACatAttributeProblem']);
        var errorElem = WCXML.GetFirstFirstLevelChild(valuesXML,"error");
        if (errorElem) {
            var errorMsg = WCXML.GetElementText(errorElem,"message");
			if (attCatObj.errorMsg)  attCatObj.errorMsg += "\n"+errorMsg; else  attCatObj.errorMsg = errorMsg;
			if (attCatObj.currentView != 'Header') {
				var errorCode = WCXML.GetElementText(errorElem,"code");
				// don't alert when there are no attributes. This is already shown on the page
				if(errorCode != "043") alert(attCatObj.langArr['ACatErrorReadingAttributes'] + ":\n" + errorMsg);
			}
			attCatObj.displayAttributeCategory();
            return;
        }


        var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(valuesXML,"attribute_category");
        if (!attributeCategoryElem) throw(this.langArr['ACatServerXMLProblem3']);
        
        // read userunits
        attCatObj.userUnits = WCXML.GetElementText(attributeCategoryElem,"user_units");

        var attributesElem=WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"atts");
        if (attributesElem) {
            // read the restricted sets
            var rsArr = WCXML.GetFirstLevelChilds(attributesElem,"rs");
            if (rsArr) {
                for (var rs=0; rs<rsArr.length; rs++) {
                    var rsElem = rsArr[rs];
                    if (rsElem) {
                        rsObj = new Object;
                        rsObj.id = WCXML.GetElementText(rsElem,"id");
                        rsObj.name = WCXML.GetElementText(rsElem,"rsn");
                        rsObj.type = WCXML.GetElementText(rsElem,"rsat");
                        var type2 = rsObj.type;
    					if (type2) {
    						type2 = type2.toLowerCase();
    					} else {
    						type2 = 'text';
    					}
                        rsObj.values = new Array();
                        var valArr = rsElem.getElementsByTagName("rsv");
                        if (valArr && valArr.length>0) {
                            for (var rsVal=0; rsVal< valArr.length; rsVal++) {
                                var id = WCXML.GetElementAttribute(valArr[rsVal],"id");
                                var value = WCXML.GetElementText(valArr[rsVal],"value");
                                
                                if (value && type2 != 'integer' && type2 != 'number' && type2 != 'text' && type2 != 'float' && type2 != 'datetime') value = WCHTML.getFloatString(value,6);
                                if (value && type2 == 'float') value = parseFloat(value).toString();  
																
                                rsObj.values.push({
                                    id:id,
                                    value:value
                                });
                            }
							
                        }
                        if (!attCatObj.restrictedSets) attCatObj.restrictedSets=new Array();
                        attCatObj.restrictedSets.push(rsObj);
                    }

                }
            } // end reading restricted sets

            // read the attribute values
            var attArr = WCXML.GetFirstLevelChilds(attributesElem,"att");
            if (attArr && attArr.length>0) {
                for (var attribute = 0; attribute < attArr.length; attribute++) {
                    var attElem = attArr[attribute];
                    var attId = WCXML.GetElementText(attElem,"id");
                    if (attId) {
                        attObj = attCatObj.findAttribute(attId);
                        if (attObj) {
                            if (attCatObj.copyValues === true) {
                            	var av = WCXML.GetElementText(attElem,"av");
								if(av!=null && av!=''){ //overwrite only when there's a value
									attObj.value = av;
									// DT112473 Fix empty value at beginning
									if (attObj.value.indexOf('|') == 0 && attObj.value.length > 1) {
										attObj.value = " " + attObj.value;
									}
									
									// Set the index of the column of list restriction if there is any. Only for immediately loading list
									var columnObj = attObj.columnObj;
									if (columnObj && columnObj.values) {
										columnObj.currentIndex = attCatObj.findColumnIndex(columnObj,attObj.value);		
										attObj.listObj.updateValuesNeeded = true;
									}
								}
							}
                            attObj.type = WCXML.GetElementText(attElem,"at");
							attObj.typeLocalized = WCXML.GetElementText(attElem,"atl");
							if (gTextManagementEnabled && attObj.name.substr(0,4) == '$dl$' || attObj.name == 'Text Statement(s)' || attObj.name == 'MasterDocument') {
								attObj.type = attObj.typeLocalized = 'documentlink'; attObj.restrictType = 'DocumentLink';
								attObj.linkAttributes = [{name:'language', id:'0001'},{name: 'count', id:'0002'},{name: 'print item', id:'0003'}];
							}
							attObj.unit = WCXML.GetElementText(attElem,"au");  
							attObj.unitLocalized = WCXML.GetElementText(attElem,"aul");  
							if (!attObj.unit) attObj.unit = attObj.type;  // typical for Integer, Text, Float
							
							// check whether restricted set
                            var rsId = WCXML.GetElementText(attElem,"rs");
                            if (rsId) {
                                for (var rs=0; rs< attCatObj.restrictedSets.length; rs++) {
                                    if (rsId == attCatObj.restrictedSets[rs].id) {
                                        attObj.rs = attCatObj.restrictedSets[rs];	
										attObj.restrictType = 'RestrictedSet';
										if (attObj.columnObj) { // a list type attribute was changed into a restricted set DT 120805
											attObj.columnObj.attObj = null;
											attObj.columnObj = null;
										}
										attObj.listObj = null; // never a listObj when type is RestrictedSet
										var valArr = attObj.rs.values;
										if (valArr && valArr.length ==2 && valArr[0].value == 'yes' && valArr[1].value == 'no') attObj.isCheckBox = true;
                                        break;
                                    }
                                }
								
                            }
                            if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !rsId) {
								if (attObj.value ) attObj.value = attCatObj.correctForDaylightSaving(new Date(parseInt(attObj.value)));			
								attCatObj.setHiddenDate(attObj);                                
                            }
                            
                        } else {
							// so you get a value for an attribute not in the category --> do nothing
						}
                    }
                }
            }
        }
		// if there's a second values url, load this one if it's not loaded yet
        if(attCatObj.valuesURLMerge!=null && $.trim(attCatObj.valuesURLMerge)!='' && !(attCatObj.secondrun==true)){
        	attCatObj.secondrun=true;
        	attCatObj.getAttributeValues(true);
        }else if(attCatObj.autoUpdateProjAttrsCheckbox){
        	attCatObj.getProjectAttributeValues();
        }else{			
        	attCatObj.loaded = true;			
        	attCatObj.displayAttributeCategory();
			attCatObj.onValuesLoaded();
    	}
		    
    } catch(err) {
        alert(attCatObj.langArr['ACatProblemIn']+ ' attributeCategory.readValuesFromXML:\n' + err);
    }
};

/**
* Must be called after each date change
* attObj.hiddenDate is used to paint a MM/DD/YYYY string in a hidden field every time the attribute cell is rendered
*/
attributeCategory.prototype.setHiddenDate = function(attObj) {
	try {
		if (attObj.value ) {										
			var month = (parseInt(attObj.value.getMonth()) + 1) + ""; // getMonth puts January on 0, + "" converts it to a string                                    
			if (month.length <2) month = "0" + month;
			var day = attObj.value.getDate() + "";                                   
			if (day.length <2) day = "0" + day;
			attObj.hiddenDate = month + "/" + day + "/" + attObj.value.getFullYear();
			attObj.oldDate = attObj.hiddenDate;  // remember the old date
		} else {
			attObj.value = "";
			attObj.hiddenDate = "";
			attObj.oldDate = "";
		}
	} catch(err) {
        alert(attCatObj.langArr['ACatProblemIn']+ ' attributeCategory.setHiddenDate:\n' + err);
    }
};		

/**
* Sets defaults for task specifications with project attribute values
* @param valuesXML: XML returning from the SDK call GetAttributes.jsp?projectid=..., see private SDK documentation for details
*/
attributeCategory.prototype.readProjectAttributeValuesFromXML= function(valuesXML) {
	var attCatObj = null;
    try {
    	attCatObj = this.objHdl;


        if (!attCatObj) throw(this.langArr['ACatAjaxHandleProblem']);
        if (!valuesXML) throw(this.langArr['ACatAttributeProblem']);
        var errorElem = WCXML.GetFirstFirstLevelChild(valuesXML,"error");
        if (errorElem) {
        	var errorCode = WCXML.GetElementText(errorElem,"code");
        	// if error code is 043 (no attributes), just do nothing
        	if(errorCode != "043"){
	            var errorMsg = WCXML.GetElementText(errorElem,"message");
				if (attCatObj.errorMsg)  attCatObj.errorMsg += "\n"+errorMsg; else  attCatObj.errorMsg = errorMsg;
				if (attCatObj.currentView != 'Header') {
					alert(attCatObj.langArr['ACatErrorReadingAttributes'] + ":\n" + errorMsg);
				}
        	}else{
        		//If no project attributes found, everything is loaded
        		attCatObj.loaded = true;

        	}
        
			attCatObj.displayAttributeCategory();
			if (attCatObj.loaded) attCatObj.onValuesLoaded();
            return;
        }

        var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(valuesXML,"attribute_category");
        if (!attributeCategoryElem) throw(this.langArr['ACatServerXMLProblem3']);

        var attributesElem=WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"atts");
        if (attributesElem) {
        	var didUpdateValue = false;
            // read the attribute values
            var attArr = WCXML.GetFirstLevelChilds(attributesElem,"att");
            if (attArr && attArr.length>0) {
                for (var attribute = 0; attribute < attArr.length; attribute++) {
                    var attElem = attArr[attribute];
                    var attId = WCXML.GetElementText(attElem,"id");
                    if (attId) {
                        attObj = attCatObj.findAttribute(attId);
                        if (attObj) {
                            attObj.projAttValue = WCXML.GetElementText(attElem,"av");
                            
                            
                        	// check whether restricted set
                        	var rsId = WCXML.GetElementText(attElem,"rs");
                                if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !rsId && attObj.projAttValue && attObj.projAttValue!="") {		
                            	attObj.projAttValue = attCatObj.correctForDaylightSaving(new Date(parseInt(attObj.projAttValue)));
                            }
                                
                            if(attObj.projAttValue){
                            	// replace attribute values for attributes that
                            	//     -- are empty, or have value "***"
                            	//     -- are read-only, and have no restriction
                            	var cellObjRes = attCatObj.findCell(attObj);
                            	var visibility = "";
                            	if(cellObjRes){
                            		visibility = attCatObj.checkVisibility(cellObjRes.cellObj);
                            	}
	                            if((!attObj.value || attObj.value == "" || attObj.value == "***" 
	                            	|| (visibility ===  "ReadOnly" && (!attObj.restrictType || attObj.restrictType === "None")))){
	                            	attObj.value = attObj.projAttValue;
	                            	didUpdateValue = true;
	                            	
		                            if (rsId) {
		                                for (var rs=0; rs< attCatObj.restrictedSets.length; rs++) {
		                                    if (rsId == attCatObj.restrictedSets[rs].id) {
		                                        attObj.rs = attCatObj.restrictedSets[rs];										
		                                        break;
		                                    }
		                                }
		                            }
	                            	if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !rsId) {							
	                                    attCatObj.setHiddenDate(attObj);
		                            }
									if(attObj.columnObj){
									
										if(attObj.listObj.loadingType == 'OnTheFly'){
											attCatObj.checkAutoSelectLazyLoading(attObj,false);
										} else {								
											attCatObj.changeListSelection(attObj,attObj.value);
		                            	}
		                            }	
	                            }
                            }
                        }
                    }
                }
            }
            if(didUpdateValue){
            	attCatObj.notifyChangeListeners();
            }
        }
        attCatObj.loaded = true;
        attCatObj.displayAttributeCategory();
		attCatObj.onValuesLoaded();
    } catch(err) {
        alert(attCatObj.langArr['ACatProblemIn']+ ' attributeCategory.readProjectAttributeValuesFromXML:\n' + err);
    }
};

/**
* called when clicking on the [!] sign when the task specification is different from the project attribute value
* @param row, col - row and col of the cell on which the operation is done 
*/
attributeCategory.prototype.recoverProjectAttributeValue = function(row,col) {
    try {
    	if(!isNaN(row) && this.cells[row]){
			cellObj = this.cells[row][col];
			if(cellObj && cellObj.attObj){
				attObj = cellObj.attObj;
				attObj.value = attObj.projAttValue;	
				if(attObj.type.toLowerCase() == 'richtext'){
					tinyMCE.get(attObj.id).setContent(attObj.value);
				} else if (attObj.type.toLowerCase() == 'datetime' && !attObj.rs) {					
					this.setHiddenDate(attObj);					
					halfhours = attObj.value.getHours() * 2 + Math.floor(attObj.value.getMinutes()/30);
					document.getElementById("longFormat_adate" + attObj.id).value = attObj.hiddenDate;
					document.getElementById("date_period_of_day_" + attObj.id).value = halfhours;
				}else{
					document.getElementById(attObj.id).value = attObj.value;
				}
				try{	
					if(attObj.type.toLowerCase() == 'datetime' && !attObj.rs){
						this.changeDateTime(row,col);
					} else if (attObj.columnObj) {  // attribute is part of cascading dropdown
					
						// redo auto-selection
						attObj.columnObj.autoSelected = false;
						if(attObj.listObj.loadingType == 'OnTheFly'){
							this.checkAutoSelectLazyLoading(attObj,true);
						} else {								
							this.changeListSelection(attObj,attObj.value);
						}
											
                	}else{
						this.changeText(document.getElementById(attObj.id),row,col);
					}
        		}catch(err){}
			}
			delayhidethumb();
			this.displayAttributeCategory();
    	}
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.recoverProjectAttributeValue:\n' + err);
    }
};


/**
* convert a value (potentially representing multiple values) to an index (potentially representing multiple indices
* example: convert Belgium | France to 10~23 where Belgium is country with index 10 and France country with index 23
* @param columnObj object representing the column containing the allowed values
* @param value Optional value to be found. If not provided, uses columnObj.attObj.value (meaning it converts value to index for the column)
*/
attributeCategory.prototype.findColumnIndex = function(columnObj,value) {
	try {
		var foundIndex = -1;
		var valueArr;
		var newIndex = '';
		
		if (value==null && columnObj.attObj) value = columnObj.attObj.value;
	
		if (columnObj && value != '' && value != null) {
			var valArr = columnObj.values;  // the set of possible values

			if (value.constructor == String) {
				valueArr = value.split(this.multiSep);  // the set of actual values
			} else {
				valueArr = [];
				valueArr.push(value);
			}
			
			
			for (var idx=0; idx< valueArr.length; idx++) {
		
				if ( valArr && valArr.length>0) {									
					var columnLength = valArr.length;				
					for (var idx2=0; idx2<columnLength; idx2++) {
						if (valArr[idx2] == valueArr[idx]) break;   // == and not === on purpose! sometimes this matches "3" to the integer 3
					}
					if (idx2<columnLength) foundIndex = idx2; else foundIndex=-1;
				}
				if (newIndex) newIndex+= '~';
				newIndex +=  foundIndex;
			}
					
			
		}

		if (newIndex == '') newIndex = -1;
		return newIndex;
	} catch(err) {
        alert(attCatObj.langArr['ACatProblemIn']+ ' attributeCategory.findColumnIndex:\n' + err);
    }
};

/**
* finds the cell where an attribute is placed
* @return null if attribute is not found in the grid
* @return {Object} {cellObj,row,col}: the found cellObj and the row and col where it's positioned
*/
attributeCategory.prototype.findCell = function(attObj) {
	try {
		var Obj = {};
		
		for (var row=0; row < this.nRows; row++) {
            for (var col=0;col < this.nCols; col++) {
                var cellObj = this.cells[row][col];
				if (cellObj) {
					var attObj2 = cellObj.attObj;
					if (attObj2 && attObj2 == attObj) {  
						Obj.cellObj = cellObj;
						Obj.row = row;
						Obj.col = col;
						return Obj;
					}
				}
            }
        }
		
		
		return null;
	} catch(err) {
        alert(attCatObj.langArr['ACatProblemIn']+ ' attributeCategory.findCell:\n' + err);
    }
};


attributeCategory.prototype.readAttributeList = function(attributeListElem, merge) {
    try {
        if (attributeListElem) {
            var attributeArr = WCXML.GetFirstLevelChilds(attributeListElem,"attribute");
            if (attributeArr) {
                if (!this.attributes) this.attributes = new Array();
                for (var att = 0; att<attributeArr.length; att++) {

                    var deleted = WCXML.GetElementText(attributeArr[att],"deleted");
                    if (!deleted || deleted == "0") {
                        var attObj = new Object();
                        attObj.name =  WCXML.GetElementText(attributeArr[att],"name");
                        attObj.id = WCXML.GetElementAttribute(attributeArr[att],"id");
                        attObj.description = WCXML.GetElementAttribute(attributeArr[att],"description");
                        attObj.unitId = WCXML.GetElementText(attributeArr[att],"attribute_unit_id");
                        attObj.rsId = WCXML.GetElementText(attributeArr[att],"restricted_set_id");
                        attObj.value = "";
                        attObj.placedOriginally = true;
                        attObj.placedArr = new Array();
                        
                        //load is_required flag from xml if task type
                        if (this.isTaskTypePage) {
                        	var isRequired = WCXML.GetElementText(attributeArr[att],"is_required");
                        	if(isRequired == '1'){
                        		attObj.isRequired = true;
                        	}
                        }
                        
                        if (merge) {
                            this.mergeAttribute(attObj);  // goes looking for the same attribute in the already existing list.
                        } else {
                            this.attributes.push(attObj);
                        }
                    }
                } // end for
            }
        }

    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.readAttributeList:\n' + err);
    }
};

/**
* Finds an attribute in the list aCatObj.attributes based on id and name. if both are given, name is given more importance than id.
* @param returnIdx {Boolean} if true, the index in aCatObj.attributes will be returned, else the attribute object.
* @return {Object} or {int} depending on setting of returnIdx. Returns null if not found in case of returnIdx false and -1 if true
*/
attributeCategory.prototype.findAttribute = function(attId,attName,returnIdx) {
    try {
        var attObj = null, attIdx = -1;
        if (attName) {
            for (var attribute=0; attribute < this.attributes.length; attribute++) {
                if (this.attributes[attribute].name.toLowerCase() == attName.toLowerCase()) {
                    attObj = this.attributes[attribute];
					attIdx = attribute;
                    break; // jump out of the for loop
                }
            }
        }
        if (attId && !attObj) {  // not found, try with the Id
            for (var attribute=0; attribute < this.attributes.length; attribute++) {
                if (this.attributes[attribute].id == attId) {
                    attObj = this.attributes[attribute];
					attIdx = attribute;
                    break; // jump out of the for loop
                }
            }
        }

        return returnIdx ? attIdx : attObj;

    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.findAttribute:\n' + err);
        return null;
    }
};

/**
* converts listName (and optionally listCounter and listType) to listObj
* @return {Object} - returns the found listObj or null if not found
* @param {String} listName - name of the list to be found. Remark that listName is not unique. Only the combination of listName, listType and listCounter is unique
* @param {String} listType - database or Excel.  If null, no checking is done on list type
* @param {String} listCounter - counter of the list if the same list is copied. Remark that listCounter can be null, also when the same list is used multiple times (the first one is null). 
*/
attributeCategory.prototype.findList = function(listName, listType, listCounter) {
	"use strict";
    var listObj = null;
	try {
		
		for (var list=0; list < this.lists.length; list++) {
			listObj = this.lists[list];
			var nameMatches = (listObj.name == listName);						
			var typeMatches = ( !listType ) || (listType == listObj.type); 						
			var counterMatches = (!listCounter && !listObj.counter) || (listCounter == listObj.counter);
			
			if (nameMatches && typeMatches &&  counterMatches) break;
						
		}
		if (list == this.lists.length) listObj = null; // not found

    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.findList:\n' + err);
    }
	return listObj;
};

/**
* Error handling for problems with ajax call
*/
attributeCategory.prototype.readError = function(status,ObjHdl,description) {
    
	var attCatObj = this.objHdl;
	var displayElement = document.getElementById(attCatObj.displayId);
	
	var loginForm = document.getElementById('sessionexpiredform');
	
	if ( (description == 'SESSION_EXP' || description == '998') && loginForm!= null ) {
		// redirect to the login.jsp for login again. 
 		loginForm.submit();  		
	} else {
		if (attCatObj.currentView != 'Header') {
			if (displayElement) {
				var htmlcode = attCatObj.langArr['ACatErrorLoadingACat'] + attCatObj.langArr['ACatServerStatus'] + status;
				displayElement.innerHTML = htmlcode;
			} else {
				// alert only when status != 0. Status 0  means typically that the user tries leaving the page
				if (status && status != '0') alert(attCatObj.langArr['ACatServerXMLProblem1'] + status);
			}
		}

		attCatObj.errorStatus = attCatObj.langArr['ACatServerXMLProblem2'];
		
		// check other objects which hoped getting the results from this call
		for (var cacheIdx = 0 ; cacheIdx< gACatCache.length; cacheIdx++) {
			if (gACatCache[cacheIdx].aCatObjects[0] == attCatObj) {
				for (var otherCat= 1; otherCat < gACatCache[cacheIdx].aCatObjects.length; otherCat++) {
					var otherACatObj = gACatCache[cacheIdx].aCatObjects[otherCat];
					otherACatObj.errorStatus = attCatObj.langArr['ACatServerXMLProblem2'];
					if (otherACatObj.currentView != 'Header') {
						var displayElement2 = document.getElementById(otherACatObj.displayId);
						if (displayElement2) displayElement2.innerHTML = htmlcode;						
					}
				}
				
				break;
			}
		}
	}
};

/**
* Remembers whether there have been done changes to the attribute values without saving
*/
attributeCategory.prototype.setAttributesChanged = function() {
    try {
        if (this.attributesChanged != true) {
            this.attributesChanged = true;
            var attributesChangedField = document.getElementById("attributesChangedField");
            if ( attributesChangedField) attributesChangedField.innerHTML = '<font color=#ff0000>' + this.langArr['ACatAttributesChanged'] + '</font>';
            var attributesChangedField2 = document.getElementById("attributesChangedField2");
            if ( attributesChangedField2) attributesChangedField2.innerHTML = '<font color=#ff0000>' + this.langArr['ACatAttributesChanged'] + '</font>';

        }
		anAttributeChanged = true;
		try{
			// WcTaskExecutionPage needs this to check for attributeChanged
			AttributeValueChanged();
		}catch(err){}
    } catch (err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.setAttributesChanged: \n' + err);
    }
};



/*
attributeCategory.prototype.loadList = function(restrictedObj) 
Obsolete function. Code can be found in AttributeCategory_lastgood_20111010.js
Code was never checked in.
*/
	


////////////////////////////////////////////////////////////////////////////////////////////
// VALIDATION FUNCTIONS
////////////////////////////////////////////////////////////////////////////////////////////

/***************************************************************
* Validates the attributes versus required constraints
* @return returns true if all ok, false if some required fields are not filled out
*/
attributeCategory.prototype.validateAttributes = function(force) {
    try {
    	if(!this.loaded){
			alert(this.langArr['ACatCannotSave'] + '\n' + this.langArr['ACatNotLoaded']);		
			return false;
		}
    	
		if (this.validationNeeded === false) return true;				
        var errorMsg = '';
		var buttonErrorMsg = '';
		var nonUniqueAttrsList = '';

        for (var row=0; row < this.nRows; row++) {
            for (var col=0;col < this.nCols; col++) {
                var cellObj = this.cells[row][col];
				if (cellObj) {
					var attObj = cellObj.attObj;
					if (attObj) {  // if it's not an attribute, no need to check on being required
						if ( this.checkVisibility(cellObj) == 'Required') {
							
							// FF workaround: apparently, FireFox does not call the onchange when 
							// submitting while the RTF is still focused when you click submit
							if(attObj.type.toLowerCase() == "richtext"){
								document.getElementById(this.prefix + attObj.id).onchange();
							}
							
							// this cell's attribute is required
							if (attObj.value == null || attObj.value == "") { 
								var label = cellObj.label ? cellObj.label : cellObj.name; // If the label is empty/null, show name, so no null check (or error message will look weird)
								errorMsg += '\n' + this.langArr['ACatRequiredAttribute'] + ' ' + label + ' ' + this.langArr['ACatRequiredButNull'];
								// highlight the element in yellow
								var elem;
								if(attObj.type.toLowerCase() == "richtext"){
									elem = tinyMCE.get(this.prefix + attObj.id).getBody();
								} else {
									elem = document.getElementById(this.prefix  + attObj.id);									
								}
								if (elem) elem.style.backgroundColor = "#FAFFBD";
							}
						}
					}
					if (cellObj.buttons && cellObj.buttons.length > 0) {
						// if there are buttons in the current cell, check whether none of them 
						// has an implicit validation action (by nature/defined)
						for (var button = 0; button < cellObj.buttons.length; button++) {
							var buttonObj = cellObj.buttons[button];
							if (buttonObj.func.toLowerCase() == "checkuniquevalue") {
								// the uniqueness of this attribute value always has to be checked
								if (attObj) {
									if (!this.validateAttributeUniqueValue(attObj, false)) {
										var label = cellObj.label ? cellObj.label : cellObj.name; // If the label is empty/null, show name, so no null check (or error message will look weird)
										nonUniqueAttrsList += '\n' + label;
									}
								}
							} else if (buttonObj.execOnSave) {
																
								var errorObj = null;
								errorObj = this.clickACatButton(buttonObj, null, 'onSave');	
								if (errorObj && errorObj.errorMsg ) buttonErrorMsg += '\nValidation Error on ' + buttonObj.label + ' : ' + errorObj.errorMsg;
							}
						}
					}


					
				}
            }
        }
        
		if (nonUniqueAttrsList) {
			if (this.type && this.type == this.typeOptions[2]) {
				// alter error message in case of task specifications
				errorMsg += '\n' + this.langArr['ACatUniqueValueCheckOnSubmitResult_spec'] + nonUniqueAttrsList;
			} else {
				// standard message for document and project attributes
				errorMsg += '\n' + this.langArr['ACatUniqueValueCheckOnSubmitResult'] + nonUniqueAttrsList;
			}
		}
        if (buttonErrorMsg) {
			alert(buttonErrorMsg);
			return false;
		}
		if (errorMsg) {
			if (this.requestedView == 'Creation' || force == true || nonUniqueAttrsList) {
				alert(this.langArr['ACatCannotSave'] + '\n' + errorMsg);		
				return false;
			} else {
				return confirm(this.langArr['ACatConfirmSave'] + '\n' + errorMsg);	
			}
        } else {
            return true;
        }
		

    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ " attributeCategory.validateAttributes: " + err.toString());
        return '';
    }
};


/**
* called as a part of attributes validation process
* @param {attObj} attObj: The attribute to be checked.
* @param {boolean} buttonClicked: TRUE if this method is called when user explicitly click 
* 	"check unique value" button, FALSE otherwise.
*/
attributeCategory.prototype.validateAttributeUniqueValue = function (attObj, buttonClicked) {
	"use strict";
	
	var retValue = false;
	try {
		var attName = attObj.name;
		var attValue = attObj.value;
		if (!attValue || attValue.length == 0) {
			if (buttonClicked) {
				// report the problem here because it's a direct action of user (not like in validation)
				alert(this.langArr['ACatUniqueValueCheckNoValue']);
				return false;
			} 
			else return true;   // empty values are ignored
		}
		if (!this.type || this.type.length == 0) {
			if (buttonClicked) {
				// report the problem here because it's a direct action of user (not like in validation)
				alert(this.langArr['ACatUniqueValueCheckNoType']);
				return false;
			}
			else return true;   // when we don't know what to check, don't block the whole save process
		}
		
		var url = 'GetObjectsHavingAttValue.jsp';
		url += '?type=' + this.type;
		url += '&attribute=' + WCXML.urlencode(attName) + ':' + WCXML.urlencode(attValue);
		url += '&returntype=count';
		// find out what to exlude (if anything)
		var excludeID = "";
		if (this.type == this.typeOptions[0]) excludeID = this.projectID;  // project
		else if (this.type == this.typeOptions[1]) excludeID = this.documentID;  // document
		else if (this.type == this.typeOptions[2]) excludeID = this.taskID;  // task
		if (excludeID.length > 0) {
			url += '&excludeid=' + WCXML.urlencode(excludeID);
		}

		var aCatObj = this;
		
		ajax({
			url: url,				
			type: "GET",
			data: "xml",
			debug: this.debug,
			synchronous: true,
			timeout: 120000,
			
			/* Ajax call returns an XML like following:
				<result>
					<count>10</count>
				</result>
			*/
		
			onSuccess: function(returnXML) {	
				try {
					var mainElem = WCXML.GetFirstFirstLevelChild(returnXML, "result");
					if (!mainElem) {
						mainElem = WCXML.GetFirstFirstLevelChild(returnXML, "error");
						if (mainElem) {
							// SDK error returned
							var sdkErrorCode = WCXML.GetElementSelfTextContent(WCXML.GetFirstFirstLevelChild(mainElem, "code"));
							var sdkErrorMsg = WCXML.GetElementSelfTextContent(WCXML.GetFirstFirstLevelChild(mainElem, "message"));
							alert("Server returned error\n\n" + sdkErrorCode + ": " + sdkErrorMsg);
							return;
						}
					}
					var countElem =  WCXML.GetFirstFirstLevelChild(mainElem, "count");
					var count = parseInt(WCXML.GetElementSelfTextContent(countElem));
					if (isNaN(count)) {
						alert("Error processing return of ajax call to GetObjectsHavingAttValue.jsp: integer value expected as count");
					} else {
						if (buttonClicked) {
							// explicit action of the user clicking the button - needs immediate feedback
							if (count > 0) {
								if (aCatObj.type == aCatObj.typeOptions[2]) {
									// alter error message for task specifications
									alert(aCatObj.langArr['ACatUniqueValueCheckResult1'] + attName + aCatObj.langArr['ACatUniqueValueCheckResult2_spec']);
								} else {
									// standard error for document and project attributes
									alert(aCatObj.langArr['ACatUniqueValueCheckResult1'] + attName + aCatObj.langArr['ACatUniqueValueCheckResult2']);
								}
							} else {
								if (aCatObj.type == aCatObj.typeOptions[2]) {
									// alter error message for task specifications
									alert(aCatObj.langArr['ACatUniqueValueCheckResultOK_spec']);
								} else {
									// standard error for document and project attributes
									alert(aCatObj.langArr['ACatUniqueValueCheckResultOK']);
								}
							}
						} else {
							// just an implicit validation check - set the return value based on AJAX call result
							retValue = (count == 0);
						}
					}
					aCatObj.displayAttributeCategory();
				} catch (err) {
					alert("Error processing return of ajax call to GetObjectsHavingAttValue.jsp: " + err);
				}
			},
			
			onError: function(errorMsg){
				alert("Error in Ajax call to GetObjectsHavingAttValue.jsp " + errorMsg);
			},
			
			objHdl: this
		});
	
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.validateAttributeUniqueValue' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);
		return false;
	}
	return retValue;
};


////////////////////////////////////////////////////////////////////////////////////////////
// ONCHANGE FUNCTIONS
////////////////////////////////////////////////////////////////////////////////////////////
/**
* Called when somebody changes the date of a date or time or a DateTime attribute
* Updates the attObj.value field with the Date
*/
attributeCategory.prototype.changeDateTime = function(row,col) {
	var mustRepaint = false;
	var attObj = null;
	try {  
        attObj = this.cells[row][col].attObj;
        
        if (attObj) {
            var attId = attObj.id;
            var dateField = document.getElementById(this.prefix + "longFormat_adate" + attId );
            var hourField= document.getElementById(this.prefix + "date_period_of_day_" + attId);

            if (dateField && hourField) {
                var dateString = dateField.value;

                if (dateString) {
                    // update the attObj.value field. this field must be in JavaScript Date format
                    newDate = new Date(dateString);  // normalized date string is used to create a Date object.
					
					if ( (newDate.constructor != Date) || isNaN(newDate.getTime()) ) {
						// entered something which can't be interpreted as a date
						// put back to old value
						var attValue = attObj.value;
						if (attValue.constructor == Date) {
							dateField.value = WCHTML.getDateString(attValue);
						} else {
							dateField.value = "";
							hourField.value = -1;
						}
						 
					} else {    

						// add hours and minutes
						var halfhours = this.defaultHourAsHalfHours;
						if (hourField.value != null && hourField.value!="") halfhours = parseInt(hourField.value);
						if (halfhours == -1) {
							halfhours = this.defaultHourAsHalfHours;
							hourField.value = halfhours;
						}
						
						var hours = Math.floor(halfhours/2);
						var minutes = (hours * 2 == halfhours)? 0:30;
						newDate.setHours(hours); newDate.setMinutes(minutes);
						attObj.value = newDate;

						// update the hidden adate field
						this.setHiddenDate(attObj);

						attObj.changed = true;
						this.setAttributesChanged();
					}
                } else { // this is not a date --> somebody emptied the field --> put it on "No date"
					hourField.value = -1;
					attObj.value = ""; attObj.hiddenDate = "";
					mustRepaint = true;
				}
				var hiddenElem = document.getElementById(this.prefix + "adate" + attId );
				if (hiddenElem) hiddenElem.value = attObj.hiddenDate;
				hiddenElem = document.getElementById(this.prefix +  attId );
				if (hiddenElem) hiddenElem.value = attObj.hiddenDate;
            }
        }
        this.checkTaskSyncWarning(attObj);
		this.executeButtonDependencies(attObj);
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.changeDateTime: " +err);
    }
    // always repaint
	if (mustRepaint) {
		this.displayAttributeCategory();
	} else {
		this.checkRepaintAttributeCategory(attObj);
	}
};


/**
* Called when somebody changes a restricted set value.
* Supports 2 cases: pure restricted sets and lists. 
* Used in the attribute pages, but also in the search panels (when somebody changes a selection in a cascading dropdown in a search panel)
* @param {DOM Object} elem DOM object of the changed element (mostly a HTML <select>)
* @param {Integer} row row of the grid in which the element sits
* @param {Integer} col column of the grid in which the element sits
* @param {String} hpSectionId ID of the home page section when using for search panels, null or '' in other cases
* @param {Boolean} radio true if this element is a radio button selector (with or without images), false if it's a dropdown
*/
attributeCategory.prototype.changeSelect = function(elem,row,col,hpSectionId,radio) {
	var attObj = null;
	var restrictedChanged = false;
    try { 
        if (!elem) throw("elem "+ this.langArr['ACatNotDefinedProblem']);
		var cellObj = this.cells[row][col];
        attObj = cellObj.attObj;
		var newValue;
		if (radio) {
			var selected = parseInt($('input[name=' + this.prefix + attObj.id + ']:checked').attr("idx"),10);
			if (isNaN(selected)) {
				newValue = '';
			} else {
				if (attObj.rs) {
					newValue = attObj.rs.values[selected].value;
				} else if (attObj.columnObj) {
					newValue = attObj.columnObj.values[selected];
				}
			}
		} else {
			newValue = elem.value;	

		}

		if (attObj) {
			if (newValue == '$$ShowAll$$' || newValue == '$$More$$') {
				if (newValue == '$$ShowAll$$') attObj.showAll = true;
				if (newValue == '$$More$$') attObj.showMore = true;
				restrictedChanged = true;
				this.updateSearchDropdowns(attObj.listObj,hpSectionId);
			} else {			
				attObj.value = newValue;			
				if (attObj.columnObj) {
					restrictedChanged = true;
					this.changeListSelection(attObj,newValue,elem,row,col,hpSectionId);
				}
			
				this.setAttributesChanged();
				attObj.changed = true;			

				// Only try to auto re-run the search if we're on a search results screen.
				if (hpSectionId != null && (typeof searchAutoReRun  == 'function')) 
				  searchAutoReRun(hpSectionId);
			}
			this.checkTaskSyncWarning(attObj);
			this.executeButtonDependencies(attObj);
		}
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.changeSelect: " +err);
    }
    
	if (restrictedChanged) {
		this.displayAttributeCategory(null);  // always repaint if this is a dropdown which is part of a list
	} else {
		this.checkRepaintAttributeCategory(attObj);   // only repaint if a rule or button is dependent on this cell
	}
	
};


/**
* Called when changing the selection of a cascading dropdown field
* @param newValue is simply the new value if the field is not multiSelect and is something like abc | def in case of a multiSelect field
* @param elem The element on which the attribute is changed (normally a HTML <select>)
* @param row row in which the changed attribute sits
* @param col col in which the changed attribute sits
* @param hpSectionId section ID of the home page section in case this code is used for cascading dropdown in a search parameter section
* @return 
* internally, column object currentIndex is used, which is encoded for multi value with ~, with indices and no spaces, so 3~123
* 
*/
attributeCategory.prototype.changeListSelection = function(attObj,newValue,elem,row,col,hpSectionId) {
	try {
		var listObj = attObj.listObj;
		var columnObj = attObj.columnObj;
		var newIndex=-1;
		if (listObj  && columnObj != null) {	
			
			// Step 1: determine the new index. For multi-value this is something like 3~123
	
			if (newValue != "") newIndex =  this.findColumnIndex(columnObj,newValue);
			
			// deselect auto-selected dropdowns when needed
			if (!attObj.allowMultipleValues) {
				if (columnObj.currentIndex != -1 && columnObj.currentIndex!= newIndex) {  // the latter is possible when changeSelect is called programmatorically
					// We change a dropdown which already had a value --> deselect all auto-selected dropdowns
					// This is not done for multi-value as there is often no need for it, but also because we don't remember by which ones the auto-selection was done. 
					for (var col2=0; col2<listObj.columns.length; col2++) {
						var colObj2 = listObj.columns[col2];
						if (colObj2.autoSelected && colObj2.attObj != attObj) {  // second condition is to avoid that you deselect again what was just selected. With Show All, you can indeed change an autoselected field
							colObj2.currentIndex = -1;
							
							//colObj2.currentValue = "";
							if (colObj2.attObj) colObj2.attObj.value = "";
							colObj2.autoSelected = false;							
						}
					}
				}
			}
			
			columnObj.currentIndex = newIndex;
			// invalidate all selected values in other dropdowns which are conflicting with the current column
			if (listObj.invalidateAutomatically) this.invalidateConflictingValues(listObj,columnObj);
				
			var success = this.updateListAttributeValues(listObj,newIndex==-1? false:true);  // do not do auto-fill when putting something back on please select, otherwise you risk filling yourself again
			if (!success && !attObj.allowMultipleValues && elem) {  // !elem can happen !!! see callers
				// an invalid selection was done. Bring back the other dropdowns to please select
				// HADW CR: no longer clear when this can happen. 
				// since the dropdowns are limited to the valid values, this should not happen
				// however it can happen anyway when first saving and when somebody changed the attribute category (for example adding extra columns or making a current column with already a value listen to a list)
				this.deselectListAttributeValues(listObj);
				elem.value= newValue; // this migth call ourselves...
				this.changeSelect(elem,row,col);  // in case we are not called (for example on Firefox...)
			}
			
			
			columnObj.autoSelected = false;  // remember that this selection was done by hand
			
			this.updateSearchDropdowns(listObj,hpSectionId);   // update dropdowns in case we are in a search result page
			
			
		}
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.changeListSelection: " +err);
    }
	
};

/** 
* Invalidate all selected values in other dropdowns in listObj which are conflicting with the values in column columnObj
* Assumes that columnObj already has the new value and the new index
* Algorithm: run over all combinations. For all ones which are valid given current value of the column, put the other columns' entries to valid
* then look whether these other column's values are valid
*/
attributeCategory.prototype.invalidateConflictingValues = function(listObj,columnObj) {
	try {
		var combiArr = listObj.combinations;
		if (!combiArr) return;  // typically means that the ajax call for this list did not yet return (or failed). 
		var col;
		var ncols = listObj.columns.length;
		
		// determine current column 
		var curCol = -1;
		for (var curCol=0; curCol< ncols; curCol++) {
			if (listObj.columns[curCol] == columnObj) break;
		}
		if (curCol>=ncols) return;
		var curColIndex = listObj.columns[curCol].currentIndex;
		if (curColIndex == -1) return;  // no need to invalidate anything if there is no value
		var curColIndexArr = null;
		if (curColIndex.constructor == String) curColIndexArr = curColIndex.split('~');
		

		
		// initialize a local valid array (which is different from the one we remember because we want to see what is valid considering only columnObj
		var lValidArr = new Array();
		for (var col=0; col< ncols; col++) {				
			lValidArr[col] = new Array();
			
			for (var idx=0; idx<listObj.columns[col].values.length; idx++) {
				lValidArr[col][idx]=0;				
			}
		}
		
		// run over all combinations and make values in lValidArr true (=valid) based on the value in the combination
		for (var combi=0; combi < combiArr.length; combi++) {
			var combination = combiArr[combi];
			var indexArr = combination.split("~");
			var combiValid = false;
			
			// look whether this combination is valid.
			// it is valid as soon as the value of this combination for the set column is one of the values for this column
			var combiIndexForCurCol = indexArr[curCol];
			for (col=0; col< ncols; col++) indexArr[col] = parseInt(indexArr[col],10);
			
			if (curColIndexArr) {	// current integer is a string with potentially multiple values
				for (var idx=0; idx<curColIndexArr.length; idx++) {
					if (curColIndexArr[idx] == combiIndexForCurCol) {combiValid = true; break;}
				}
			} else {  // current index is just an integer
				if (curColIndex == combiIndexForCurCol) combiValid = true;  
			}
			
			// when the combination is valid, validate the according values in lValidArr
			if (combiValid) {
				for (var col=0; col< ncols; col++) lValidArr[col][indexArr[col]]=true;	
			}
		
		}	
		
		//pick up the indices for each of the columns.  
		var currentArr = new Array();
		for (col=0; col<ncols; col++) {
			if (col != curCol) {
				var columnObj = listObj.columns[col];
				var colIndex = columnObj.currentIndex;
				if (colIndex == -1) continue;  // never invalidate what is not there
				var colIndexArr = null;
				var newIndex = '', newValue='';
				if (colIndex.constructor == String) {
					colIndexArr = colIndex.split('~');
					
					for (var idx=0; idx<colIndexArr.length; idx++) {  // build up a new encoded index with only the allowed values
						if (lValidArr[col][parseInt(colIndexArr[idx],10)]) {
							if (newIndex) newIndex += '~';
							newIndex += colIndexArr[idx];
							if (newValue) newValue += this.multiSep;
							newValue += columnObj.values[colIndexArr[idx]];
						}
						if (newIndex != columnObj.currentIndex) {
							columnObj.currentIndex = newIndex? newIndex : -1;
							if (columnObj.attObj) columnObj.attObj.value = newValue;
						}
					}
				} else {
					if (!lValidArr[col][colIndex]) {
						columnObj.currentIndex = -1; columnObj.autoSelected = false;
						if (columnObj.attObj) columnObj.attObj.value = '';
						
					};
				}
				
				
				
			}
		}		
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.invalidateConflictingValues: " +err);
    }

}


// update dropdowns for affected columns in case of search results panel (which you detect by the existance of the parent db_attr_td_xxx)
attributeCategory.prototype.updateSearchDropdowns = function(listObj,hpSectionId) {

	try {

		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		var changedPrefix = this.prefix ? this.prefix : "";
		var endPos = changedPrefix.indexOf('_db_attr_input');
		if (endPos>0) {  // home page
			changedPrefix=changedPrefix.substring(0,endPos);  // now looks like 00002_0000000011, which is the home page ID
			changedPrefix+='_db_attr_td_';  // now looks like 00002_0000000011_db_attr_td_ on home page 
		} else {
			changedPrefix = 'db_attr_td_';
		}
		
		for (var col2=0; col2<listObj.columns.length; col2++) {
			var colObj2 = listObj.columns[col2];
			var attObj2 = colObj2.attObj;
			
			if (attObj2) {
				var parentElemId = changedPrefix +  attObj2.id;
				var parentElem = document.getElementById(parentElemId);   
				// on home page, must look like 00002_0000000011_db_attr_td_00002_0000000043
				// however the prefix looks like this: 00002_0000000011_db_attr_input_
				if (parentElem) {
					// you need to update the dropdown for the search panel
					// for that we need the position in the cell array (renderRestrictedList needs this)
					var row3,col3;
					for (row3=0; row3<this.cells.length; row3++) {
						var row3Arr = this.cells[row3];
						if (row3Arr) {
							for (col3=0; col3<row3Arr.length; col3++) {
								var cellObj = row3Arr[col3];
								if (cellObj && cellObj.attObj == attObj2) {
									parentElem.innerHTML = this.renderRestrictedList(this.cells[row3][col3],row3,col3,true,hpSectionId);
								}
							}
						}
					}
				}
			}
		}
	} catch(err) {
		alert(this.langArr['ACatProblemIn'] + " attributeCategory.updateSearchDropdowns: " +err);
	}

}

/**
 * Notify listeners that a value was updated from an external source (e.g. ProjectAttributes)
 */
attributeCategory.prototype.notifyChangeListeners = function() {
	var len=this.valueChangedListeners.length;
	for(var i=0; i<len; i++) {
		var value = this.valueChangedListeners[i];
		value.call();
	}
}

/**
* Called when somebody changes a single line or multiline text
*/
attributeCategory.prototype.changeText = function(elem,row,col) {
	var attObj = null;
	var valueChanged = false;
	
    try {    
        if (!elem) throw("elem "+ this.langArr['ACatNotDefinedProblem']);
		this.setAttributesChanged();
        var newValue = elem.value;

		if (!attObj && this.cells[row] && this.cells[row][col]) attObj = this.cells[row][col].attObj;
		
        if(attObj && attObj.type){
	        
			// check floating point and integer types
			switch (attObj.type.toLowerCase()) {
				case 'richtext':
					newValue = tinyMCE.get(elem.id).getContent();
					elem.value = newValue;
					break;
				case 'text':
				case 'datetime':
					break;// do nothing
				case 'integer':
				case 'number':
					WCHTML.checkNumericInput(elem,"Integer",false,attObj.value);
					break;
				case 'area':
				case 'area1':
				case 'area2':
				case 'volume':
				case 'volume1':
				case 'volume2':
				case 'weight':
				case 'weight1':
				case 'float':
					WCHTML.checkNumericInput(elem,null,false,attObj.value);
					break;
				case 'distance': 
				case 'distance1':
					if(attObj.fractionalIfImperial  && this.userUnits && this.userUnits.toLowerCase() == "imperial") {
						newValue = WCHTML.fractionalToDecimal(newValue);
						var fracValue = '';
						if(isNaN(newValue)){
							newValue = '';
						}else{
							fracValue = WCHTML.decimalToFractional(newValue);
						}
						elem2 = elem;
						elem = document.getElementById(this.prefix + attObj.id);
						elem.value = newValue;
						elem2.value = fracValue;
					} else {
						WCHTML.checkNumericInput(elem,null,false,attObj.value);
					}
					break;
				default:				
					break;
			} // end switch
			if (attObj.value !== elem.value) valueChanged = true;
			attObj.value = elem.value;
			// CR HADW: this looks contradictory to valueChanged however it is not really. changeText can be called more than once for 1 change. The second time valueChanged will be false (since the first call already did the job)
			// Still if you get here, you are actually sure that the value has changed (otherwise changeText should not be called at all). 
			// valueChanged is needed to avoid calling the checkAutoSelectLazyLoading too many times (each time is a server call).
			if (attObj.listObj && attObj.listObj.loadingType == 'OnTheFly'  && valueChanged) {
				this.checkAutoSelectLazyLoading(attObj,false);  // auto-selects single remaining				
			}
			attObj.changed = true;  
			
			this.checkTaskSyncWarning(attObj);     
	        
        }
		this.executeButtonDependencies(attObj);
		
    } catch (err) {
		if (err.name != "NS_ERROR_UNEXPECTED") {  // DT112225: ugly message shown for tiny_mce, also when no good reason
			alert(this.langArr['ACatProblemIn'] + " attributeCategory.changeText: " +err);
		}

    }
   
	if (attObj) this.checkRepaintAttributeCategory(attObj);
};

/**
 * Checks if the warning flag for synchronizing project attributes with task specifications should be set
 */
attributeCategory.prototype.checkTaskSyncWarning = function(attObj){
	var isDateAndNotRS = ((attObj.type && attObj.type.toLowerCase() == 'datetime') && !attObj.rs);

	// may have to place warning flag if specification will alter existing project attribute value
	if(this.autoUpdateProjAttrsCheckbox){
		var warning = document.getElementById("imgwarning"+attObj.id);
		if(warning){
			if((isDateAndNotRS && (attObj.value.toString() != attObj.projAttValue.toString())) || 
					(!isDateAndNotRS && (attObj.value != attObj.projAttValue))){
				warning.style.display = "inline";
				warning.style.visibility = "visible";
			} else {
				warning.style.display = "none";
				warning.style.visibility = "hidden";
			}
		}
	}
}


/**
* Updates property validArr in all attributes coupled to list represented by listObj
* Algorithm: Look into all combinations and mark according indices to be valid 
* validArr[idx]==0 means not valid, 1 means always valid, 2 means valid because what makes it invalid is a selection on either the same column or a column further downstream in a hierarchical set
* 3 means that the value is currently selected
* @param doAutoSelect {Boolean} determines whether elements will be autoselected. Autoselection is normally only done after a change of one of the attributes by the user, not when loading from the database 
* We want to guarantee that what sits in the database is also shown. 
* @return {Boolean} false if an invalid selection was made, true in all other circumstances (including that you can't do the work because lists are not yet loaded)
*/
attributeCategory.prototype.updateListAttributeValues = function(listObj,doAutoSelect) {
	try {
		
		var combiArr = listObj.combinations;
		if (!combiArr) return true;  // typically means that the ajax call for this list did not yet return (or failed). 
		var col;
		var ncols = listObj.columns.length;
		var lastColWithValue = -1;
		
		//pick up the indices for each of the columns.  
		var currentArr = new Array();
		for (col=0; col<ncols; col++) {
			if (listObj.columns[col].autoSelected) {
				currentArr.push(-1);	// effect is that autoselected columns don't restrict other columns. 		
			} else {
				currentArr.push(listObj.columns[col].currentIndex);			
			}
		}
		
		// reset all valid arrays (colObj.validArr) to 0
		for (var col=0; col< ncols; col++) {				
			var colObj = listObj.columns[col];
			var columnLength = colObj.values.length;
			if (!colObj.validArr) colObj.validArr = new Array(columnLength);
			for (var idx=0; idx<columnLength; idx++) {
				colObj.validArr[idx]=0;				
			}
			if (colObj.currentIndex.constructor == String) {
				var selectedArr = colObj.currentIndex.split('~');
				for (var selected=0; selected<selectedArr.length; selected++) {
					colObj.validArr[selectedArr[selected]]=3;  // whatever is selected is set to 3
				}
			}
			if (colObj.currentIndex!=-1) lastColWithValue = col;
		}
		
		// run over all combinations and make values in validArr true (=valid) based on the value in the combination
		for (var combi=0; combi < combiArr.length; combi++) {
			var combination = combiArr[combi];
			var indexArr = combination.split("~");
			for (col=0; col< ncols; col++) indexArr[col] = parseInt(indexArr[col],10);
			
			
			var mismatches = 0;  // Count number of mismatches for this combination.  A mismatch is an index for a column which is not currently selected in a column which has a selection
			var colMismatch = new Array();
			var firstMismatch = -1;
			for (var col=0; col< ncols; col++) {colMismatch[col]=false;}
			
			for (var col=0; col< ncols; col++) {	
				if (currentArr[col] == -1) continue;  // column has no value or is not in use --> never invalidate a combination for this
				
				if (currentArr[col].constructor != String || currentArr[col].indexOf("~") < 0) {	 // single value selected for this column
					if (currentArr[col]!=indexArr[col]) {
						mismatches++;colMismatch[col]=true;
						if (mismatches ==1) firstMismatch=col;
					};
				} else {
					// multiple values are selected for this column
					var currentArr2 = currentArr[col].split('~');
					var found = false;
					for (var idx2=0; idx2<currentArr2.length; idx2++) {
						if (currentArr2[idx2]==indexArr[col]) {found=true; break;};
					}
					if (!found) {
						mismatches++;colMismatch[col]=true;
						if (mismatches ==1) firstMismatch=col;
					};
				}
			}
			
			
			
			for (var col=0; col< ncols; col++) {
				var idx = indexArr[col];
				var colObj = listObj.columns[col];
				
				if (colObj.validArr[idx]==1 || colObj.validArr[idx]==3) continue; // it's already fully valid, no reason to find more proof
				
				if (currentArr[col] != -1 && colMismatch[col]) {
					// This column is already filled out with a different value than the one of this combination
					// If that is the only mismatch, or in the case of hierarchical lists, the first mismatch, you still need to allow changing this dropdown
					// So set this combination's validity for this column ONLY to true
					
					if (listObj.hierarchical) {
						if (col == firstMismatch) colObj.validArr[idx]=2;	
					} else {
						if (mismatches <2) colObj.validArr[idx]=2;							
					}
					
				} else {  
					// this column is not filled out or has the value of this combination
					// If there are no mismatches, allow all column's validity
					if (mismatches == 0) {
						colObj.validArr[idx] = 1;
					} else if (listObj.hierarchical && col<firstMismatch) {
						colObj.validArr[idx]= 2;
					}
				}
				
			}
		
		}
		
		// autoselect single remaining 		
		for (var col=0; col< ncols; col++) {
			var nValid = 0;
			var colObj =  listObj.columns[col];			
			var columnLength = colObj.values.length;
			var validIdx = -1;	
			for (var idx=0; idx<columnLength; idx++) {
				if (colObj.validArr[idx] ) {nValid++; validIdx = idx;};
			}
			colObj.nValid = nValid;
			if (nValid ==1 && colObj.currentIndex == -1) {  // only auto-fill what is not filled yet
				if (doAutoSelect) {
					colObj.currentIndex = validIdx;				
					var newValue = colObj.values[validIdx];
					//colObj.currentValue = newValue;
					var attObj = colObj.attObj;
					if (attObj) attObj.value = newValue;					
					colObj.autoSelected = true;  // Remember what was automatically selected. 
				}
			} else if (nValid==0 && colObj.currentIndex != -1){
				// something was selected which makes nothing in this column still valid 
				// DT102015. Fix 22 March: also check whether something was there in the first place - check on currentIndex !=-1
				// If you don't do this, you will fail any selection for which no values exist in another column
				// this includes "irrelevant columns". 
				// Worse: you risk an endless loop because return false will force another call to updateListAttributeValues...
				return false;
			}
			
		}
		
		// backwards autoselect for hierarchical	
		if (listObj.hierarchical && doAutoSelect) {
			for (var col=0; col< lastColWithValue; col++) {
				var colObj = listObj.columns[col];
				var attObj = colObj.attObj;
				if (attObj && attObj.allowMultipleValues) {
					// a further column in a hierarchical set has a value. Select everything which is allowed for that value (this risks some funny effects if the data is not really hierarchial...)
					var newValue = attObj.value;
					var newIndex = colObj.currentIndex;
					if (newIndex == -1)  newIndex = '';
					for (var idx=0; idx<columnLength; idx++) {
						if (colObj.validArr[idx]==1 ) {
							if (newValue) newValue += ' | ';
							newValue += colObj.values[idx];
							if (newIndex) newIndex += '~';
							newIndex += idx;					
						}			
					}
					// TO DO check on maximum size
					colObj.currentIndex = newIndex;
					if (newIndex == '') colObj.currentIndex = -1;
					
					attObj = colObj.attObj;
					if (attObj) attObj.value = newValue;
					// TO DO: check on need to update hidden fields (calling changeText)
				}
			}
		}
		
		
		
		// autoselect unique identifyer column
		var uniqueIDCol = listObj.uniqueIdentifyerColumn
		if (uniqueIDCol != -1 && uniqueIDCol !== ""  && listObj.columns[uniqueIDCol]) {
			// check whether all other columns have values
			for (col=0; col<listObj.columns.length; col++) {
				if (col==uniqueIDCol) continue;
				if (listObj.columns[col].currentIndex == -1) break;
			}
			if (col==listObj.columns.length) {
				// all other columns have values: select all valid values for the unique column
				var columnObj = listObj.columns[uniqueIDCol];
				var attObj = columnObj.attObj;
				if (attObj) {
					var validArr = columnObj.validArr;
					var valueArr = columnObj.values;
					var nItems = validArr.length;
					var newValue = '',newIdx='';
					for (var idx=0; idx< nItems; idx++) {
						if (validArr[idx]) {
							if (newValue) newValue += ' | ';
							newValue += valueArr[idx];
							if (newIdx) newIdx += '~';
							newIdx += idx;
						}
					}
					columnObj.value = newValue;
					attObj.value = newValue;
					attObj.combiIdx = this.findCombiIdx(attObj.listObj, columnObj, -1, attObj.value);
					//this.changeText(document.getElementById(this.prefix  + columnObj.attObj.id),null,null, columnObj.attObj);
				}
			
			}
		
		}
		
		// full repaint - I think not needed
		//this.displayAttributeCategory();
		listObj.updateValuesNeeded=false;
		return true;
		
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.updateListAttributeValues" + this.langArr['ACatAtLine'] + err.lineNumber + "  : " + err.message);

    }
	
};

/**
* deselects all dropdowns from a list 
*/
attributeCategory.prototype.deselectListAttributeValues = function(listObj) {
	try {
		var ncols = listObj.columns.length;
		var isSearchPanel = false;
		if (this.prefix && this.prefix.indexOf('db_attr_input') != -1) isSearchPanel = true;
		for (var col=0; col< ncols; col++) {
			
			var colObj = listObj.columns[col];			
			colObj.validArr = null;			
			colObj.currentIndex = -1;		
			if (colObj.attObj) {
				colObj.attObj.value = "";  // attObj is null if the column is not assigned.
				if (isSearchPanel) {
					$('#' + this.prefix + colObj.attObj.id).val('');
				}
			}
		}
		
			
		
		this.displayAttributeCategory();
	
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.deselectListAttributeValues" + this.langArr['ACatAtLine'] + err.lineNumber + "  : " + err.message);

    }
};

/**
* highlights all attributes linked to the same list 
* @param {String} listName - name of the list
* @param {String} listType - type of the list (Excel or database)
* @param {String} listCounter - counter of the list (if duplicated)
* @param {Boolean} highlight - if true, related items will be  highlighted, else highlight will be taken away
*/
attributeCategory.prototype.highlightRelatedValues = function(listName, listType,listCounter,highlight) {
	try {
		// find list object
		var listObj = this.findList(listName, listType, listCounter);
		
		if (listObj) {
			var ncols = listObj.columns.length;
			for (var col=0; col< ncols; col++) {
				
				var colObj = listObj.columns[col];			
				if (colObj.attObj) {
					var attId = colObj.attObj.id;
					var elem = document.getElementById(this.prefix + attId);
					// single value selector
					if (elem) {
						if (highlight) {
							elem.style.backgroundColor = 'yellow';
						} else {
							elem.style.backgroundColor = '';	// not entirely right, should be the highlight of the style, but I won't solve this... (not important enough and with next repaint it's again ok)
						}
					}
					
					// multi value selector
					var elem = document.getElementById('mvdiv_' + this.prefix + attId);
					if (elem) {
						if (highlight) {
							elem.style.backgroundColor = 'yellow';
						} else {
							elem.style.backgroundColor = '';	// not entirely right, should be the highlight of the style, but I won't solve this... (not important enough and with next repaint it's again ok)
						}
					}
				}
			}
			
		}
	
	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.deselectListAttributeValues" + this.langArr['ACatAtLine'] + err.lineNumber + "  : " + err.message);

    }
};

/**
* Checks whether an attribute category  must be repainted entirely (and does the repaint) when a certain attribute changed value
* @param attObj: the attribute which changed value
*/
attributeCategory.prototype.checkRepaintAttributeCategory = function(attObj) {
	
	if (attObj.isInRule || (attObj.buttonDependency && attObj.buttonDependency.length>0) ) {
	    this.displayAttributeCategory();
	}
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DISPLAY FUNCTIONS
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
var logInfo = "";
/**
* Used to display an attribute category in a document or project or as preview in admin
*/
//attributeCategory.prototype.displayAttributeCategory = function(temp,hpSectionId) {
attributeCategory.prototype.displayAttributeCategory = function(temp) {
	if(!this.loaded && !this.errorMsg) return;
	
	try {
    	temp = temp || (this.currentView == 'Header'); // variable that prevents richt text editors from initializing
    												   // (not needed in header view, not needed in page whara attributes are not loaded yet)
		this.repaintNeeded = false;

        // check the element to write into
        var displayElement = document.getElementById(this.displayId);
        if (!displayElement) return;
        if (this.noCurrentViewDisplay) return;  // a non-existing view with no fallback is asked. Display nothing
		
        // before html is removed from elementy, be sure to unload the rich text editors, or we will get errors when using Internet Explorer
		tinyMceMgrObj.hideRichTextFields(this.id);
        
		var htmlcode = '';
				
		if (this.currentView == 'Header' && this.noHeader == true)
		{
			// system asks for header view but it does not exist
			// leave htmlcode empty, so display nothing
		} else {
			logInfo += "time: " + new Date() + "  errorMsg "  + this.errorMsg + "\n"; 
			if (this.errorMsg) {
				if (this.currentView != 'Header') {
					htmlcode += '<span class="darktext_darklink_bold_12">' + this.errorMsg + '</span>';
				}
			} else if (this.currentView == 'Header') {
				htmlcode += this.displayTableHeader();							
			} else {				
				htmlcode += this.displayTable();
			}
		}
		if (this.currentView != 'Header') htmlcode += '<div id="'+this.prefix+'hiddenFieldsContainer"></div>';  // will be filled with hidden fields in case of creation view
		
        displayElement.innerHTML = htmlcode;
		
		if (htmlcode != '')
		{
			// enable calendars
			for (var row=0;row<this.nRows;row++) {
				for (var col=0; col<this.nCols; col++) {
					if (this.cells[row] && this.cells[row][col] && this.cells[row][col].attObj) {
						var attObj = this.cells[row][col].attObj;
						if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
							var dateId = "#" + this.prefix + "longFormat_adate" + attObj.id;
							var hourId = "#" + this.prefix + "date_period_of_day_" + attObj.id;
							this.createCalendar(dateId,hourId);
						}
					}
				}
			} // end enable calendars
			// enable sDrops
			//this.initSDrops();
		}
		
		if (this.requestedView == 'Creation') this.addDefaultHiddenFields();
		
		if (this.repaintNeeded) {
			this.displayAttributeCategory(temp); // needed when you have chained rules
		} else if(!temp){
			try {
				tinyMceMgrObj.addRichTextFields(this.id);
			} catch(err) {
				// do nothing. Typical case is code being used in 10.2 for cascading dropdowns. 
			}
		}

    } catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.displayAttributeCategory' + this.langArr['ACatAtLine'] + err.lineNumber + ' :\n' + err.message);
    }
};

/** 
* Displays table in WebCenter look
*/
attributeCategory.prototype.displayTable = function(forPrint) {
    try {
		
        if (!this.cells) return '';

		var displayRow=0; // counts the rows you are really displaying
		var startRow=0;
		var hiddenHTML = '';

        var htmlcode = '';
		
		// determine whether to put a header bar or a rounded title
		var showRoundedHeaderBar = false, showRoundedTitle = false;
		if (this.layout == "OneColumn" && this.currentView != 'Header') {
			showRoundedHeaderBar = true;
		} else {
			if (this.cells[0] && this.cells[0][0]) {	
				var visibility = this.checkVisibility(this.cells[0][0]);
				
				if (this.cells[0][0].type == 'HeaderBar' && this.cells[0][0].colSpan == this.nCols && visibility != 'Hidden' && visibility != 'HiddenKeepValues') {
					showRoundedHeaderBar = true;
					startRow = 1;
				}
				if (this.cells[0][0].type == 'Title' && this.cells[0][0].colSpan == this.nCols && this.cells[0][0].styleObj.name == 'WebCenter Title') {
					showRoundedTitle = true;
					startRow = 1;
				}
			}
		}
		
		if (this.currentView !='Header') {
			if (showRoundedHeaderBar || showRoundedTitle) {
				htmlcode += '<table class="ordtable" width="100%">';
			} else {  // if there is no header, then there is also no frame around the attributes. Then the bottom line is ugly and should be removed.
				htmlcode += '<table class="ordtable" width="100%" style="border-bottom:0">';
			}
		} else {
			htmlcode += '<table>';
		}
		
        // put the header bar
		if (showRoundedHeaderBar) {
			htmlcode += '<thead><tr>';
			htmlcode += '<th class="leftCorner"/>';
			for (var col=0; col< this.nCols; col++) {
				htmlcode += '<th align="left" >' + this.langArr['ACatName'] + '</th>';
				htmlcode += '<th align="left">' + this.langArr['ACatValue'] + '</th>';
				var localizedType = this.showTypeAsUnit? this.langArr['ACatType'] : this.langArr['ACatUnit'];
				if (col ==this.nCols-1) {
					htmlcode += '<th class="lastHeaderCol" align="left">' + localizedType + '</th>';
				} else {
					htmlcode += '<th align="left">' + localizedType + '</th>';
				}
			}
			htmlcode += '<th class="rightCorner"/>';
			htmlcode += '</tr></thead>';
		}
		
		// also put a rounded header if the first row is a full spanning title of style WebCenter Title
		if (showRoundedTitle) {
			htmlcode += '<thead><tr>';
			htmlcode += '<th class="leftCorner"/>';
			htmlcode += this.renderTitle(this.cells[0][0], true, 0, 0, "Hidden");		
			htmlcode += '<th class="rightCorner"/>';
			htmlcode += '</tr></thead>';
		}
		

		htmlcode += '<tbody>';
		
		// for IE: force minimum widths for the value column
		if (detectIfIE()) {
			htmlcode += '<tr style="padding:0;"><td style="padding:0;"></td>';
			var minWidth = Math.floor(300/this.nCols);			
			for (var col=0; col<this.nCols; col++) {
				htmlcode += '<td style="padding:0;"></td><td style="padding:0;font-size:0px"><img src="images/clearpixel.gif" height="0" width="'+ minWidth +'"/></td><td style="padding:0;"></td>';
			}
			htmlcode += '<td style="padding:0;"></td></tr>';
		}
		
		
        for (var row=startRow; row< this.nRows; row++) {
            var htmlcode2= ''; // avoid too much appending to the same string, slow in IE
			var rowEmpty = true;
			var htmlArr = new Array(this.nCols);
			var lastBackGround = null;  // remember the style at the end
			var lastHeight = null;
			
            
            if (this.cells[row]) {

                for (var col=0; col<this.nCols; col++) {
                    var cellObj = this.cells[row][col];
					if (cellObj) {
						var visibility = "Editable";
						visibility = this.checkVisibility(cellObj);  // returns one of  Editable, ReadOnly, Hidden, HiddenKeepValues or Required                    
						
						if (visibility !=  "Hidden" && visibility !=  "HiddenKeepValues") {	
							
							switch (cellObj.type) {
								case 'Title':							
									htmlArr[col] = this.renderTitle(cellObj,false, row, col, visibility);
									break;
								case 'Spacer':
									htmlArr[col] = this.renderSpacer(cellObj);
									break;
								case 'Ruler':
									htmlArr[col] = this.renderRuler(cellObj);
									break;
								case 'Button':
									if (visibility !=  "ReadOnly") htmlArr[col] = this.renderButton(cellObj,row,col,false, forPrint);
									break;
								case 'Attribute':
											
									var attObj = cellObj.attObj;
									if (attObj) {
										if (cellObj.visibility == 'Hidden' || cellObj.visibility == 'HiddenKeepValues') {
											// returns from being hidden
											var colObj = attObj.columnObj;
											if (colObj) {  // part of a list.  Value can already be determined by select single options, so don't overwrite this.												
												if (attObj.isInRule && (attObj.columnObj.autoSelected )) this.repaintNeeded = true; // for chained rules
											} else if (cellObj.visibility == 'Hidden' && attObj.previousVisibleValue!=null ) {
												// Show the last value when it was still visible.
												attObj.value = attObj.previousVisibleValue;
												if (attObj.isInRule) this.repaintNeeded = true; // for chained rules											
											}											
										}
										if (cellObj.visibility == 'Hidden') cellObj.attObj.previousVisibleValue = "";  // no longer remember once shown
										if (attObj.columnObj && attObj.columnObj.uniqueIdentifyer=='1') {
											htmlArr[col] = this.renderItemList(cellObj,row,col,visibility);										

										} else {
											htmlArr[col] = this.renderAttribute(cellObj,row,col,visibility, forPrint);
										}
										
									} // end if attObj
									break;                            
								case 'HeaderBar':
									var colSpan = cellObj.colSpan? cellObj.colSpan : 1;
									htmlArr[col] = this.renderHeaderBar(colSpan, col==0, col+colSpan>=this.nCols);
									break;
								case 'Property':
									htmlArr[col] = this.renderProperty(cellObj,row,col,visibility,forPrint);
									break;
								default:
									alert(this.langArr['ACatUnknownCellTypeProblem'] + cellObj.type + this.langArr['ACatId'] + cellObj.id);
									break;

							}
							rowEmpty = false;
							
							// determine colspan
							var colSpan = parseInt(cellObj.colSpan);
							if (!colSpan) colSpan = 1;
							
							// check whether this is the last column - needed later
							if (col + colSpan == this.nCols) {
								if (cellObj.styleObj) {
									lastBackGround = cellObj.styleObj.background;
									if (cellObj.styleObj.height) {
										lastHeight = cellObj.styleObj.height;
									}
								} else if (cellObj.background) {
									lastBackGround = cellObj.background;
									if (cellObj.height) {
										lastHeight = cellObj.height;
									}
								}
							}
							// skip colspanned columns
							col += colSpan-1;
						} else {					
							htmlArr[col] = this.renderEmptyCell();
							var attObj = cellObj.attObj
							if (attObj) {		
								
								hiddenHTML += this.renderHiddenCell(attObj, visibility == "HiddenKeepValues");							
								if (visibility !=  "HiddenKeepValues" && attObj.value != "" && !attObj.columnObj) {  // invisible attributes carry no value except when they are part of a cascading dropdown								
									attObj.previousVisibleValue = cellObj.attObj.value;
									attObj.value = "";
									if (attObj.isInRule) this.repaintNeeded = true; // for chained rules
								}
								
							}
						}
						cellObj.visibility = visibility;  // remember to see whether it changes
					
					} else {  // !cellObj
						htmlArr[col] = this.renderEmptyCell();
					}
                }
            }
			
			// now show the row
			if (rowEmpty === false) {
				
				if (displayRow%2 == 0 || !this.alternateBackground) {
				  htmlcode2 +='<tr>';  
				} else {
				  htmlcode2 +='<tr class="alt">';  
				}
				// paint left end with correct background
				var extraBackground = null;
				var extraHeight = null; // needed for spacers
				var cellObj = this.cells[row][0];
				if (cellObj) {					
					if (cellObj.styleObj && cellObj.styleObj.background) {
						extraBackground = cellObj.styleObj.background;
						if (cellObj.height) {
							extraHeight = cellObj.styleObj.height;	
						}
					} else {
						if (cellObj.background) {
							extraBackground = cellObj.background;	
						}
						if (cellObj.height) {
							extraHeight = cellObj.height;	
						}
					}
				}
				if (!cellObj || cellObj.type != 'HeaderBar') {
					htmlcode2 += '<td class="firstCol"';

					if (extraBackground != null){
						htmlcode2 += ' style="background-color:' + this.fixStrictModeColor(extraBackground) + ';" ';
					}
					htmlcode2 += '>';
					
					if (extraHeight != null) {
						htmlcode2 += '<div style="background-color:' + this.fixStrictModeColor(extraBackground) + '; height:' + extraHeight + 'px;"></div>';					



					}
					htmlcode2 += '</td>';					
				}
				
				for (var col=0; col<this.nCols; col++) {
					if (htmlArr[col]) {
						htmlcode2 += htmlArr[col];						
					}
				}	
				
				// paint right end with correct background
				htmlcode2 += '<td class="lastCol" '; 	

				if (lastBackGround != null) htmlcode2 += ' style="background-color:' + this.fixStrictModeColor(lastBackGround) + ';" ';
				htmlcode2 += '>';				
				if (lastHeight ) {
					htmlcode2 += '<div style="background-color:' + this.fixStrictModeColor(lastBackGround) + '; height:' + lastHeight + 'px;"></div>';					



				}
				htmlcode2 += '</td>';	

				htmlcode2+= '</tr>';
				displayRow++;
			}

            htmlcode += htmlcode2;
        }
		htmlcode += hiddenHTML;  // hidden fields
		htmlcode += '</tbody>';
        htmlcode +='</table>';

        return htmlcode;


    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.displayTable:\n' + err );
    }
};


/** 
* Displays table in WebCenter header look
* supposes to get a OneColumn layout so the rows and columns you see in the header are constructed on the fly
*/
attributeCategory.prototype.displayTableHeader = function() {
    try {
        if (!this.cells) return '';
		//if (this.layout != 'OneColumn') alert("Header view display designed for OneColumn display. Other columns will be disregarded");  // can only happen with categories designed before 6 Sep 2010 so before Beta
        var htmlcode = '<table class="projectInfo">';
		
		// construct a new array with all elements from the header view. This to cater with empty cells. 
		// it is seldomly needed but problems have arrived probably caused by historical bugs of which the resulting view is stored in XML so not easy to clean
		var tempArr = new Array();
		for (var row=0; row< this.nRows; row++) {
			var cellObj = this.cells[row][0];  // only one column is taken into account
			if (cellObj && cellObj.type =='Attribute' ) tempArr.push(cellObj);
		}
		       
		var nCols = Math.floor((tempArr.length-1)/4) +1;
		var nRows = Math.floor((tempArr.length-1)/nCols) + 1;
	   
        for (var row=0; row< nRows; row++) {
            var htmlcode2= ''; // avoid too much appending to the same string, slow in IE
            htmlcode2 +='<tr>';  
                                    
            for (var col=0; col< nCols; col++) {  // columns in the actual header display, not in the header view!!!
				var rowIdx = row + nRows*col;
				if (rowIdx < tempArr.length) {
					var cellObj = tempArr[rowIdx];
								
					var attObj = cellObj.attObj;					   
					if (attObj) {
						htmlcode2 += this.renderAttribute(cellObj,row,col,"ReadOnly",false);
					} 
				}
				                
            }            
            htmlcode2+= '</tr>';

            htmlcode += htmlcode2;

        }

        htmlcode +='</table>';

        return htmlcode;


    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.displayTableHeader:\n' + err );
    }
};



/**
* Creates htmlcode for an attribute field. Like each renderer, it returns a list of <td>'s adding up to the needed column span (as defined in cellObj.colSpan)
* @param cellObj cell object containing the attribute
* @param row col : row and col where the field is
* @param visibility: any of Editable, Required, Hidden (typically not),  HiddenKeepValues, ReadOnly
*/
attributeCategory.prototype.renderAttribute = function(cellObj,row,col, visibility, forPrint) {
	var attObj = null;
    try {
        var htmlcode = '';
        var changeMethod;
        var editable = false;
        if (this.currentUser && this.currentUser.canEditAttributes == true) editable=true;
        if (this.showFakedReadOnly) editable = false;  // header preview in admin page
		if (visibility == "ReadOnly") editable = false;
		var hasButton = false;
		if (cellObj.buttons && cellObj.buttons.length>0) hasButton = true;

        attObj = cellObj.attObj;
        if (!attObj) throw(this.langArr['ACatRenderAttrProblem'] + " attObj");
        
		//cellObj.labelPosition = 'Top';  //TO DO remove after testing
		
		 // determine colspan
        var colSpan = 1;
        if (cellObj.colSpan >1) colSpan =  cellObj.colSpan * 3 - 2 ;
		if (cellObj.labelPosition == 'Top') colSpan += 1;
		
		if (cellObj.labelPosition == 'Top') htmlcode += '<td colspan="'+colSpan+'" ' + this.renderStyle(cellObj.styleObj,"Value") + '><table margin="0" width="100%"><tr>';
		
		
		// RENDER THE LABEL 		
		//////////////////////////////////////////////////////////////////////////////////////		
		 htmlcode += this.renderAttributeLabel(cellObj, row, col, visibility,editable,cellObj.renderOption == 'Images' || cellObj.renderOption == 'RadioCheckboxes');   // always renders a <td>...</td>
		
		if (cellObj.labelPosition == 'Top') htmlcode += '</tr><tr>';
       

        // RENDER THE VALUE     Looks like  <td id="doc1_td_5_5" style="..." colspan="...">...content of a render function...</td>
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        htmlcode += '<td ';
		if (this.currentView != 'Header') htmlcode += ' id="' + this.prefix + 'td_' + row + '_' + col + '"';  
        htmlcode += this.renderStyle(cellObj.styleObj,"Value");

       
       
        if (colSpan >1 && cellObj.labelPosition != 'Top') {
            htmlcode += ' colspan="' + colSpan +  '" ';
        }
        htmlcode += '>';

        if (attObj) {  // test is probably not necessary, you're thrown out earlier if attObj does not exist
            var attValue;
			attValue = attObj.value;

			htmlcode += this.renderHiddenAttributeInfo(attObj);
            
			if (cellObj.renderOption == 'Images' || cellObj.renderOption == 'RadioCheckboxes') {
				htmlcode += this.renderAttributeImages(cellObj,row,col,visibility);
			
			} else if (attObj.rs) { // restricted set
				
				if (attObj.isCheckBox) {
					// render as checkbox 
					htmlcode += this.renderCheckbox(cellObj,row,col,editable);
				} else {
					if (editable) {
						if (attObj.allowMultipleValues) {
							htmlcode += this.renderSelectMultiple(cellObj,row,col,true);
						} else {
							htmlcode += this.renderSelect(cellObj,row,col);
						}
					} else {
						if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
							if (attObj.rs) {
								var attDate = this.correctForDaylightSaving(new Date(parseInt(attValue)));
								
								if (attValue) htmlcode += WCHTML.getDateHourString(attDate);  // in case of restricted set, the attValue is a time stamp, not a Date object!
							} else {
								if (attValue) htmlcode += WCHTML.getDateHourString(attValue);
							}
						} else if(attObj.fractionalIfImperial && this.userUnits && this.userUnits.toLowerCase() == "imperial"){
							if (attValue) htmlcode += WCHTML.decimalToFractional(attValue);
						} else {
							if (attObj.allowMultipleValues && this.currentView != 'Header') {
								htmlcode += this.renderSelectMultiple(cellObj,row,col,false);
							} else {
								htmlcode += attValue;
							}
						}				
					}
				}
				
				
				
			} else if ( attObj.listObj  && (attObj.listObj.loadingType != 'OnTheFly' || attObj.allowMultipleValues)) {
				if (attObj.columnObj && attObj.columnObj.uniqueIdentifyer=='2') {
					htmlcode += this.renderUniqueIdentfyerBox(cellObj,row,col,editable);
				} else if (attObj.allowMultipleValues) {
					htmlcode += this.renderSelectMultiple(cellObj,row,col,editable);
				
				  
				} else {
					htmlcode += this.renderRestrictedList(cellObj,row,col,editable);
				}
			} else if (  attObj.restrictType == 'Attribute' ) {
				if (attObj.allowMultipleValues) {
					htmlcode += this.renderSelectMultiple(cellObj,row,col,editable);
				} else {					
					if (editable) {
						htmlcode += this.renderOtherAttributeSelector(cellObj,row,col);
					} else {
						htmlcode += attValue;
					}
				}
			} else if (cellObj.ruleObj && cellObj.ruleObj.visibilityRestriction && cellObj.ruleObj.visibilityRestriction.restrictedValues) {
				// restriction based on rules --> not released feature
				if (editable) {
					htmlcode += this.renderRestriction(cellObj,row,col);
				} else {
					htmlcode += attValue;
				}
				
			} else if (gTextManagementEnabled &&  attObj.restrictType == 'DocumentLink') {  
				if (cellObj.renderOption == 'Table') {
					htmlcode += this.renderDocumentLinkTable(cellObj,row,col,visibility);						
				} else {
					htmlcode += this.renderDocumentLinkSimple(cellObj,row,col,visibility);			
				}
			} else {  // not restricted or an on-the-fly list column
                           
				if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
                    // DATE TIME

                    var dateString = '';
                    if (attValue.constructor == Date) dateString = WCHTML.getDateString(attValue);

                    if (editable) {
                        // put hidden field for date in format 12/03/2010
                        //var dateName = this.prefix +  attObj.id;
                        //var aDateName = this.prefix + "adate" + attObj.id;
                        //var saveADateName = this.prefix + "save_adate" + attObj.id;
                        var longDateName = this.prefix + "longFormat_adate" + attObj.id;
                        var hourName = this.prefix + "date_period_of_day_" + attObj.id;
						htmlcode += '<table class="inlinecontent"><tr><td ' + this.renderStyle(cellObj.styleObj,"Value") + '  style="padding-left:0">';
                        changeMethod = this.functionPrefix + '.changeDateTime(' + row + ',' + col + ')';
						
						
                        htmlcode += '<input type="text" size="16" onchange="' + changeMethod + '"  value=" ' + dateString + '" ';                        
						htmlcode += this.renderStyle(cellObj.styleObj,"DateField");
                        htmlcode += ' name="'+ longDateName +'" id="'+ longDateName +'" valign="middle"></td>';
                        
						htmlcode += '<td ' + this.renderStyle(cellObj.styleObj,"Value") + '>';
                        htmlcode += '<a target="_self" href="#" onclick="javascript:$(\'#'+longDateName+'\')[0].focus(); return false;" style="vertical-align:middle;"><img id="IMG1" src="images/show-calendar.gif" border="0" width="24" height="15" align="absmiddle"></a></td>';

                        //  hour, uses same changeMethod
                        var halfhours = this.defaultHourAsHalfHours;
                        
                        if (attValue) {
							if (attValue.constructor != Date) throw(this.langArr['ACatAttrDateProblem'] + attValue);
							halfhours = attValue.getHours() * 2 + Math.floor(attValue.getMinutes()/30);
						} else {
							halfhours = -1; // will display as --:--
						}
						htmlcode += '<td ' + this.renderStyle(cellObj.styleObj,"Value") + '>';
                        htmlcode +=  this.displayHourDropdown(halfhours,hourName,hourName,changeMethod,cellObj.styleObj) + '</td></tr></table>';

                    } else {
                        if (!this.showFakedReadOnly) {
                            if (attValue) {
								htmlcode += WCHTML.getDateHourString(attValue);	
								// DT110156: hour was not submitted so you fell back on 0:00. Date is submitted because you called renderHiddenAttributeInfo already
								var theName2 = this.prefix + 'date_period_of_day_' + attObj.id;
								var attValue2 = attValue.getHours() * 2 + Math.floor(attValue.getMinutes()/30);	
								htmlcode += '<input type="hidden" name="'+theName2+'" value="'+attValue2+'">';			
							}		
							
                        } else {
                            htmlcode += 'May, 24 2010 18:00';
                        }
                    }

                } else if(attObj.type && attObj.type.toLowerCase() == 'richtext'){
                	
                	var rowCnt = (cellObj.rowSize)? cellObj.rowSize : 120;
                	var theName = this.prefix + attObj.id;
                	
                	if (editable) {
						changeMethod = this.functionPrefix + '.changeText(this,' + row + ',' + col+')';
						
						htmlcode += '<textarea style="width: 100%;height: ' + rowCnt + 'px;" type="text" id="' + theName + '" name="' + theName + '" onchange="' + changeMethod + '" ';
						htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton);
						htmlcode += '>'+ WCXML.xmlencode(attValue)+'</textArea>';
						htmlcode += '<a class="darktext_greenlink_12" href="#" onclick="javascript:edittmce(\''+theName+'\'); return false;">'+this.langArr['ACatShowAdvancedControls']+'</a>';
	
						if(!tinyMceMgrObj.tmceEditors[this.id]) tinyMceMgrObj.tmceEditors[this.id] = new Array();
						var found = false;
						for(var i=0 ; i<tinyMceMgrObj.tmceEditors[this.id].length ; i++){
							if(tinyMceMgrObj.tmceEditors[this.id][i]==theName){
								found = true;
							}
						}
						if(!found) tinyMceMgrObj.tmceEditors[this.id].push(theName);
						tinyMceMgrObj.rows[attObj.id] = row;
						tinyMceMgrObj.cols[attObj.id] = col;
                	
                	}else{
                		if (!this.showFakedReadOnly) {
                			var divheight = rowCnt;
                			var value = attValue;
                			
	                		htmlcode += '<div class="ReadOnlyRichText" style="width: 100%;';
	                		if (!this.printLayout) {
	                			htmlcode += ' max-height: ' + divheight + 'px; ';
		                		if (this.currentView != 'Header')  htmlcode +=  'height: ' + divheight + 'px;';
		                		else  htmlcode +=  'height: expression( this.scrollHeight > '+(divheight-1)+' ? \''+divheight+'px\' : \'auto\' );';
	                		}
	                		htmlcode += '">';
	                    	htmlcode += value;
	                    	htmlcode += "</div>";
                    	}else{
                    		htmlcode += '<div style="width: 100%; max-height: 120px; min-height: 16px; overflow: auto; text-align: left; margin: 0px 0px 0px 0px; ">Ille ego qui <b>fuerim</b>, tenerorum lusor <i>amorum</i>, <br>quem legis, ut noris, <u>accipe posteritas</u>.</div>';
                    	}
                	}
                	
                } else if(attObj.fractionalIfImperial && this.userUnits && this.userUnits.toLowerCase() == "imperial"){
                	attValue = WCXML.xmlencode(attValue);
                	var displayValue = attValue;
                	if (displayValue) {
                		displayValue = WCHTML.getFloatString(attValue,6);
                		displayValue = WCHTML.decimalToFractional(displayValue);
                	}
                	
                	if (editable) {                        
						var theName = this.prefix + attObj.id;
						changeMethod = this.functionPrefix + '.changeText(this,' + row + ',' + col+')';
						
						// field for fractional value
						htmlcode += '<input type="text" value="' + displayValue + '" id="' + theName + '_disp" name="' + theName + '_disp" onchange="' + changeMethod + '" ';						
						htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton);
						htmlcode += '>';
						
						// hidden field with real value
						htmlcode += '<input type="hidden" value="' + attValue + '" id="' + theName + '" name="' + theName + '">';

                    } else { // not editable
                        if (!this.showFakedReadOnly) {
                            htmlcode += displayValue;                            
                        } else {
							htmlcode += '1 + 1/4';
						}
                    }
                	
				} else {
					// ALL THE OTHERS: units, single line text, text area, Float ...
                	
					attValue = WCXML.xmlencode(attValue);
					var displayValue = attValue;
					var type2;  
					type2 = attObj.type;
					if (type2) {
						type2 = type2.toLowerCase();
					} else {
						type2 = 'text';
					}
					if (attValue && type2 != 'integer' && type2 != 'number' && type2!= 'text' && type2 != 'float') displayValue = WCHTML.getFloatString(attValue,6);
					if (attValue && type2 == 'float') displayValue = parseFloat(attValue).toString();  // normalized display of floats - DT119585, this will make 25.00000 show as 25 while not changing the actual value, see DevTrack for advanced reasoning.
					
					var theName = this.prefix + attObj.id;
					if (editable) {                        
						changeMethod = this.functionPrefix + '.changeText(this,' + row + ',' + col+')';

						if(cellObj.rowSize && parseInt(cellObj.rowSize) > 1) {
							htmlcode += '<textarea id="' + theName + '" name="' + theName + '" class="' + this.valueClass + '" rows="' + cellObj.rowSize + '" onchange="' + changeMethod + '" ';
							//if (cellObj.background) htmlcode += 'background-color:' + this.fixStrictModeColor(cellObj.background) + ';';
							htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton);
							htmlcode += ' >';
							htmlcode += attValue;
							htmlcode += '</textarea>';
						} else {
							var sDropStyle = false;
							if (attObj && attObj.listObj && attObj.listObj.loadingType == 'OnTheFly') sDropStyle = true;
							
							if (sDropStyle) htmlcode += '<span class="nobr listselector">';
							htmlcode += '<input type="text" value="' + displayValue + '" id="' + theName + '" name="' + theName + '" onchange="' + changeMethod + '" ';
							htmlcode += this.renderStyle(cellObj.styleObj,"InputField",0,hasButton || sDropStyle);
							if (sDropStyle) {
								var keyUpMethod = this.functionPrefix + '.getDBValues(this,event,' + row + ',' + col+ ')';
								htmlcode += ' onkeyup="'+keyUpMethod+'" ';
								htmlcode += ' autocomplete = "off" '; 
							}
							htmlcode += '>';
							if (sDropStyle) {
								var clickMethod = this.functionPrefix + '.getDBValues(this,event,' + row + ',' + col+ ')';
								if (attObj.allowMultipleValues) {
									htmlcode += '<img src = "./images/ar_tp.gif" onclick = "'+clickMethod+'">';
								} else {
									htmlcode += '<img src = "./images/dropdown_icon.gif" onclick = "'+clickMethod+'">';
								}
								
								htmlcode += '</span>';
							}
								
						}
                                                        
                        
                    } else { // not editable
                        if (!this.showFakedReadOnly) {
                        
                            htmlcode += displayValue;                            
                            htmlcode += '<input type="hidden" value="' + displayValue + '" id="' + theName + '" name="' + theName + '" />';	
                            
                        } else {
                            if (cellObj.rowSize && parseInt(cellObj.rowSize) > 1) {
								// this probably never happens since we don't allow rowsize anymore for header view
								htmlcode += ' <div align="left">Ille ego qui fuerim, tenerorum lusor amorum, <br>quem legis, ut noris, accipe posteritas.</div>';
							} else {
								htmlcode += 'Esko';
							}
						}
                    }
                    
                }
			}
				
			/*
			// show a * if changed
			if (attObj.changed===true) {
				htmlcode += '*';
			}
			*/
		}
		
		
		if (this.currentView != 'Header') {  // units are in header view stuck immediately after the value
			if (cellObj.buttons &&  (this.currentUser && this.currentUser.canEditAttributes)) htmlcode += '&emsp;'+  this.renderButton(cellObj,row,col,true, forPrint);
			htmlcode += '</td>';
		} else {
			//htmlcode += '&nbsp;'; // is there a reason for this space? removed it because rich text attributes 
									// in header looked ugly, as it started a new line for the space
		}
        
		if (cellObj.labelPosition == 'Top') htmlcode += '</tr></table></td>';
		
		// RENDER UNIT COLUMN
		/////////////////////////////////////////////////////////////////////////
		var styleObj = cellObj.styleObj;
		if (styleObj && styleObj.valueStyleObj) styleObj = styleObj.valueStyleObj;
		var paddingRight = 15;
		if (this.nCols >3) paddingRight = Math.round(45/this.nCols);
        if (this.currentView != 'Header') {
			htmlcode += '<td ' + this.renderStyle(styleObj,"Unit",paddingRight) + ' >'; // large padding right creates the effect that maximum space is put here
		}
		
		if (attObj && (this.currentView != 'Header' || (attObj.value!="" && attObj.value!=null)) ) {  // don't show units in header when there is no value
			if (attObj.unitLocalized) {
				htmlcode += '<nobr>'+ attObj.unitLocalized + '</nobr>';
			} else if (this.showTypeAsUnit) {
				if (attObj.rs) {
					htmlcode += '<nobr>'+ attObj.rs.name + '</nobr>';
				} else if (attObj.typeLocalized) {
					htmlcode += '<nobr>'+ attObj.typeLocalized + '</nobr>';
				} else if (attObj.unit) {
					htmlcode += '<nobr>'+ attObj.unit + '</nobr>';				
				} else if (attObj.type) {
					// DT125977: you typically get here in the admin page for just placed attributes. For these, typeLocalized is not known yet and unit neither --> just put type (while previously nothing was put)
					htmlcode += '<nobr>'+ attObj.type + '</nobr>';  
				}
			}
		}
		
		
        htmlcode += '</td>';
        return htmlcode;
    } catch(err) {
		var msg = "";
		if (attObj) msg += "\n attObj.name: " + attObj.name + "\n attObj.type: " + attObj.type;
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderAttribute at line : ' + err.lineNumber + ": " +  err.message + msg);
		return '<td class="darktext_greenlink_11">Missing attribute</td><td></td><td></td>';
    }
};

/**
* renders the label of the attribute and the Select All, Deselect All and Show All actions in case of attribute images
* @param {Object} cellObj - cellObj for this attribute, remark you cannot reconstruct this from row,col in case of header view (where the cells are onecolumn layout)
*/
attributeCategory.prototype.renderAttributeLabel = function(cellObj,row, col, visibility,editable, showActions,colSpan) {
	var htmlcode = '';
	try {

		var attObj = cellObj.attObj;
		if (!cellObj || !attObj) throw new Error("cellObj or attObj is null for row " + row + "  col " + col);
		
		var label = cellObj.label != null ? cellObj.label : cellObj.name; // distinguish null from ''; '' should show empty label, null should show name
		htmlcode += '<td ';
		htmlcode += this.renderStyle(cellObj.styleObj,"Label");
		if (cellObj.labelImage)  htmlcode += ' align="center" ';
		if (attObj.name != label) htmlcode += ' title="'+WCXML.xmlencode(cellObj.attObj.name)+'" ';  // tooltip with the real attribute name
		if (colSpan) htmlcode += '  colspan="' + colSpan + '"'; 
		
		htmlcode += '><label for="' + attObj.id + '">';
		
		if (cellObj.labelImage) {
			htmlcode += '<img src="./custom/AttributeLabelImages/'+cellObj.labelImage+'" alt="'+label+'" title="'+label+'">';
		} else {
			
			if (this.currentView == 'Header') {
				var defaultLabel = this.findHeaderLabel(attObj);  // look for the same attribute in the default view
				if (defaultLabel) label = defaultLabel;
			}
			htmlcode += WCXML.xmlencode(label);
			
			if (this.currentView == 'Header') htmlcode += ':';  //Add a colon after the label
		}
		// Red asterisk if "Required"        
		if (visibility == 'Required'  && editable) htmlcode += '<font color="red">*</font>';  

		htmlcode += '</label>';
		
		// Warning if value will overwrite project attribute value        
		if (attObj.projAttValue && attObj.projAttValue != "" && editable) {
			var clickMethod = this.functionPrefix + '.recoverProjectAttributeValue('+row+','+col+'); return false';
			
			htmlcode += '<div  id="warning'+attObj.id+'" style="visibility:hidden;display:none"><div class="darktext_darklink_11" style="background-color:#FFFFDD">';
			htmlcode += '<font color="red">'+this.langArr['ACatProjectAttributeOverwriteWarning']+'</font>';
			htmlcode += '<br><a class="darktext_darklink_11" href="#" onclick="' + clickMethod +'">'+this.langArr['ACatLoadCurrentProjectAttributeValue']+'</a>';
			htmlcode += '</div></div>'; // double div to shown the innerHTML of the outer one in a popup

			var hoverMethod = 'try{delayshowpopupbyid(event, \'warning'+attObj.id+'\', 300)}catch(err){}';
			var clickMethod2 = 'try{showpopupbyid(event, \'warning'+attObj.id+'\', 300);}catch(err){}';

			// only display when there's a difference between the values
			htmlcode += '<img id="imgwarning'+attObj.id+'" src="images/warning.png"';
			if (attObj.type && attObj.type.toLowerCase() == 'datetime' && !attObj.rs){
				if (attObj.projAttValue.toString() == attObj.value.toString()){
					htmlcode += 'style="visibility:hidden;display:none"';
				}
			}else{
				if (attObj.projAttValue == attObj.value ){
					htmlcode += 'style="visibility:hidden;display:none"';
				}
			}
			
			htmlcode += ' onmouseover="' + hoverMethod +'" onclick="' + clickMethod2 +'" onmouseout="delayhidethumb()">';
		}	

		if (cellObj && cellObj.infoPage) {		
			var clickMethod = 'popw=window.open(&quot;./custom/AttributesHelp/'+  WCXML.xmlencode(cellObj.infoPage) + '&quot;,&quot;Info&quot;,' + this.helpOptionsString + ');popw.focus();';
			//var clickMethod = 'javascript:void( popw=window.open(&quot;./custom/AttributesHelp/'+ cellObj.infoPage + '&quot;,&quot;Info&quot;,' + this.helpOptionsString + ' );popw.focus();)'
			htmlcode += '&nbsp;&nbsp;<img src="images/help_icon_16.gif" onclick="' + clickMethod +'">';
		}
		
		
		if (showActions && visibility != 'ReadOnly' && editable) {
			if (attObj.allowMultipleValues) {
				clickMethod = this.functionPrefix + '.selectAllMultiValue(\''+ this.prefix + 'ImgChk_' + attObj.id +'\', '+row+', '+ col+'); return false; ';
				htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatSelectAll']+'</a>';
				clickMethod = this.functionPrefix + '.deselectAllMultiValue(\''+ this.prefix + 'ImgChk_' + attObj.id +'\', '+row+', '+ col+'); return false; ';
				htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatDeselectAll']+'</a>';
				if (attObj.columnObj) {
					if (attObj.showAll) {
						clickMethod = this.functionPrefix + '.showAllImages('+row+', '+ col+',\'' + visibility + '\',false); return false; ';
						htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatShowAllowed']+'</a>';								
					} else {
						clickMethod = this.functionPrefix + '.showAllImages('+row+', '+ col+',\'' + visibility + '\',true); return false; ';
						htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatShowUnfiltered']+'</a>';		
					}
				}					
			} else {
				var clickMethod = this.functionPrefix + '.deselectAllSingle(\''+ this.prefix + attObj.id +'\','+row+','+ col+'); return false; ';
				htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatDeselect']+'</a>';
			}
		}

		
		htmlcode += '</td>';
	} catch(err) {
		alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderAttributeLabel at line : ' + err.lineNumber + ": " +  err.message );
	}
	return htmlcode;
}

attributeCategory.prototype.findHeaderLabel = function(attObj) {
	try {
		if (!this.views) return null;
		var cells = this.views[0].cells; 
		for (var row=0; row<cells.length; row++) {
			var rowArr = cells[row];
			if (rowArr) {
				for (var col=0; col<rowArr.length; col++) {
					if (rowArr[col] && rowArr[col].attObj== attObj) {		
						return rowArr[col].label != null ? rowArr[col].label : rowArr[col].name; // distinguish null from ''; '' should show empty label, null should show name
					}
				}
			}
		
		}
		return null;
	
	} catch(err) {
		// do nothing
		return null;
	}

};

/**
* @param leftCorner determines whether you need to put a left corner cell
* @param rightCorner determines whether you need to put a right corner cell
*/
attributeCategory.prototype.renderHeaderBar = function(colSpan,leftCorner,rightCorner) {
    try {
        var htmlcode = '';
		
		if (!colSpan) colSpan=1;
					
			//if (leftCorner) htmlcode += '<th class="leftCorner"/>';
			for (var col=0; col<colSpan; col++) {
				if (col==0 && leftCorner) {
					htmlcode += '<td align="left" class="attributesGridLayoutHeader" colspan="2">&nbsp;&nbsp;' + this.langArr['ACatName']+ '</td>';
				} else {
					htmlcode += '<td align="left" class="attributesGridLayoutHeader">' + this.langArr['ACatName']+ '</td>';					
				}
				htmlcode += '<td align="left" class="attributesGridLayoutHeader">' + this.langArr['ACatValue']+ '</td>';
				if (col == colSpan-1 && rightCorner) {
					if (this.showTypeAsUnit) {
						htmlcode += '<td class="attributesGridLayoutHeader" colspan="2" align="left">' + this.langArr['ACatType'] + '</td>';
					} else {
						htmlcode += '<td class="attributesGridLayoutHeader" colspan="2" align="left">' + this.langArr['ACatUnit'] + '</td>';
					}
				} else {
					if (this.showTypeAsUnit) {
						htmlcode += '<td align="left" class="attributesGridLayoutHeader">' + this.langArr['ACatType'] + '</td>';
					} else {
						htmlcode += '<td align="left" class="attributesGridLayoutHeader">' + this.langArr['ACatUnit'] + '</td>';
					}
				}
			}
			//if (rightCorner) htmlcode += '<td class="rightCorner"/>';
			
	
        return htmlcode;
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderHeaderBar:\n' + err);
    }
};

/**
* This function creates a calendar object.
* @param  idName id of the input field
* @param  idHour id of the selection of period in the day
*/
attributeCategory.prototype.createCalendar = function(idName, idHour) {
    if ($(idName) !=null )
    {

        $(idName).datepicker(
        {
            dateFormat: "M dd, yy",
            onClose: function(dateText, inst)
            {
				/*
                var element = $(idHour)[0];
                if(element && element.selectedIndex == 0) element.selectedIndex = (36 + 1);
				*/
            }
        });


    }
};

/**
* creates HTML code for a dropdown box for restricted sets
* @param cellObj: cell containing the attribute with restricted set
* @param row: row index of the cell
* @param col: col index of the cell
*/
attributeCategory.prototype.renderSelect = function(cellObj,row,col) {
    
    var htmlcode = '';
    try {
        var rsObj = cellObj.attObj.rs;
        if (!rsObj || !rsObj.values) return '';
        var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';
        var theName = this.prefix  + cellObj.attObj.id;

    	var type2 = cellObj.attObj.type;
		if (type2) {
			type2 = type2.toLowerCase();
		} else {
			type2 = 'text';
		}
        var attValue = cellObj.attObj.value;
        if (attValue && type2 != 'integer' && type2 != 'number' && type2 != 'text' && type2 != 'float' && type2 != 'datetime') attValue = WCHTML.getFloatString(attValue,6);
		if (attValue && type2 == 'float') attValue = parseFloat(attValue).toString();  

        htmlcode += '<select onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
        htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");
        
        htmlcode += '>';
        htmlcode += '<option value="">---' + this.langArr['ACatPleaseSelect'] + '---</option>';
		var found = false; // did we find the selected value in the list of options
        for (var option=0; option < rsObj.values.length; option++) {
            var value = rsObj.values[option].value;
			var valueDisplay = value;
			if (cellObj.attObj.type.toLowerCase() == 'datetime') valueDisplay= WCHTML.getDateHourString(this.correctForDaylightSaving(new Date(parseInt(value))));		
			if(cellObj.fractionalIfImperial && this.userUnits && this.userUnits.toLowerCase() == "imperial"){
				valueDisplay = WCHTML.decimalToFractional(value);
			}
            htmlcode += '<option value="'+ WCXML.xmlencode(value) +'"';
            if (attValue == value) {htmlcode += ' selected '; found=true;};
            htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
        }
		if (cellObj.attObj.value && !found) {
			var value = cellObj.attObj.value;
			var valueDisplay = value;
			
			if (cellObj.attObj.type.toLowerCase() == 'datetime') valueDisplay= WCHTML.getDateHourString(this.correctForDaylightSaving(new Date(parseInt(value))));		
			htmlcode += '<option value="'+ WCXML.xmlencode(value) +'" selected ';            
            htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + this.langArr['ACatDeleted'] + ')</option>';
		}
        htmlcode += '</select>';
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderSelect:\n' + err);
    }
    return htmlcode;
};

attributeCategory.prototype.correctAttributeRestricted = function(attObj) {
    try {
		var currentValueArr = attObj.value.split(this.multiSep);
		var attObj2 = attObj.restrictToAttObj;
		var newValue = '';
		if (attObj2) {
			currentValueArr2 = attObj2.value.split(this.multiSep);
			for (idx = 0 ; idx<currentValueArr.length; idx++) {
				for (idx2 = 0 ; idx2<currentValueArr2.length; idx2++) {
					if (currentValueArr[idx] == currentValueArr2[idx2]) break;
				}
				if (idx2 < currentValueArr2.length) {
					if (newValue != '') newValue += this.multiSep;
					newValue += currentValueArr[idx];
				}
			}
		
		}
		if (newValue != attObj.value) {
			attObj.value = newValue;
			// To do: figure out what more must be done here
		}
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderSelect:\n' + err);
    }

}

/**
* creates HTML code for a dropdown box for restricted sets
* @param cellObj: cell containing the attribute with restricted set
* @param row: row index of the cell
* @param col: col index of the cell
*/
attributeCategory.prototype.renderSelectMultiple = function(cellObj,row,col,editable) {
    var htmlcode = '';
    try {
		var attObj = cellObj.attObj;    
		var listObj = attObj.listObj;
		var valid;
		
		
		if (attObj.restrictType == 'Attribute') {
			this.correctAttributeRestricted(attObj);    // throw out all values which are no longer allowed. TO DO : move to where the change is actually done instead of doing this each time a repaint is done
			if (!attObj.restrictToAttObj || attObj.restrictToAttObj.value=='') return '';  // show nothing in case of attribute restriction with an empty father attributes					
		}
		
		if (this.currentView != 'Header') {
		
			var currentValueArr = attObj.value.split(this.multiSep);	
			var currentIndexArr = null;
			var clickMethod = this.functionPrefix + '.showMultiValueDropdown(this,event,'+row+','+col+'); ';
			if (listObj && listObj.loadingType == 'OnTheFly') clickMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+col+'); ';
			try {
				if (attObj.columnObj) {
					if (!attObj.columnObj.validArr || (attObj.listObj && attObj.listObj.updateValuesNeeded) ) this.updateListAttributeValues(attObj.listObj, false);
					currentIndexArr = attObj.columnObj.currentIndex.split('~');
				}
			} catch(err) {
				// any reason why currentIndexArr cannot be calculated, for example it's not a list or it has no value, or value -1 (an integer) etc
				// do nothing --> no currentIndexArr --> always valid
			}
			htmlcode += '<table valign="top" padding="0">';
			
			
			
			htmlcode += '<tr padding="0">';
			htmlcode += '<td style="padding:0px;'+((cellObj.styleObj && cellObj.styleObj.background)?'background-color:'+this.fixStrictModeColor(cellObj.styleObj.background)+';':'') + '">';
			
			htmlcode += '<div id="mvdiv_'+ this.prefix + attObj.id+'" style="vertical-align:top; padding: 0 0 0 0;';
			if (editable || currentValueArr.length>4){
				htmlcode += 'border:1px solid grey;min-width:160px;background-color:white;';
			}
			if (currentValueArr.length>4 && !this.printLayout) htmlcode+='height:80px; overflow-y:auto;';
			htmlcode+= '"  onclick="'+clickMethod+'"><table padding="0">';
			
			var fieldType = 'MultiValue';
			if(!editable){
				fieldType = 'Value';
			}
			
			if (attObj.value != "") {	
				for (var idx=0; idx< currentValueArr.length; idx++) {
					//valid = currentIndexArr? (attObj.columnObj.validArr[currentIndexArr[idx]]) : true;
					
					if (currentIndexArr && attObj.columnObj.validArr) {
						valid = attObj.columnObj.validArr[currentIndexArr[idx]];
					} else {
						valid=true;
						
					}
					
					
					if (gTextManagementEnabled && attObj.restrictType == 'DocumentLink') {
						htmlcode += '<tr><td class="darktext_greenlink_11" ' + this.renderStyle(cellObj.styleObj,fieldType) + '><nobr>'+WCXML.xmlencode(currentValueArr[idx])+'&nbsp;';
					} else {
						htmlcode += '<tr><td class="darktext_greenlink_11" ' + this.renderStyle(cellObj.styleObj,fieldType) + '><nobr>'+WCXML.xmlencode(currentValueArr[idx])+'&nbsp;';
					}
					if (!valid) htmlcode += WCXML.xmlencode(this.langArr['ACatNotValid']);
					htmlcode += '</nobr></td></tr>';  // the &nbsp; is needed for the case of empty (attObj.value = " "). 
				}
			} else if (editable) {
				htmlcode += '<tr><td class="darktext_greenlink_11"  ' + this.renderStyle(cellObj.styleObj,fieldType) + '><nobr>---'+this. langArr['ACatPleaseSelect']+'---</nobr></td></tr>';	
			}
			htmlcode += '</table></div></td>';
			
			
				
			if (editable) {
				var theName = this.prefix  + attObj.id;
				// drop icon
				
				htmlcode += '<td style="padding:0 0 0 0; vertical-align:top; '+((cellObj.styleObj && cellObj.styleObj.background)?'background-color:'+this.fixStrictModeColor(cellObj.styleObj.background)+';':'') + '"><img id="'+theName+'_popUpIcon" src="./images/ar_tp.gif"  onclick="'+clickMethod+'"></td>';
				
				
				// hidden element
				htmlcode += '<input type="hidden" id="' + theName +'" name="' + theName +'" value="'+WCXML.xmlencode(attObj.value)+'">';
			}
			htmlcode += '</tr></table>';
		} else {
			htmlcode += WCXML.xmlencode(attObj.value);
		}
		
		
		
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderSelectMultiple:\n' + err);
    }
    return htmlcode;
};


/**
* opens a pop-up with the values of a restricted set or a list
*/
attributeCategory.prototype.showMultiValueDropdown = function(parentObj,event,row,col,showAll) {
	try {
		var htmlcode = '';
		// find the values
		var cellObj = this.cells[row][col];		// should always exist if we get here
		var isMobileMultiValue = WCHTML.isMobile();  // when you are here, you are always multi-value
		
		if(this.checkVisibility(cellObj) != "ReadOnly"){
			var attObj = cellObj.attObj;
			if (isMobileMultiValue) htmlcode += '<div id="dbValuesDivAround" style = "height:250px; overflow-y:auto;">';
			htmlcode += '<table id="multiValueDropdownTable">';
			var clickMethod = this.functionPrefix + '.selectAllMultiValue(); return false; ';
			htmlcode += '<tr><td class="darktext_greenlink_11" ><a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatSelectAll']+'</a></td></tr>';
			var clickMethod = this.functionPrefix + '.deselectAllMultiValue(); return false; ';
			htmlcode += '<tr><td class="darktext_greenlink_11"><a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatDeselectAll']+'</a></td></tr>';
			var clickMethod = this.functionPrefix + '.showAllMultiValue(\''+attObj.id+'\'); return false; ';
			htmlcode += '<tr><td class="darktext_greenlink_11"><a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatShowUnfiltered']+'</a></td></tr>';
			
			htmlcode += '<tr><td class="darktext_greenlink_11" id="multiValueDropdownTableInner">';
			htmlcode += this.multiValueDropdownHTML(attObj,cellObj);
			htmlcode += '</td></tr>';
			
			htmlcode += '</table>';
			if (isMobileMultiValue) { // add an OK button
				htmlcode += '</div>';
				htmlcode += '<div style="width: 95%; text-align: center; padding: 6px;"><input type="button" class="buttonsecondary" value="OK" onclick="hidemenu()"></div>';
			}

			var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
			menuItems.push(htmlcode);	
			var menuWidth = '250px', menuHeight = '300px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=true;
			dropdownmenu(parentObj,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;	
			var aCatObj = this;  // closure
			
			dropmenuobj.onchange = function(event) {
				if (attObj.type == 'documentlink') {
					aCatObj.changeDocumentLink( document.getElementsByName('dropmenudiv'),row,col,true);
				} else {
					aCatObj.changeSelectMultiple(row,col); 			
				}
			};
			
			// prevent the internal of the dropdown menu to trigger the onmouseout/onchange of the enclosing div by giving it its own onmouseout which stops the event propagation
			document.getElementById("multiValueDropdownTable").onchange = function(event){ if (event) event.stopPropagation();};
		}
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.showMultiValueDropdown' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }


};


/**
* opens a pop-up with the values of a restricted set or a list
*/
attributeCategory.prototype.showSearchDialog = function(parentObj,event,row,col,showAll) {
	try {
		var htmlcode = '';
		// find the values
		var cellObj = this.cells[row][col];		// should always exist if we get here
		
		if(this.checkVisibility(cellObj) != "ReadOnly"){
			var attObj = cellObj.attObj;
			htmlcode += '<div id="SearchResultsDiv">';
						
			
			
			htmlcode += '</div>';

			var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
			menuItems.push(htmlcode);	
			var menuWidth = '700px', menuHeight = '500px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=true;
			dropdownmenu(parentObj,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;	
			var aCatObj = this;  // closure
			
			
			document.getElementById("SearchResultsDiv").onchange = function(event){ if (event) event.stopPropagation();};
		}
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.showSearchDialog' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }


};


/**
* Called when selection of a multi-value attribute is changed
* @param row {int} row of the cell containing the multivalue attribute
* @param col {int} column of the cell containing the multivalue attribute
* @param imageList {Boolean} indicates that this function is called from an image list with checkboxes
*/
attributeCategory.prototype.changeSelectMultiple = function(row,col,imageList) {
	try {
		
		var cellObj = this.cells[row][col];		// should always exist if we get here
		if (!cellObj) return;
		var attObj = cellObj.attObj;  if (!attObj) return;
		var restrictedChanged=false;  // remembers whether a part of a cascading dropdown was changed
		
		if (attObj.listObj && attObj.listObj.loadingType == 'OnTheFly' && dropmenuobj && dropmenuobj.style.visibility != 'hidden') return;
		
		// figure out what was selected
		var newValue = '';
		var checkArr;
		if (imageList) {

			checkArr = document.getElementsByName(this.prefix + 'ImgChk_' + attObj.id);	
		} else {
			checkArr = document.getElementsByName('MultiValueSelect');
		}
		if (checkArr && checkArr.length>0) {				
			for (var idx=0; idx<checkArr.length; idx++) {
				if (checkArr[idx].checked) {
					if (newValue) newValue += this.multiSep;						
					newValue += checkArr[idx].value;							
				}
			}
		}	
				
		// see whether the value has changed
		if (newValue != attObj.value) {			
			if (attObj.listObj && attObj.listObj.loadingType == 'Immediate' ) {
				this.changeListSelection(attObj,newValue,document.getElementById('multiValueDropdownTable'),row,col);
				restrictedChanged = true;
			}
			
			
			attObj.value = newValue;
			if (attObj.columnObj && attObj.columnObj.uniqueIdentifyer) attObj.combiIdx = this.findCombiIdx(attObj.listObj, attObj.columnObj, -1, attObj.value);
			
			if (imageList) {
				document.getElementById(this.prefix + attObj.id).value = newValue;
			} else {
				// repaint the element itself
				document.getElementById(this.prefix + 'td_'+row+'_'+col).innerHTML =  this.renderHiddenAttributeInfo(attObj) + this.renderSelectMultiple(cellObj,row,col,true);				
			}

			
			var elem = document.getElementById(this.prefix  + attObj.id);
			if (elem) this.changeText(elem,row,col);  // typically not called for a table based document link
			
			if (attObj.listObj && attObj.listObj.loadingType == 'OnTheFly' ) this.checkAutoSelectLazyLoading(attObj,false);  // auto-selects single remaining
			
			this.executeButtonDependencies(attObj);
			
			
			// check if a global repaint is needed
			if (restrictedChanged) {
				this.displayAttributeCategory(null);  // always repaint if this is a dropdown which is part of a list
			} else {
				this.checkRepaintAttributeCategory(attObj);   // only repaint if a rule is applied to this cell
			}
			
		}
			
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.changeSelectMultiple' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
	
}

/**
* should only be called for unique identifyer columns
* determines the current indices in the combination array of the list based on the value provided for the unique identifyer
*/
attributeCategory.prototype.findCombiIdx = function(listObj, columnObj, currentIndex, value) {
	var foundIdx = '';
	try {
		if (currentIndex == -1) currentIndex = this.findColumnIndex(columnObj, value);
		if (currentIndex == -1) return '';
		
		var idxArr = currentIndex.split("~");
		
		
		// find which column this is about
		for (var column=0; column<listObj.columns.length;column++) {
			if (columnObj == listObj.columns[column]) break;
		}
				
		for (var idx=0; idx<idxArr.length; idx++) {	
			for (var combi=0; combi<listObj.combinations.length; combi++) {
				combiString = listObj.combinations[combi];
				idxArr2 = combiString.split("~");
				thisColIdx = idxArr2[column];
				if (thisColIdx == idxArr[idx]) {
					if (foundIdx) foundIdx += '~';
					foundIdx += combi;
					break;
				}
			}
			
		
		}
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.findCombiIdx' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
	return foundIdx;
}

/**
* creates HTML code to be put in the dropdownmenu having all items. 
* The invalid items are still painted but with display:none, allowing a show all option to just make them visibly by changing the visibility of the rows
*/
attributeCategory.prototype.multiValueDropdownHTML = function(attObj,cellObj) {
	var htmlcode='';
	try {
		var currentValueArr = attObj.value.split(this.multiSep);
		var valueArr;
		var controlType = 'checkbox';
		
		
		htmlcode += '<table>';				
		if (attObj) {
			var rsObj = attObj.rs;
			var columnObj = attObj.columnObj;			
			var rsType = attObj.restrictType;
			
			
			switch (rsType) {
				case 'RestrictedSet':
					if (rsObj) valueArr = rsObj.values;
					break;
				case 'List':
					if (attObj.columnObj) valueArr = columnObj.values;
					break;
				case 'Attribute':
					var attObj2 = attObj.restrictToAttObj;
					if (attObj2) valueArr = attObj2.value.split(this.multiSep);
					break;
				case 'DocumentLink':
					valueArr = cellObj.docList;
					if (!attObj.allowMultipleValues) controlType = 'radio';
					break;
			}

			if (valueArr) {
				for (var idx=0;idx<valueArr.length;idx++) {
					var curVal = (rsType == 'RestrictedSet') ? valueArr[idx].value : valueArr[idx];
					if (!curVal) continue; //don't show empty strings
					// figure out whether this value is currently selected
					var displayVal =  curVal;
					if (rsType == 'DocumentLink') {displayVal =  WCXML.xmlencode(valueArr[idx].name); curVal = valueArr[idx].versionId};
					var selected = false;
					for (var idx2=0 ; idx2<currentValueArr.length; idx2++) {
						if (currentValueArr[idx2] == curVal) {selected=true; break;};
					}
						
					var showItalic = false;
					var showInvalid = false;
					if (columnObj && columnObj.validArr && !columnObj.validArr[idx]) {
						if (selected) showInvalid=true;						
						showItalic = true;
					}
					
					
					curVal = WCXML.xmlencode(curVal);

					
					htmlcode += '<tr class="darktext_greenlink_11" name="MultiValueSelectTr" ';
					if (showItalic) htmlcode += 'style="display:none;" ';
					htmlcode += '><td><input type="'+controlType+'" id="mvchk_' + idx + '" value="'+curVal+'" name="MultiValueSelect"';
					if (selected) htmlcode += ' checked="checked" ';						
					htmlcode += '></td>';
					htmlcode += '<td>';
					if (showItalic) htmlcode += ' <i><font color="#c0c0c0">';
					htmlcode += '<label for="mvchk_' + idx + '">'+ displayVal +'</label>';						
					if (showItalic) htmlcode += ' </font></i>';
					if (showInvalid) htmlcode += '&nbsp;'+ WCXML.xmlencode(this.langArr['ACatNotValid']) ;
					htmlcode += '</td></tr>';
					if (rsType == 'DocumentLink') {						
						htmlcode += '<tr><td/><td id="popUpDetails_'+idx+'"></tr>';						
					}

				
				}
			
			} 
		}
		htmlcode += '</table>';
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.multiValueDropdownHTML' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }	
	return htmlcode;

};

attributeCategory.prototype.selectAllMultiValue = function(elemName,row,col) {
	try {
		var checkArr;
		if (elemName) {
			checkArr = document.getElementsByName(elemName);
		} else {
			checkArr = document.getElementsByName('MultiValueSelect');
		}
		if (checkArr && checkArr.length>0) {
			for (var idx=0; idx<checkArr.length; idx++) {
				var checkElem = checkArr[idx];
				var trElem = checkElem.parentNode.parentNode;
				if (trElem.style.display != 'none') {  // only select what is visible
					checkElem.checked = true;
				}
			}
		
		}
		if (elemName) this.changeSelectMultiple (row,col,true);
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.selectAllMultiValue' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
};

attributeCategory.prototype.deselectAllMultiValue = function(elemName,row,col) {
	try {
		var checkArr;
		if (elemName) {
			checkArr = document.getElementsByName(elemName);
		} else {
			checkArr = document.getElementsByName('MultiValueSelect');
		}
		
		if (checkArr && checkArr.length>0) {
			for (var idx=0; idx<checkArr.length; idx++) {
				checkArr[idx].checked = false;
			}
		
		}
		if (elemName) this.changeSelectMultiple (row,col,true);
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.selectAllMultiValue' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
};



attributeCategory.prototype.deselectAllSingle = function(elemName,row,col) {
	try {
		var checkArr;
		if (elemName) {
			radioArr = document.getElementsByName(elemName);
			if (radioArr && radioArr.length >0) {
				for (var idx=0; idx<radioArr.length; idx++) {

					radioArr[idx].checked = false;
				}
				this.changeSelect (radioArr[0],row,col,null,true);
			}
		

		}

	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.deselectAllSingle' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
};
/**
* executed after clicking on show all 
*/
attributeCategory.prototype.showAllMultiValue = function(attId) {
	try {

		
		$('tr[name=MultiValueSelectTr]').each( function(index) {
            $(this).show();            
		});
		
		// DT125934: by doing the above, the hover will expand potentially very big if it wasn't limited to 350 pixels before (which is the case when only few items were valid)
		$(dropmenuobj).height(350);
		$(dropmenuobj).css('overflow-y', 'scroll');
		var objId= this.prefix + attId + '_popUpIcon';
		var obj = document.getElementById(objId); 
		if (obj) updateHoverPosition( obj, 0, 0);
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.selectAllMultiValue' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
};



/**
* Sets showAll parameter on the attribute determined by row,col to the value of showAll and repaints the applicable cell
*/
attributeCategory.prototype.showAllImages = function(row,col,visibility,showAll) {
	try {		
		var cellObj = this.cells[row][col];
		var attObj = cellObj.attObj;
		attObj.showAll = showAll;
		var elem = document.getElementById('td_'+row+'_'+col);
		if (elem) elem.innerHTML = this.renderAttributeImages(cellObj,row,col,visibility);
	
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.showAllImages' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
};

/**
* renders an item list in the cell determined by cellObj by putting a table in there with all columns of the list. 
* Might get extra parameters later to determine which ones are really shown
* TO DO Decide whether to throw this away
*/
attributeCategory.prototype.renderItemList = function(cellObj,row,col,editable) {
	var htmlcode = '';
	try {
		var attObj = cellObj.attObj;
		var listObj = attObj.listObj;
		if (!attObj || !listObj) return '';
		var col;
		

		htmlcode += '<td ';
		if (this.currentView != 'Header') htmlcode += ' id="' + this.prefix + 'td_' + row + '_' + col + '"';  
        htmlcode += this.renderStyle(cellObj.styleObj,"Value");

        // determine colspan
        var colSpan = 1;
        if (cellObj.colSpan >1) colSpan =  cellObj.colSpan * 3 - 2 ;
		colSpan ++;  // to cater for the label column        
        htmlcode += ' colspan="' + colSpan +  '" ';
        
        htmlcode += '>';		
		
		htmlcode += '<table id="itemList_'+attObj.id+'" style="width:100%;" class="ordtable darktext_greenlink_11" >';
		// Paint the header		
		htmlcode += '<th/>';  // column for the delete icon
		for (col=0; col<listObj.columns.length; col++) {
			htmlcode += '<th>' + listObj.columns[col].columnName + '</th>';		
		}
		// Paint the data
		var idxArr = new Array();
		if (attObj.combiIdx) idxArr= attObj.combiIdx.split("~");  // unique identifyers have a combiIdx field like 123~349 saying which unique combinations it represents
		for (var idx=0; idx< idxArr.length; idx++) {
			
			var combination = idxArr[idx];
			var colIdxArr = listObj.combinations[combination].split("~");
			htmlcode += '<tr>';
			
			var clickMethod = '';			
			htmlcode += '<td><img src="./images/delete_icon_15.gif" onclick="'+clickMethod+'"/></td>';		
			
			for (var col=0;  col<listObj.columns.length; col++) {
				htmlcode += '<td>' + listObj.columns[col].values[colIdxArr[col]] + '</td>';					
			}
			htmlcode += '</tr>';
		}
		
		htmlcode += '</table>';
		htmlcode += '</td>';
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.renderItemList' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
	return htmlcode;
}

/**
* renders an item list in a text box with a select button and a clear button. 
* Might get extra parameters later to determine which ones are really shown
* TO DO Decide whether to throw this away
*/
attributeCategory.prototype.renderUniqueIdentfyerBox = function(cellObj,row,col,editable) {
	var htmlcode = '';
	try {
		var attObj = cellObj.attObj;
		var listObj = attObj.listObj;
		if (!attObj || !listObj) return '';
		var col;
		

		
		htmlcode += '<table><tr>';
		htmlcode += '<td><textarea id="textArea_'+attObj.id+'" cols="40" rows="6">';
		// Paint the data
		var idxArr = new Array();
		if (attObj.combiIdx) idxArr= attObj.combiIdx.split("~");  // unique identifyers have a combiIdx field like 123~349 saying which unique combinations it represents
		for (var idx=0; idx< idxArr.length; idx++) {
			var combination = idxArr[idx];
			var colIdxArr = listObj.combinations[combination].split("~");
			if (col>0) htmlcode += '\n';
			for (var col=0;  col<listObj.columns.length; col++) {
				if (col>0) htmlcode += ' ~ ';
				htmlcode += listObj.columns[col].values[colIdxArr[col]];					
			}
			
		}
		htmlcode += '</textarea></td>';
		var clickMethod = this.functionPrefix + '.showCascadingTreeDialog(this,' + row + ',' + col+')';
		htmlcode += '<td><input type="button" value="Select" class="buttonsecondary" onclick="'+clickMethod+'"></td>';
		
		htmlcode += '</tr></table>';
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn'] + ' attributeCategory.renderUniqueIdentfyerBox' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );
    }
	return htmlcode;
}



/**
* creates HTML code for a checkbox
* @param cellObj: cell containing the attribute with restricted set
* @param row: row index of the cell
* @param col: col index of the cell
*/
attributeCategory.prototype.renderCheckbox = function(cellObj,row,col,editable) {
    
    var htmlcode = '';
    try {
        var changeMethod = this.functionPrefix + '.changeCheckbox(this,' + row + ',' + col+')';
        var theName = this.prefix  + cellObj.attObj.id;
		var attObj = cellObj.attObj;
		if (!attObj) return '';

        htmlcode += '<input type="checkbox" onclick="' + changeMethod + '" id="chkbox_' + cellObj.attObj.id + '" ';
        htmlcode += this.renderStyle(cellObj.styleObj,"Checkbox");
        if (attObj.value == 'yes') {
			htmlcode += ' checked ';
		} else {
			attObj.value = 'no';  // a matter of avoiding complaints for required fields
		}
		if (!editable) htmlcode += ' disabled ';
        htmlcode += '>';
		if (editable) {
			htmlcode += '<input type="hidden" id="' + theName + '" name="' + theName + '" value="'+ WCXML.xmlencode(attObj.value)+'">';
		}
        
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderCheckbox:\n' + err);
    }
    return htmlcode;
};

/**
* Called when a checkbox is changed
*/
attributeCategory.prototype.changeCheckbox = function(elem,row,col) {
	var attObj = null;
	try {
		var cellObj = this.cells[row][col];
		if (cellObj) {
			this.setAttributesChanged();
			attObj = cellObj.attObj;
			if (attObj) {
				// update the attribute value based on the checkbox state
				if (elem.checked) {
					attObj.value = 'yes';
				} else {
					attObj.value = 'no';
				}
				attObj.changed = true;  
				// update the hidden field so that submit is done correctly
				var hiddenElem = document.getElementById(this.prefix  + cellObj.attObj.id);
				if (hiddenElem) hiddenElem.value=attObj.value;
				
				this.executeButtonDependencies(attObj);
			}
			
		}
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.changeCheckbox:\n' + err);
    }
	if (attObj) this.checkRepaintAttributeCategory(attObj);

};

/**
* creates HTML code for a dropdown box for restricted values created from rules (read from excel etc)
* @param cellObj: cell containing the attribute with restricted set
* @param row: row index of the cell
* @param col: col index of the cell
*/
attributeCategory.prototype.renderRestriction = function(cellObj,row,col) {
    
    var htmlcode = '';
    try {
        var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';
        var theName = this.prefix  + cellObj.attObj.id;
		
		// find the restricted object
		var rsObj = cellObj.ruleObj.visibilityRestriction; // you are sure it exists otherwise you don't get here
		// what attribute?
		var attObj = null;
		var attIdx = rsObj.attribute0;
		if (attIdx >=0 && attIdx<this.attributes.length) attObj = this.attributes[attIdx];
		// find the array with values
		if (attObj && attObj.value!=null && attObj.value!= "" && rsObj.restrictedValues[attObj.value]) {
		var valArr = null;
			valArr = rsObj.restrictedValues[attObj.value];
		} else {
			valArr = rsObj.restrictedValues["$$$ALL$$$"];
		}
		
        htmlcode += '<select onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
        htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");
        
        htmlcode += '>';
        htmlcode += '<option value="">---' + this.langArr['ACatPleaseSelect'] + '---</option>';
		var found = false; // did we find the selected value in the list of options
        for (var option=0; option < valArr.length; option++) {
            var value = valArr[option];
			var valueDisplay = value;
			
            htmlcode += '<option value="'+ WCXML.xmlencode(value) +'"';
            if (cellObj.attObj.value == value) {htmlcode += ' selected '; found=true;};
            htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
        }
		if (cellObj.attObj.value && !found) {
			var value = cellObj.attObj.value;
			var valueDisplay = value;
			
			htmlcode += '<option value="'+ WCXML.xmlencode(value) +'" selected ';            
            htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + this.langArr['ACatDeleted'] + ')</option>';
		}
        htmlcode += '</select>';
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderRestriction'+ this.langArr['ACatAtLine'] + err.lineNumber + ": " +  err.message );
    }
    return htmlcode;
};

/**
*  creates HTML code for a dropdown box for restricted values from a List in an Excel sheet (or later in WebCenter)
* @param cellObj: cell containing the attribute with restricted set
* @param row: row index of the cell
* @param col: col index of the cell
*/
attributeCategory.prototype.renderRestrictedList = function(cellObj,row,col,editable,hpSectionId) {
    
    var htmlcode = '';
    try {
        var attObj = cellObj.attObj;
		if (!attObj) return '';
	
		if (hpSectionId == null) hpSectionId = '';
		var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+ ',\'' + hpSectionId+'\')';
		
        var theName = this.prefix  + attObj.id;
		
	
		var listObj = attObj.listObj; 
		if (!listObj) return '';
		
		var columnObj = attObj.columnObj;
		if (!columnObj) throw(this.langArr['ACatColNotSelectedProblem']);
		
		
		if (editable) {
			// find the array with values
			//var date1 = new Date();
			
			try {
				if (attObj.columnObj  ) {
					if (!attObj.columnObj.validArr || (attObj.listObj && attObj.listObj.updateValuesNeeded) ) this.updateListAttributeValues(attObj.listObj,false);					
				}
			} catch(err) {
				// do nothing (cascading dropdown will not be initialized), but this is probably solved by later rendering calls.
			}
			if (attObj.columnObj  ) {
				valArr = columnObj.values;
				if (!valArr) {
					debugMsg += "\n" + new Date() + ": valArr does not exist for " + attObj.name; 
					return 'loading...';
				}
			}
					
			htmlcode += '<select onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
			htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");
			
			htmlcode += '>';
			htmlcode += '<option value="">---' + this.langArr['ACatPleaseSelect'] + '---</option>';
			
					
			var validArr = columnObj.validArr;  // using an array to allow counting the number of remaining options
			
			if (attObj.columnObj  ) {
				// Determine maximum number of entries to show in the dropdown
				var maxValuesInDropdown = columnObj.maxValuesInDropdown;
				if (!maxValuesInDropdown) maxValuesInDropdown = 500;
				
				// determine how many valid entries there are to display
				var nValid = columnObj.nValid;
				if (!nValid && nValid !== 0 || attObj.showAll) nValid =  valArr.length;  // we'll show everything
				
				var found = false; // did we find the selected value in the list of options
				var optionLength = valArr.length;
				var cutOff = 0;  // number cut off because there would be too many entries in the dropdown
				
				if (maxValuesInDropdown && nValid > maxValuesInDropdown && !attObj.showMore && !attObj.showAll) {optionLength = maxValuesInDropdown; cutOff = ( nValid - maxValuesInDropdown);};
				var htmlcodeArr = new Array();
				logger += "\n dropdown for " + attObj.name + " length " + optionLength + " maxvaluesinDropdown "+ maxValuesInDropdown + " columnObj.maxvaluesinDropdown " +  columnObj.maxValuesInDropdown + " columnObj.nValid "+columnObj.nValid + "  !showmore " + !attObj.showMore;
				var somethingHidden = false;  // remembers whether something has been hidden because it is filtered away by previous selections
				var displayedOptions =0;
				//for (var option=0;displayedOptions < optionLength && option < valArr.length; option++) {
				for (var option=0; option < valArr.length; option++) {
					var value = valArr[option];
					//var selected = (attObj.value == value);
					var selected = (option == parseInt(columnObj.currentIndex));
					var valid = true;
					if (validArr && validArr[option]==false) valid = false;
					
					if (displayedOptions < optionLength || selected) {  // never hide what is selected
						if (valid || attObj.showAll || selected) {					
							//if (!strtrim(value)) continue;  // do not show empties...
							
							var hc = '<option value="'+ WCXML.xmlencode(value) +'"';
							if ( selected) {hc += ' selected '; found=true;};						
							if (!valid && !selected) hc+= ' style= "font-style:italic;color:#c0c0c0;" '
							hc += '>' + WCXML.xmlencode(value);
							if (!valid && selected) hc+= ' <i>' + this.langArr['ACatNotValid'] + '</i>';  // say that it is invalid (there is a second case of this below)
							hc += '</option>';
							htmlcodeArr.push(hc);
							displayedOptions++;
						} else {
							somethingHidden=true;
						}
					}
				}
				if (cutOff) {
					var hc = '<option value="$$More$$"><i>'+ cutOff + this.langArr['ACatMoreEntries'] +'</i></option>';
					htmlcodeArr.push(hc);
				}
				htmlcode += htmlcodeArr.join();
				if (somethingHidden) htmlcode += '<option value="$$ShowAll$$" class="greentext_11">' + this.langArr['ACatShowAll'] + '</option>';
				
				// If selection not found and there is a value anyway, show it with "(NOT VALID)" 
				if (strtrim(attObj.value) && !found) {
					var value = attObj.value;			
					
					htmlcode += '<option value="'+ WCXML.xmlencode(value) +'" selected ';            
					htmlcode += '><i>' + WCXML.xmlencode(value) + this.langArr['ACatNotValid'] + '</i></option>';
				} 
				
				//alert("render list " + attObj.name + " took "+ (new Date() - date1));
				attObj.showAll = false;  // only renders once with all values
				attObj.showMore = false;  // only renders once with all values		
			
			} else {
				// only show the current value  
				htmlcode += '<option value="'+ WCXML.xmlencode(attObj.value) +'" selected>'+WCXML.xmlencode(attObj.value)+'</option>';
			}
			htmlcode += '</select>';
			htmlcode += '<input type="checkbox" value="ON" style="display:none" checked="" id="' + theName + '_excel" name="' + theName + '_excel" >';  // HADW CR: what is this?
		} else {
			htmlcode += attObj.value;
			htmlcode += '<input type="hidden" name="' + theName + '" value="' + WCXML.xmlencode(attObj.value) + '">';
		}
		
		
		
    } catch (err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderRestrictedList' + this.langArr['ACatAtLine'] + err.lineNumber + ": " +  err );
    }
    return htmlcode;
};

attributeCategory.prototype.renderOtherAttributeSelector = function(cellObj,row,col) {
    
    var htmlcode = '';
    try {
        
        var attObj = cellObj.attObj;
		if (!attObj) return;
		
        var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+')';
        var theName = this.prefix  + cellObj.attObj.id;
		var attObj2 = attObj.restrictToAttObj;
		
		if (attObj2 && attObj2.value) {
			var valArr = attObj2.value.split(this.multiSep);
			
			if (valArr.length == 1) attObj.value = attObj2.value;   // automatically select the single one allowed
			htmlcode += '<select onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
			htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");
			
			htmlcode += '>';
			htmlcode += '<option value="">---' + this.langArr['ACatPleaseSelect'] + '---</option>';
			var found = false; // did we find the selected value in the list of options
			for (var option=0; option < valArr.length; option++) {
				var value = valArr[option];
				var valueDisplay = value;
				
				htmlcode += '<option value="'+ WCXML.xmlencode(value) +'"';
				if (attObj.value == valArr[option]) {htmlcode += ' selected '; found=true;};
				htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
			}
			if (cellObj.attObj.value && !found) { 
				
				if (attObj.allowMismatchingRestriction) {  // currently never set for lack of a method to do so
					var valueDisplay = attObj.value;
					
					htmlcode += '<option  value="NOT FOUND" selected ';            
					htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + this.langArr['ACatDeleted'] + ')</option>';
				} else {
					attObj.value = ''; 	// TO DO: is this sufficient?
				}
			}
			htmlcode += '</select>';
		} else {
			// do nothing. No dropdown will be shown when there are no base values
		}
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderOtherAttributeSelector:\n' + err);
    }
    return htmlcode;
};


/**
* creates HTML code to be put in a <td> or similar element to contain styling of styleObj
* @param styleObj: object containing all styling information
* @param fieldType: one of 'Label','DateField','InputField','Dropdown','Value' 
* @param paddingRight adds padding to the right, typical for the unit column
* @param roomForButton - leaves room to put a button after an input field
*/
attributeCategory.prototype.renderStyle = function(styleObj,fieldType,paddingRight,roomForButton) {
  try {
    var htmlcode = '';
    if (styleObj) {  
		// apply class if there is one
        var cssClass = null;  // don't call it class because that's a reserved word
		if (fieldType == 'Label') {
			cssClass = styleObj.classLabel; 
			if (!cssClass) cssClass = this.styles[0].classLabel;            
        } else {
			cssClass = styleObj.classValue; 
			if (!cssClass) cssClass = this.styles[0].classValue;               
        }
		htmlcode += ' class="'+ cssClass +'" ';
        
		// apply styling 
		htmlcode += ' style=" ';
        if (fieldType != 'Label' && styleObj.valueStyleObj) { 
          // if it's a value and label and value are styled differently, take the value style
		      styleObj = styleObj.valueStyleObj;
        }
	
        // set background color but not for the input fields and dropdowns		
        if (styleObj.background && fieldType!='InputField' && fieldType != 'Dropdown' && fieldType != 'DateField' && fieldType != 'MultiValue' && fieldType != 'RoundedTitle') {
			htmlcode += 'background-color:' + this.fixStrictModeColor(styleObj.background) + ';';
		}

        // set alignment
        //if (styleObj.alignment != "") htmlcode += 'text-align:' + styleObj.alignment + ';';    // does not check on null or undefined
		if (styleObj.alignment) htmlcode += 'text-align:' + styleObj.alignment + ';'; 
		
		if (fieldType != 'Button') htmlcode += 'vertical-align: top;';
		/*
		// vertical alignment currently not in use, so always defaulting to top
        if (styleObj.verticalAlignment) {
			htmlcode += 'vertical-align:' + styleObj.verticalAlignment + ';';
		} else {
			htmlcode += 'vertical-align:top;'; 
		}
		*/
		
		
		// set padding-right (typical for unit/type column)
		if (paddingRight >0) {
			htmlcode += 'padding-right:' + paddingRight + 'px;';
		}
	
        // set height
      	if(styleObj.height) {
        	htmlcode += 'height:' + styleObj.height + 'px;';       		
      	}
      	
      	// set width should not be applied to the <td> but should be applied to the cell itself (the input cell).
      	
		if (fieldType == 'InputField') {
			if(styleObj.width && fieldType != 'Label') {
				htmlcode += 'width:' + styleObj.width;
				if(styleObj.widthunit && styleObj.widthunit == "%") {
					htmlcode +=  '%;';  
				}
				else {
					htmlcode +=  'px;';  
				}
			} else {
				if (!roomForButton) htmlcode += 'width:100%;';
			}
		}
      	
        // set font properties
      	if (styleObj.fontWeight) htmlcode += 'font-weight:' + styleObj.fontWeight + ';';
      	if (styleObj.fontStyle) htmlcode += 'font-style:' + styleObj.fontStyle + ';';
      	if (styleObj.fontWeight) htmlcode += 'font-weight:' + styleObj.fontWeight + ';';
      	//if (styleObj.fontFamily) htmlcode += 'font-family:\'' + styleObj.fontFamily + '\';';
		if (styleObj.fontFamily) htmlcode += 'font-family:' + styleObj.fontFamily + ';';
      	if (styleObj.fontSize) htmlcode += 'font-size:' + styleObj.fontSize + 'pt;';
      	if (styleObj.fontColor) htmlcode += 'color:' + styleObj.fontColor + ';';
		
		
		// Width of the column		
		if ( (fieldType == 'Label' || fieldType == 'Value') && styleObj.columnWidth) {			
			htmlcode += 'width:' + styleObj.columnWidth;  
			
			if(styleObj.columnWidthUnit && styleObj.columnWidthUnit == "%") {
				htmlcode +=  '%;';  
			} else {
				htmlcode +=  'px;';  
			}			
		}
		
		
		if (fieldType == 'MultiValue') {
			htmlcode += 'padding:1px; background-color:white;';  			
		}
		
      
      	htmlcode += '" '; // end style


    } else { // no style
		if (this.currentView != 'Header') {
			if (fieldType == 'Label') {
			htmlcode += ' class="darktext_darklink_11" ';
			}
			else if (fieldType == 'InputField') {
			htmlcode += ' class="darktext_darklink_11" style="width:100%" ';
			} else {
			htmlcode += ' class="darktext_darklink_11" ';
			}
		} else { // Header view
			if (fieldType == 'Label') {
				htmlcode += ' class="Black_Bold_11" align="right" style="padding-left:40px;" ';
			} else {
				htmlcode += ' class="Black_11" ';
			}
		}
    }
    
  } catch (err) {
    alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderStyle:\n' + err);
	return '';
  }
  return htmlcode;
};

/**
* creates HTML code to render a ruler (horizontal line) with thickness and color
* @return returns the HTML code
*/
attributeCategory.prototype.renderRuler = function(cellObj) {
	var htmlcode = '';
	try {
		htmlcode += '<td colspan="' + (cellObj.colSpan*3) + '"><hr "';
		if (cellObj.rulerColor) {
			htmlcode += ' color="' + cellObj.rulerColor + '" ';
		} else {
			htmlcode += ' color="black" ';
		}
		if (cellObj.rulerThickness) {
			htmlcode += ' size="' + cellObj.rulerThickness +'" ';		
		}
		htmlcode += ' /></td>';
    	   
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderRuler:\n' + err);
		return '';
    }
	return htmlcode;
};

/**
* creates HTML code to render a set of buttons defined in cellObj.buttons
* @param {Boolean} inline: when true, the button will be put inline in an existing cell
* @return returns the HTML code
*/
attributeCategory.prototype.renderButton = function(cellObj,row,col,inline,forPrint) {
	var htmlcode = '';
	try {
		
		if (!inline) {
			htmlcode += '<td colspan="' + (cellObj.colSpan*3) + '" ' + this.renderStyle(cellObj.styleObj,"Button") + '>';  
		}
		if (!forPrint) {
			var clickMethod =  this.functionPrefix + '.clickACatButton(null, this,\'click\');return false;';
			
			if (cellObj.buttons) {
				for (var button=0; button < cellObj.buttons.length; button++) {
					var buttonObj = cellObj.buttons[button];
					if (buttonObj && buttonObj.style != 'hidden') {
						var id= this.prefix + 'ACatButton~' + row + '~' + col + '~' + button; 
						if (button >0) htmlcode += '&emsp;&emsp;';  // breakable spaces
						if (buttonObj.style != 'greenlink') {					
							htmlcode += '<input type="button" value="'+ WCXML.xmlencode(buttonObj.label) +'" onclick="'+clickMethod+'" id="'+id+'"';
							if (buttonObj.style && buttonObj.style !='plainbutton') htmlcode += ' class="'+buttonObj.style+'" ';
							htmlcode += '/>';
						} else {
							htmlcode += '<span class="darktext_greenlink_11"><a  href="#" onclick="'+clickMethod+'"  id="'+id+'">'+WCXML.xmlencode(buttonObj.label)+'</a></span>';
							
						}
					}
				}
			}
		}
		
		if (!inline) htmlcode += '</td>';
		
    	   
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderButton:\n' + err);
		return '';
    }
	return htmlcode;
};

/**
* creates HTML code to render space with a background color
* @return returns the HTML code
*/
attributeCategory.prototype.renderSpacer = function(cellObj) {
	var htmlcode = '';
	var requestedHeight;
	try {
		htmlcode += '<td colspan="' + (cellObj.colSpan*3) + '" style="';		
		if (cellObj.height) {
			htmlcode += 'height:' + cellObj.height +'px;';	
			requestedHeight = cellObj.height;
		} else {
			htmlcode += 'height:28px;';	//18 typical height + 2 x 5 pix padding	
			requestedHeight = 28;
		}
		// if multiple columns, cellpadding af neighbours will interfere with height, so place a div in here
		htmlcode += 'padding:0;"><div style="height:'+requestedHeight+'px;width:100%;padding:0;';
		if (cellObj.background) {
			htmlcode += 'background-color:' + this.fixStrictModeColor(cellObj.background) + ';';		
		}
		htmlcode += '"></div></td>';
    	   
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderSpacer:\n' + err);
		return '';
    }
	return htmlcode;
};

/**
* creates HTML code to render a title with styling
* @param roundedTitle creates HTML optimized for the typical WCR rounded title. Other styling is disregarded
* @return returns the HTML code
*/
attributeCategory.prototype.renderTitle = function(cellObj, roundedTitle, row, col, visibility, forPrint) {
    try {
        var htmlcode = '';
        
		if (roundedTitle) {
			htmlcode += '<th colspan="' + (cellObj.colSpan*3) + '" class="lastHeaderCol" >' ;
		} else {
			htmlcode += '<td colspan="' + (cellObj.colSpan*3) + '" ' + this.renderStyle(cellObj.styleObj,"Label") + '>' ;
		}
		htmlcode += WCXML.xmlencode(cellObj.label);
		if (cellObj && cellObj.infoPage) {		
			var clickMethod = 'popw=window.open(&quot;./custom/AttributesHelp/'+ WCXML.xmlencode(cellObj.infoPage) + '&quot;,&quot;Info&quot;,' + this.helpOptionsString + ');popw.focus();';
			//var clickMethod = 'javascript:void( popw=window.open(&quot;./custom/AttributesHelp/'+ cellObj.infoPage + '&quot;,&quot;Info&quot;,' + this.helpOptionsString + ' );popw.focus();)'
			htmlcode += '&nbsp;&nbsp;<img src="images/help_icon_16.gif" onclick="' + clickMethod +'">';
		}
		
		// buttons if there are any
		var editable = false;
		if (this.currentUser && this.currentUser.canEditAttributes) editable = true;
		if (visibility != 'Editable' && visibility != 'Required') editable = false;
		if (editable && cellObj.buttons) htmlcode += '&emsp;'+  this.renderButton(cellObj, row, col, true, forPrint);
		htmlcode += roundedTitle? '</th>' : '</td>';
		
        return htmlcode;
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderTitle:\n' + err);
    }
};

/**
* creates HTML code to render an attribute as a grid of images
* Known restriction: doesn't listen to maxValuesInDropdown --> Put this in FS
* @return returns the HTML code
*/
attributeCategory.prototype.renderAttributeImages = function(cellObj,row,col,visibility) {
    try {
		var editable = false;
		var clickMethod;
		if (this.currentUser && this.currentUser.canEditAttributes == true) editable=true;
        var htmlcode = '';
		var columnObj;
        var attObj = cellObj.attObj;  
		var idx,value, rsType = '';
		if (!attObj) return '';
		var imageHeight = cellObj.imageHeight;  // images will be scaled to all have this height
		if (!imageHeight) imageHeight = 100;
		//var onloadFunction = 'WCHTML.scaleImageProportionally(this,100,100);';
		var listName = attObj.listObj? attObj.listObj.name : 'default';
		if (cellObj.renderImageType == 'automatic' || !cellObj.renderImageType) {
			onerrorFunction = 'WCHTML.alternativeImage(this,\'./custom/AttributeValueImages/'+listName+'.jpg\');';
			cellObj.renderImageType = 'automatic';  // in case renderImageType is not set and you get here anyway, set it to automatic.
		} else {
			onerrorFunction = '';			
		}
		if (!cellObj.nImageColumns) cellObj.nImageColumns = 6;
		

		
		var valueArr;
		var curValArr = [];
		var colSpan;
		
		if (attObj.rs) {valueArr = attObj.rs.values; rsType = 'RestrictedSet';};
		if (attObj.columnObj) {
			columnObj =  attObj.columnObj; valueArr = columnObj.values; 
			rsType = 'List';
			if (attObj.listObj && attObj.listObj.updateValuesNeeded) this.updateListAttributeValues(attObj.listObj,false);					
		};	
		
	
		if (valueArr && visibility != 'Hidden' && visibility != 'HiddenKeepValues') {
			colSpan = cellObj.colSpan * 3;
			if (!colSpan) colSpan = 3;
			//htmlcode += '<td colspan="' + colSpan + '" ' + this.renderStyle(cellObj.styleObj,"Label") + ' id="'+this.prefix+'td_'+row+'_'+col+'">' ;
			//htmlcode += this.renderHiddenAttributeInfo(attObj);
			if (attObj.allowMultipleValues) htmlcode += '<input id="'+ this.prefix + attObj.id +'" type="hidden" value="'+ WCXML.xmlencode(attObj.value)+'" name="'+  attObj.id +'">';
			htmlcode += '<div width="100%" ';
			if (cellObj.renderHeight) htmlcode += 'style="height:' + cellObj.renderHeight + ';overflow:auto;"';
			htmlcode += '>';
			var changeMethod = this.functionPrefix + '.changeSelect(this,' + row + ',' + col+',null,true)';
			if (attObj.allowMultipleValues) {
				changeMethod = this.functionPrefix + '.changeSelectMultiple(' + row + ',' + col+',true)';
				if (attObj.value) curValArr = attObj.value.split(this.multiSep);				
			}
			
			if (cellObj.imageAlignment == 'DistributeOverWidth') {
				htmlcode += '<table width="100%">';
			} else {
				htmlcode += '<table>';
			}
			// Put Select All, Select None and Show All
			/*
			colSpan = cellObj.nImageColumns -1;
			if (!colSpan) colSpan = 1;
			htmlcode += '<tr>';
			htmlcode += this.renderAttributeLabel(cellObj, row,col,visibility,editable,true,colSpan);
			
			htmlcode += '</tr>';
			*/
			
			// determine which and how many valid entries there are to display
			var nValid;
			if ( rsType == 'RestrictedSet') {
				nValid = valueArr.length;
			} else if (rsType == 'List') {
				nValid = columnObj.nValid;
				if (!nValid && nValid !== 0) nValid =  valueArr.length;  // we'll show everything
				if ( nValid > 0) {
					// construct a new valueArr with only the valid entries
					var valueArr2 = [];
					for (idx = 0; idx<valueArr.length; idx++) {
						if (attObj.showAll || !columnObj.validArr || columnObj.validArr[idx]) { 
							valueArr2.push({value: valueArr[idx], idx: idx});
						}
					}
					valueArr = valueArr2;
				}
			}
			
			// determine whether there are selected values which are no longer in validArr (f.e. restricted set value taken out)
			var extraValueArr = [];
			if (attObj.allowMultipleValues) {
				for (var idx = 0 ; idx< curValArr.length; idx++) {
					for (var idx2=0; idx2<valueArr.length; idx2++) {
						if (valueArr[idx2].value == curValArr[idx]) break;
					}
					if (idx2 >= valueArr.length) extraValueArr.push(curValArr[idx]);
				}
			} else if (attObj.value) {
				for (var idx2=0; idx2<valueArr.length; idx2++) {
					if (valueArr[idx2].value == attObj.value) break;
				}
				if (idx2 >= valueArr.length) extraValueArr.push(attObj.value);
			}
			
			// determine number of rows
			var nImageRows = Math.ceil((nValid +extraValueArr.length ) / cellObj.nImageColumns );
			var widthPerc = 100/ cellObj.nImageColumns ;
			var imgExtension = (cellObj.renderImageType == 'automatic') ? '.jpg' : ('.' + cellObj.renderImageType);
			var cellAlign = 'center';
			if (cellObj.restrictedValueLabelPosition != 'LabelBelow' ) cellAlign = 'left';
		
			
			for (var row=0; row < nImageRows; row++) {
				
				// Images
				if (cellObj.renderOption === 'Images') {
					htmlcode += '<tr>';			
					for (col=0; col< cellObj.nImageColumns; col ++) {									
						idx = row*cellObj.nImageColumns + col; 
						if (idx < valueArr.length ||   (row== nImageRows-1 &&  extraValueArr.length>0  && (idx-valueArr.length <extraValueArr.length) ) ) {
							if (idx < valueArr.length) {
								value = valueArr[idx].value;
							} else {
								value = extraValueArr[idx-valueArr.length];
							}
							if (cellObj.imageAlignment == 'DistributeOverWidth') {
								htmlcode += '<td align="'+cellAlign+'" width="'+ widthPerc+'%">';
							} else {
								htmlcode += '<td align="'+cellAlign+'"style="padding:0 10 0 10" >';
							}
							//htmlcode += '<img src="./custom/AttributeValueImages/'+ WCXML.xmlencode(value) + imgExtension + '" height="' + imageHeight +'" onload="'+onloadFunction+'" onerror="'+onerrorFunction+'">';
							htmlcode += '<img src="./custom/AttributeValueImages/'+ WCXML.xmlencode(value) + imgExtension + '" height="' + imageHeight +'"  onerror="'+onerrorFunction+'">';
							htmlcode += '</td>';
						
						} else {
							htmlcode += '<td/>';  // needed to avoid that there are less columns which will make them take more space --> ugly alignment
						}
						
					}
				
					htmlcode += '</tr>';
				}
				
				// Radio buttons or checkboxes
				htmlcode += '<tr>';
				for (col=0; col< cellObj.nImageColumns; col ++) {
					idx = row*cellObj.nImageColumns + col; 
					if (idx < valueArr.length ||   (row== nImageRows-1 &&  extraValueArr.length>0  && (idx-valueArr.length <extraValueArr.length) ) ) {
						var curIdx = idx;					
						if (idx < valueArr.length) {
							value = valueArr[idx].value;
							if (rsType == 'List') curIdx = valueArr[idx].idx;
						} else {
							value = extraValueArr[idx-valueArr.length];
						}
						if (cellObj.imageAlignment == 'DistributeOverWidth') {
							htmlcode += '<td align="'+cellAlign+'" width="'+ widthPerc+'%">';
						} else {
							htmlcode += '<td align="'+cellAlign+'"style="padding:2 10 2 10" >';
						}
						
						
						if (attObj.allowMultipleValues) {   // put checkboxes
							htmlcode += '<input type="checkbox" style="vertical-align: middle; margin: 0px;" name="'+this.prefix + 'ImgChk_' + attObj.id +'" id="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '"  value="'+ WCXML.xmlencode(value) +'" idx="'+curIdx+'" onchange="'+ changeMethod +'"';		
							for (var idx2=0; idx2<curValArr.length; idx2++) {
								if (curValArr[idx2] === value) {htmlcode += ' checked=checked '; break;};
							}						
							if (!editable || visibility == 'ReadOnly') htmlcode += ' disabled=disabled ';
							htmlcode += '>';
						
						} else {   // put radio buttons
							htmlcode += '<input type="radio" style="vertical-align: middle; margin: 0px;" name="'+this.prefix + attObj.id+'" value="'+ WCXML.xmlencode(value) +'" idx="'+curIdx+'" id="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '"  onchange="'+ changeMethod +'"' 
							if (attObj.value === value) htmlcode += ' checked=checked ';
							if (!editable || visibility == 'ReadOnly') htmlcode += ' disabled=disabled ';
							htmlcode += '>';
						}
						if (cellObj.restrictedValueLabelPosition == 'LabelRight' ) {
							htmlcode +=  '&nbsp;<span class="darktext_greenlink_11"><label for="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '">' + WCXML.xmlencode(value) + '</label></span>';
							if (idx>=valueArr.length) htmlcode += '&nbsp;' + this.langArr['ACatNotValid'];
						}
						htmlcode += '</td>';
					} else {
						htmlcode += '<td/>';
					}
				}
				htmlcode += '</tr>';
				
				// Text
				if (cellObj.restrictedValueLabelPosition == 'LabelBelow' ) {
					htmlcode += '<tr>';
					for (col=0; col< cellObj.nImageColumns; col ++) {
						idx = row*cellObj.nImageColumns + col; 
						if (idx < valueArr.length ||   (row== nImageRows-1 &&  extraValueArr.length>0  && (idx-valueArr.length <extraValueArr.length) ) ) {
							if (idx < valueArr.length) {
								value = valueArr[idx].value;
							} else {
								value = extraValueArr[idx-valueArr.length];
							}
							htmlcode += '<td align="'+cellAlign+'" style="padding:2 10 10 10" class="darktext_greenlink_11">';
							htmlcode +=  '<label for="'+this.prefix+'ImgChk_'+attObj.id+'_'+ idx + '">' +WCXML.xmlencode(value) + '</label>';
							if (idx>=valueArr.length) htmlcode += '&nbsp;' + this.langArr['ACatNotValid'];
							htmlcode += '</td>';
						} else {
							htmlcode += '<td/>';
						}
					}
					htmlcode += '</tr>';
				}
			}
			
			htmlcode += '</table>';
			htmlcode += '</div></td>';
		}
		
        return htmlcode;
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderAttributeImages at line' +  err.lineNumber + ':\n' + err);
    }
};

/**
* creates HTML code to render an emty cell
* @return returns the HTML code
*/
attributeCategory.prototype.renderEmptyCell = function() {
    try {

        return '<td></td><td></td><td></td>';
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderEmptyCell:\n' + err);
    }
};

attributeCategory.prototype.renderHiddenAttributeInfo = function(attObj, hiddenCell) {
	var htmlcode = '';
	try {
		var theName = this.prefix + "UN" + attObj.id;
        htmlcode += '<input type="hidden" name="' + theName + '" value="' + attObj.unitId + '">';

        // set the isRS hidden field to indicate whether it is a restricted set
        var rsVal = "0";
        if (attObj.rs) rsVal = "1";
        theName = this.prefix + "isRS" + attObj.id;
        htmlcode += '<input type="hidden" name="' + theName + '" id="' + theName + '" value="' + rsVal + '">';
		
		if (attObj.type && attObj.type.toLowerCase() == 'datetime' && rsVal != "1") {
			var dateName = this.prefix +  attObj.id;
            var aDateName = this.prefix + "adate" + attObj.id;
            var saveADateName = this.prefix + "save_adate" + attObj.id;                        
                        
			htmlcode += '<input type="hidden" name="' + dateName + '" id="' + dateName + '" value="' + (hiddenCell?"":attObj.hiddenDate) + '">';  
            htmlcode += '<input type="hidden" name="' + aDateName + '" id="' + aDateName + '" value="' + (hiddenCell?"":attObj.hiddenDate) + '">';
            htmlcode += '<input type="hidden" name="' + saveADateName + '" id="' + saveADateName + '" value="' + (hiddenCell?"":attObj.oldDate) + '">';
		
			
		}
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderHiddenAttributeInfo:\n' + err);
    }
	return htmlcode;
};

/**
* creates HTML code to render a hidden cell
* @param values If true, attObj.value is added, if false empty values are used
* @return returns the HTML code
*/
attributeCategory.prototype.renderHiddenCell = function(attObj,values) {
	var htmlcode = '';
    try {
		if (attObj) {
			htmlcode += this.renderHiddenAttributeInfo(attObj, !values); 
			var theName = this.prefix  + attObj.id;	
			var	value = values? attObj.value : '';
			
			htmlcode += '<input type="hidden" value="'+ WCXML.xmlencode(value)+'" name="'+theName+'" id="'+theName+'" >';
			if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
				var longDateName = this.prefix + "longFormat_adate" + attObj.id;
				htmlcode += '<input type="hidden" value="" name="'+longDateName+'" id="'+longDateName+'" >';	 // assumes hidden = empty
				var hourName = this.prefix + "date_period_of_day_" + attObj.id;	
				var halfhours = 0;
				if (value) halfhours = value.getHours() * 2 + Math.floor(value.getMinutes()/30);
				htmlcode += '<input type="hidden" value="'+ halfhours +'" name="'+hourName+'" id="'+hourName+'" >';				
			}				
		}
        
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderHiddenCell:\n' + err);
    }
	return htmlcode;
};

/**
* Figures out whether a cell is visible or not
* This function is recursive
* @param cellObj: object representing the cell which visibility needs to be investigated
* @param ruleObj: rule object to check against, if null cellObj.ruleObj is used
* @return one of Hidden, ReadOnly, Required, Editable
*/
attributeCategory.prototype.checkVisibility = function(cellObj) {
    try {
		if (!cellObj) return true;  // should not happen
		ruleObj = cellObj.ruleObj;
		
        if (!ruleObj) return "Editable";  // happens when called and there is no rule applied (=frequent)
		if (!ruleObj.condition) return ruleObj.name; // this is a standard rule
		var trueOrFalse = this.checkVisibilityRecursive(ruleObj);
        if (trueOrFalse) {
			return ruleObj.visibility;
		} else {
			return ruleObj.visibilityElse;
		}

         
        
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.checkVisibility:\n' + err);
        return false;  // whatever cannot be interpreted is so ugly you don't want to show it...
    }
};

/**
* function works case sensitive and does cut leading and trailing spaces
*/
attributeCategory.prototype.checkMultipleValues = function(valueString,compareValue) {
	var valueArr = valueString.split(this.multiSep);
	compareValue = strtrim(compareValue).toLowerCase();
	var match = false;
	for (var idx=0; idx<valueArr.length; idx++) {
		if (strtrim(valueArr[idx]).toLowerCase() == compareValue) {match=true; break;};
	}
	return match;

};
/**
* Figures out whether the combined condition of a rule is true or false
* @param ruleObj: rule object to check against, if null cellObj.ruleObj is used
* @return true or false
*/
attributeCategory.prototype.checkVisibilityRecursive = function(ruleObj) {
    try {
               
        if (!ruleObj) return true;  // happens when called and there is no rule applied (=frequent)

        if (ruleObj.attObj) {  // visibility  depends on attribute value -- leaf of the rule tree
            // find the attribute with this id
            var attObj = ruleObj.attObj;
            switch (ruleObj.operator) {
                case '=':
					if (!attObj.allowMultipleValues) {
						return (strtrim(attObj.value).toLowerCase() == strtrim(ruleObj.compareValue).toLowerCase()) ;
					} else {
						return (this.checkMultipleValues(attObj.value,ruleObj.compareValue));
					}
                    break;
                case '!=':
					if (!attObj.allowMultipleValues) {
						return (strtrim(attObj.value).toLowerCase() != strtrim(ruleObj.compareValue).toLowerCase()) ;
					} else {
						return !(this.checkMultipleValues(attObj.value,ruleObj.compareValue));
					}
                    break;
                case '>':
                    return ( parseFloat(attObj.value) > parseFloat(ruleObj.compareValue)) ;
                    break;
                case '>=':
                    return ( parseFloat(attObj.value) >= parseFloat(ruleObj.compareValue)) ;
                    break;
                case '<':
                    return ( parseFloat(attObj.value) < parseFloat(ruleObj.compareValue)) ;
                    break;
                case '<=':
                    return ( parseFloat(attObj.value) <= parseFloat(ruleObj.compareValue)) ;
                    break;
                case 'One Of':
                    // split compareValue to line breaks
                    var compareArr = ruleObj.compareValue.split("\n");
                    var found = false;
                    for (var idx=0; idx<compareArr.length; idx++) {
						if (!attObj.allowMultipleValues) {
							if (strtrim(attObj.value).toLowerCase() == strtrim(compareArr[idx]).toLowerCase()) {
								found = true; break;
							}
						} else {
							if (this.checkMultipleValues(attObj.value,compareArr[idx])) {
								found = true; break;
							}
						}
                    }
                    return (found) ;
                    break;

                default:

                    throw(this.langArr['ACatOpNotSupportedProblem1'] + ruleObj.operator + this.langArr['ACatOpNotSupportedProblem2']);
            }
        } else { // and-or node
            var visible1 = this.checkVisibilityRecursive(ruleObj.ruleObj1);
			var visible2 = this.checkVisibilityRecursive(ruleObj.ruleObj2);
			if (ruleObj.condition == 'AND' && visible1 && visible2) return true;
			if (ruleObj.condition == 'OR' && (visible1 || visible2)) return true;
			return false;
        }
        return true;  // probably we never get here
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.checkVisibilityRecursive:\n' + err);
        return false;  // whatever cannot be interpreted is so ugly you don't want to show it...
    }
};

/**
* Creates html code for the hour selector typical for selecting a due hour
* Re-used between the plain panel and the due date recalculation pop-up
*/
attributeCategory.prototype.displayHourDropdown = function(hour,id,name,changeMethod,styleObj)  {
    var htmlcode = '';
    try {



        if (!id) throw(this.langArr['ACatDispHourProblem1']);
        if (!name) throw(this.langArr['ACatDispHourProblem2']);

        htmlcode += '<select id="' + id + '" name="' + name + '"';
        if (changeMethod) htmlcode += ' onchange="' + changeMethod + '"';
		if (styleObj) htmlcode += this.renderStyle(styleObj,"Dropdown");
        htmlcode += '>';

		// first the --:--
		if (hour == -1) {
			htmlcode += '<option value="-1" selected>--:--</option>';
		} else {
			htmlcode += '<option value="-1">--:--</option>';
		}
        for (var counter=0; counter < 24; counter++) {
            var value = counter * 2;
            var selected = (value == hour); // true or false
            var displaybase = counter<10 ? '0'+counter : counter;
            var display = displaybase + ":00";
            htmlcode += '<option value="' + value + '"' + (selected ? ' selected ' : '') + '>' + display + '</option>';
            value = counter * 2 + 1;
            display = displaybase + ":30";
            selected = (value == hour); // true or false
            htmlcode += '<option value="' + value + '"' + (selected ? ' selected ' : '') + '>' + display + '</option>';
        }
        htmlcode += '</select>';
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ " attributeCategory.displayHourDropdown: " +err);
    }
    return htmlcode;
};

/**
* Typically called for a creation view
* When documents are in creation view, the document itself typically does not yet exist in WebCenter
* There is only a document template but the document template does not get copied when uploading a file, only the attribute category is taken over
* This means the default values of the document template don't make it to the new document unless you submit them here
* Since they are not in the creation view or they are read-only or hidden in the creation view, we need to add them here
*/
attributeCategory.prototype.addDefaultHiddenFields = function () {
	
    try {
		var htmlcode = '';	
		var cells = this.cells;
		if (this.views && this.requestedView != 'Default') cells = this.views[0].cells;
		// now cells will always points to the cells in the default view
		
		// Run over all attributes in the default view
		if (cells) {
			for (var row=0; row<cells.length; row++) {
				var rowArr = cells[row];
				if (rowArr) {
					for (var col=0; col<rowArr.length;col++) {
						var cellObj = cells[row][col];
						if (cellObj && cellObj.type=='Attribute') {
							var attObj = cellObj.attObj;
							if (attObj && attObj.type ) {
								var theName;
								var attValue = attObj.value;

								if (attObj.type.toLowerCase() == 'datetime' && attValue && !attObj.rs) {  // if it's a date and there is actually a date filled out
									theName = this.prefix + 'date_period_of_day_' + attObj.id;
									attValue = attValue.getHours() * 2 + Math.floor(attValue.getMinutes()/30);							
								} else {									
									theName = this.prefix + attObj.id;	
									if (attObj.type.toLowerCase() == 'richtext'	) attValue = WCXML.QuoteEncode(attValue);
								}
								var exists = document.getElementsByName(theName);  // check whether such an element already exists (meaning it is put there mostly editable). If not, put it hidden.
								if (!exists || exists.length==0) {
									htmlcode += '<input type="hidden" name="'+theName+'" value="'+  WCXML.xmlencode(attValue)+'">';
									htmlcode += this.renderHiddenAttributeInfo(attObj);
								}
								
							}
							
						}
					}
				}
			
			}
		}
		var hiddenFieldsContainer = document.getElementById(this.prefix + "hiddenFieldsContainer");
		if (hiddenFieldsContainer) hiddenFieldsContainer.innerHTML = htmlcode;
	} catch(err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.addDefaultHiddenFields' + this.langArr['ACatAtLine'] + err.lineNumber + ': ' +  err.message );	
	}
};



/**
* Function not used, could be used later
*/
attributeCategory.prototype.noenter = function() {
    var flag = true;    
    if(window.event && window.event.keyCode == 13) {
        flag = false;
    }
    return flag;
};

/**
* Corrects for a flaw in the design: 
* The server knows the time zone offset of the client: at login, a little javascript is run which submits the 
* clients time zone offset to the server.
* Then the server uses this time zone offset to interpret the time zone of the entered data
* However, it falsely assumes that the entered dates are in the same time zone as the time zone of the client.
* This goes wrong when the client is in a country with daylight saving.  
* When in August, a user is entering 10 December 10:00, this user means 10:00 winter time.
* However, the server assumes Summer time since it's August...
* The misery is that this bug made it to the database and is stored wrongly.
* So the problem cannot be corrected here. All we can do is show the information consistently.
* consistency means that at least the date is shown the same when clicking on Change (and the page reloads). 
* Without the below code, the date can shift an hour each time you click change (if the entered date is in Summer while it's current winter or reversely). 
*/
attributeCategory.prototype.correctForDaylightSaving = function (date) {

	try {
		var shift = date.getTimezoneOffset() - (new Date()).getTimezoneOffset();
		if (shift !=0) date = new Date(date.getTime() + shift*60000); 
	} catch (err) {
		// do no correction 		
	}
	return date;
};

var logger ='';
/**
* @author HADW 23 May 2012
* Displays a tree structure of a cascading attribute list set in an HTML element with id PanelId

*
*/
attributeCategory.prototype.initDisplayCascadingTrees = function (xmlInput,PanelId,hpSectionId) {
	try {
		var htmlcode = '';	
		var hiddenXMLInput;
		var listIdx;
		//var time0 = new Date(); 
		if (hpSectionId == null) hpSectionId = '';
		
		var panelElem = document.getElementById(PanelId);
		if (!panelElem) return;
		this.cascadingLists = new Array();

		var hiddenXMLElem = document.getElementById(hpSectionId+'cascading_attribute_trees_hidden_input');
		if (!hiddenXMLElem) return ;
		var hiddenXMLString = hiddenXMLElem.value;		
		if (hiddenXMLString!= undefined && hiddenXMLString.constructor == String) hiddenXMLInput=WCXML.xmlTextToObj(hiddenXMLString);
		
		var mainElem = WCXML.GetFirstFirstLevelChild(hiddenXMLInput,'displayPreferences');
		// find width for the tree panel
		var panelWidth	= WCXML.GetElementText(mainElem,'SelectTreePanelWidth');
		if (panelWidth) panelWidth = parseInt(panelWidth,10)-30; else panelWidth= this.defaultPanelWidth; 
		
		// read what needs to come in the tree panel	
		var treeListElem = WCXML.GetFirstFirstLevelChild(mainElem,'cascading_attribute_trees');
		
		var treeListElem = WCXML.GetFirstFirstLevelChild(mainElem,'cascading_attribute_trees');
		if (treeListElem) {
			var version = parseInt(WCXML.GetElementAttribute(treeListElem,'version'),10);
			if (version >=2) {
				var treeArr=WCXML.GetFirstLevelChilds(treeListElem,'treeElem');
				if (treeArr && treeArr.length>0) {
					
					for (var idx=0; idx<treeArr.length; idx++) {
						var treeElem = treeArr[idx];
						var type = WCXML.GetElementText(treeElem, 'type');
						var listName = WCXML.GetElementText(treeElem, 'name');
						var depth = parseInt(WCXML.GetElementText(treeElem, 'depth'),10);
						if (!depth) depth=0; // meaning all columns
						var height = parseInt(WCXML.GetElementText(treeElem, 'height'),10);
						if (!height) height=0; // meaning no maximum
						var restrictColumn = WCXML.GetElementText(treeElem, 'restrictColumn');
						var restrictValue = WCXML.GetElementText(treeElem, 'restrictValue');
						
						if (listName && type=='list') {   // To DO: this will break with copied lists.  Change needed in the saved searches to allow selecting other lists
							listIdx=-1;
							for (var list2 = 0; list2< this.lists.length; list2++) {
								var extendedName = this.lists[list2].name;
								var counter = this.lists[list2].counter;
								if (counter && counter > 1) extendedName += ' (' + counter + ')';
								if (extendedName == listName) {listIdx=list2; break;};
							}
							
													
							if (listIdx>=0) this.cascadingLists.push({label:listName, listName: listName, listIdx: listIdx,depth:depth,  height:height, restrictColumn: restrictColumn, restrictValue: restrictValue});						
						}													
					}
				}
			
			} else {
				// assume old style Packaging ~ Brands ~ Usage
				var tildeString = WCXML.GetElementText(mainElem,'cascading_attribute_trees');
				var treeArr = tildeString.split(" ~ ");
				if (treeArr && treeArr.length>0) {
					for (var list=0; list<treeArr.length; list++) {
						if (treeArr[list]) {
							var listName = treeArr[list].replace(/(\r\n|\n|\r)/gm,"");  // remove linefeeds added by XML creation
							for (var list2 = 0; list2< this.lists.length; list2++) {
								if (this.lists[list2].name == listName) {listIdx=list2;break;};
							}
							if (listIdx>=0) this.cascadingLists.push({label:listName, listName: listName, listIdx: listIdx,depth:0});
							
						}
					}
				}
			}
			
		}
						
		if (this.cascadingLists.length==0) return;

		htmlcode += '<table id="'+hpSectionId+'mainCascadingTreesTable" width="100%">';
		
		
		// Current selection
		htmlcode += '<tr><td colspan="2" id="'+hpSectionId+'CurrentSelectionTopLeft"><div id="' + hpSectionId+'CurrentSelectionDiv" style="width:' + (panelWidth + 20) +';overflow-x:auto"><table id="'+hpSectionId+'CurrentSelectionTable">';				
		htmlcode += '<tr><td class="darktext_darklink_bold_12">'+this.langArr['ACatCurrentSelection']+'</td></tr>';  

	
		for (var list=0; list<this.cascadingLists.length; list++) {			
			htmlcode += '<tr><td  id="'+hpSectionId+'currentSelection_'+this.cascadingLists[list].listIdx+'" class="darktext_greenlink_11"></tr>';
		}
		htmlcode += '</table></div></td></tr>';
			
		// grey line above
		htmlcode += '<tr><td colspan="2" class="grey2_background" style="padding: 0pt;font-size:0;"><img height="1" width="1" src="images/clearpixel.gif"/></td></tr>';
		
		for (var list=0; list<this.cascadingLists.length; list++) {
			
			var label = this.cascadingLists[list].label;
			//var listName =  this.cascadingLists[list].listName;
						
			var listIdx =  this.cascadingLists[list].listIdx;
			var listObj= this.lists[listIdx];
			var htmlcode2 = '';

			if (listObj) {
				listObj.treeListObj = {listObj:listObj,label:label};
				
				htmlcode2 += '<tr>';
				var clickMethod = this.functionPrefix + '.showHideTree(\''+hpSectionId+'List_'+listIdx+'\','+'\''+hpSectionId+'\'); ';
				htmlcode2 += '<td width="20px"><img src="'+cCollapseIcon+'" onclick="'+clickMethod+'" id="'+hpSectionId+'Img_'+listIdx+'"></td>';  // triangle icon
				
				var clickMethod = this.functionPrefix + '.showHideTree(\''+hpSectionId+'List_'+listIdx+'\','+'\''+hpSectionId+'\',true);return false; ';
				htmlcode2 += '<td class="darktext_darklink_bold_12" id="'+hpSectionId+'td_'+listIdx+'" value="'+WCXML.xmlencode(label)+'" >';
				htmlcode2 += '<span style="cursor:default; class="darktext_darklink_bold_12" onclick="'+clickMethod+'"><nobr>' + WCXML.xmlencode(label) +'</nobr></span>';   			
				
				htmlcode2 += '</td>';  
				htmlcode2 += '</tr>';
				
				htmlcode2 += '<tr style="display:none" id="'+ hpSectionId +'List_'+listIdx+'"><td/>';
				htmlcode2 += '<td><div id="'+hpSectionId+'DivList_'+listIdx+'" style="';
				if (this.cascadingLists[list].height) {
					var h = this.cascadingLists[list].height;
					htmlcode2 += 'max-height: '+ h +'px; height: expression(this.scrollHeight > '+ (h-1) + '? \'' + h + 'px\' : \'auto\' );overflow-y:auto;'; 
				} 
			
				htmlcode2 += 'max-width:' + panelWidth +'px; min-width:' + panelWidth + 'px; width:expression(this.scrollWidth > '+ (panelWidth-1)+'? \''+panelWidth+'px\' : \'auto\' );overflow-x:auto; ">';		
				if (listObj.loadingType != 'OnTheFly') {
					htmlcode2 += this.displayCombinationsInTree(list,hpSectionId);
				} else {
					// go loading it now. It will display itself in the correct section.
					this.loadTreeOnTheFly(list,listIdx,this.cascadingLists[list].depth, hpSectionId );
				}
				
				htmlcode2 += '</div></td></tr>';
				
				// grey line below
				htmlcode2 += '<tr><td colspan="2" class="grey2_background" style="padding: 0pt;font-size:0;"><img height="1" width="1" src="images/clearpixel.gif"/></td></tr>';
			}
			htmlcode += htmlcode2;
		}
		
		htmlcode += '</table>';
		panelElem.innerHTML = htmlcode;
		
		// initialize depth
		for (var list=0; list< this.cascadingLists.length; list++) {
			var listIdx = this.cascadingLists[list].listIdx;
			var listObj = this.lists[listIdx];
			var depth = this.cascadingLists[list].depth;
			if (!depth || depth> listObj.columns.length) depth=listObj.columns.length;
		}
		
		if (document.getElementById(hpSectionId+'ImageArea_tr1')) document.getElementById(hpSectionId+'ImageArea_tr1').style.display = '';
		if (document.getElementById(hpSectionId+'ImageArea_tr2')) document.getElementById(hpSectionId+'ImageArea_tr2').style.display = '';  // grey line under it
		this.updateImageArea('allLists', hpSectionId);
		this.currentSelectedTreeId = 'allLists';

		// For front end lists this call has to be done here for the tree and image panel to be restored properly
		if (this.lists[0] && this.lists[0].loadingType != 'OnTheFly') {
			this.initExpandTree(hpSectionId);
		}
		
		
		/*
		 * Paint the dropdown to select between the different styles (LeftPanelOnly, ImagePanelOnly, Both)
		 * Don't paint it again if it's already present on the screen. This can occur when this function 
		 * is called by the "Clear this form" link on the search results screen. 
		 */
		if (!document.getElementById(hpSectionId+'treeDisplaySelect'))
		{
			// figure out visibility. If not allowed to change, the dropdown is painted invisible
			var visible = true;
			var ImageAreaAllowChangeElem = document.getElementById(hpSectionId+'ImageAreaAllowChange');
			if (ImageAreaAllowChangeElem && parseInt(ImageAreaAllowChangeElem.value) == 0) visible=false;
			
			
			var htmlcode = '';
			htmlcode += '<img width="30px" height="1px" src="./images/clearpixel.gif"/> ';
			var changeMethod = this.functionPrefix + '.changeTreeDisplay(this, ' + '\''+hpSectionId+ '\'' + '); ';
			htmlcode += '<select class="darktext_greenlink_11" style="font-size: 11px;" id="'+hpSectionId+'treeDisplaySelect" onchange="'+changeMethod+'">';			
			htmlcode += '<option value="LeftPanelOnly" selected="selected">' + this.langArr['ACatSearchLeftPanelOnly'] + '</option>';
			htmlcode += '<option value="ImagePanelOnly">' + this.langArr['ACatSearchImagePanelOnly'] + '</option>';
			htmlcode += '<option value="LeftAndImagePanel">' + this.langArr['ACatSearchLeftAndImagePanel'] + '</option>';
			htmlcode += '</select>';
			
			var parentElem = document.getElementById(hpSectionId+'searchResultsMainHeaderTr');
			if (parentElem) {
				var newTD = document.createElement('td');				
				newTD.innerHTML = htmlcode;	
				parentElem.appendChild(newTD);	
				if (!visible) document.getElementById(hpSectionId+'treeDisplaySelect').style.display = 'none';
			} else {
				 $("#" + hpSectionId+"leftTreeMainPane").parent().before('<tr><td class="lastCol">' + htmlcode + '</td></tr>');
				 if (!visible) document.getElementById(hpSectionId+'treeDisplaySelect').style.display = 'none';
			}
		}

		// read default image area style from hidden input element and set the dropdown to this value, then paint the appropriate		
		var imageAreaStyle = 'LeftPanelOnly';
		var imageAreaStyleElem = document.getElementById(hpSectionId+'ImageAreaStyle');
		if (imageAreaStyleElem) imageAreaStyle = imageAreaStyleElem.value;
		treeDisplaySelectElem = document.getElementById(hpSectionId+'treeDisplaySelect');
		if (treeDisplaySelectElem) treeDisplaySelectElem.value = imageAreaStyle;
		this.changeTreeDisplay(treeDisplaySelectElem, hpSectionId);

	} catch (err) {
		alert(this.langArr['ACatTreeProblem'] + err.lineNumber + ' : ' + err.message + ' \nlogger' + logger);	
		
	}

	
};

/**
 * Initiates the reset of the asset tree to its previous state if we're returning to the search results
 * screen via a menu item or the back button.
 * 
 *  First, check if there's a session variable with the persisted tree state. If there is, then call the 
 *  doExpandTree() function to recreate the tree structure. 

 *
 * @author GEMU
 * @param {String} hpSectionId - The id of the home page section (blank if not on home page)
 */
attributeCategory.prototype.initExpandTree = function (hpSectionId) {
	'use strict';
	try {
			var sessionVariableName = document.getElementById(hpSectionId+'search_session_variable_name');
			if (!sessionVariableName) return;
			var sessionVariableNameValue = sessionVariableName.value;
			if (!sessionVariableNameValue || sessionVariableNameValue == null ) return;

			var sectId = "";
			if (hpSectionId != null && hpSectionId.length == 17) {
				sectId = hpSectionId.slice(0, -1);
			}

			var url = 'GetSessionVariable.jsp?sessionvariablename='+sessionVariableNameValue+sectId;
			var aCatObj = this;  // closure

			// ajax call
			ajax({
				url: url,				
				type: "GET",
				data: "xml",
				debug: this.debug,
				synchronous: false,
					
				onSuccess: function(sessionVarXML) { 
					try {	
						if (sessionVarXML != null) {
							var xmlMainObj = WCXML.GetFirstFirstLevelChild(sessionVarXML, 'search_criteria');							
							if (xmlMainObj) {
								var xmlTreeObj = WCXML.GetFirstFirstLevelChild(xmlMainObj, 'selectedTreeAttributes');
								if (xmlTreeObj != null) {
									aCatObj.doExpandTree(hpSectionId, xmlTreeObj);
									// The value embedded in the page URL's hash (#) is the 
									// timestamp of the last search re-run and is set in the loadSectionReRun() 
									// function of search.js. When using the browser's back button this timestamp
									// doesn't always equal the timestamp stored in a hidden field in the page.
									// The hidden field's timestamp is set in the pagingStyle template of 
									// searchresultstemplate.xsl. This happens mainly for IE & Chrome. Firefox 
									// is Ok. If there is a mismatch in these two timestamps we need to re-run 
									// the search or the Asset Tree & the listed search results will be out of sync. 
									// For a results screen with a hidden Search button the search will be automatically
									// re-run when the asset tree is restored to its previous state via the call to
									// doExpandTree (above), so we don't need to run the section of code below.
									var hashValue = window.location.hash;
									var submitButton = $('#'+sectId+'_SearchPanelSearchButton');
									if (hashValue.length > 0 && submitButton && submitButton.is(':visible')) {
										var resultTimestamp = $('#'+hpSectionId+'result_timestamp');									
										if (resultTimestamp && (hashValue != resultTimestamp.value)) {
											submitButton.click();
										}
									}
									return;
								}
							}
						}
						aCatObj.doExpandTree(hpSectionId);
					} catch (err) {
						alert("Error processing return of ajax call to GetSessionVariable.jsp in initExpandTree: " + err ); 
					}
				},
				onError: function(status, objHdl, errorMsg){
					aCatObj.handleAjaxError(status,objHdl, errorMsg); 				
				}
			});						
	} catch (err) {
		alert('Problem in attributeCategory.initExpandTree on line: ' + err.lineNumber + ' : ' + err.message + ' \nlogger' + logger);	
	}
};


/**
 * Resets the asset tree to its previous state if we're returning to the search results
 * screen from another screen. 
 * 
 * If the second parameter passed into this function is undefined then we should go to 
 * the page itself to find the element that contains the XML for the restoration of the asset tree.
 *
 * @author GEMU
 * @param {String} hpSectionId - The id of the home page section (blank if not on home page)
 * @param {Object} treeAttributeXMLElemParam - An object containing the selected tree attribute nodes.
 */
attributeCategory.prototype.doExpandTree = function (hpSectionId, treeAttributeXMLElemParam) {
	"use strict";
	try {
		// Find any previously clicked check boxes in order to restore the tree to its correct state
		// hiddenTreeArr will be used to store values in the format 0 ~ 0, 0 ~1 ~ 12, 1 ~ 2 ~ 0 for instance
		var treeAttributeMainElem;

		if (treeAttributeXMLElemParam === undefined) {
			var hiddenTreeAttributeXMLElem = document.getElementById(hpSectionId+'selectedTreeAttributeInputText');

			if (hiddenTreeAttributeXMLElem) {

				var hiddenTreeAttributeXMLString = hiddenTreeAttributeXMLElem.value;		

				if (hiddenTreeAttributeXMLString!= undefined && hiddenTreeAttributeXMLString.constructor == String) {
					var hiddenTreeAttributeXMLInput = WCXML.xmlTextToObj(hiddenTreeAttributeXMLString);
					treeAttributeMainElem = WCXML.GetFirstFirstLevelChild(hiddenTreeAttributeXMLInput, 'selectedTreeAttributes');
				}			
			}			
		} else {
			treeAttributeMainElem = treeAttributeXMLElemParam;
		}

		if (treeAttributeMainElem !== undefined) {
						
			var hiddenTreeArr = WCXML.GetFirstLevelChilds(treeAttributeMainElem, 'selectedTreeAttribute');
	




			if (hiddenTreeArr && hiddenTreeArr.length > 0) {
				

				for (var i=0; i<hiddenTreeArr.length; i++) {
					var treeElemVal = WCXML.textContent(hiddenTreeArr[i]);
					var treeElemArr = treeElemVal.split('~');
					var prefixStr = '';
						
					// Go thru' each level expanding the tree and finally clicking the appropriate checkbox
					for (var j=0; j<treeElemArr.length; j++) {
						prefixStr += '_' + treeElemArr[j].trim();
						if (j == treeElemArr.length-1) {
							var cbObj = document.getElementById(hpSectionId + 'Chkbox' + prefixStr);
							if (cbObj) {
								cbObj.click();
							} 

						} else {	
							var listObj = document.getElementById(hpSectionId + 'List' + prefixStr);
							var imgObj = document.getElementById(hpSectionId + 'Img' + prefixStr);
							if (listObj && imgObj) {
								// expand the tree
								if (imgObj.src.indexOf("collapse") != -1 ) {
									this.showHideTree(hpSectionId + 'List' + prefixStr, hpSectionId); 
								}

							} else {
								//  can't get a reference to the asset tree - throw an error
								throw new Error ('Cannot restore tree to its previous state');
							} 
						}
					}			




				}
			}
		}

	} catch (err) {
		alert('Problem in attributeCategory.doExpandTree on line: ' + err.lineNumber + ' : ' + err.message + ' \nlogger' + logger);	
	}	
};

/**
* @param list - the index in this.cascadingLists
* @para listIdx - the index in this.lists
*/
attributeCategory.prototype.loadTreeOnTheFly = function(list,listIdx,depth, hpSectionId) {
	'use strict';
	try {
		var listObj = this.lists[listIdx];
		var nColumns = listObj.columns.length;
		
		if (depth && depth < nColumns) nColumns = depth;
		var url = 'GetRowsFromList.jsp?listType=WCR_DB_List&outputType=JSON&indexed=1';
		url += '&list=' + WCXML.urlencode(listObj.name);		
		url += '&maxrows=10000';  // To do: make this configurable
		url += '&maxcolrows=1000';
		
		for (var column=0; column< nColumns; column++ ) {
			var columnObj = listObj.columns[column];
			url += '&output=' + WCXML.urlencode(columnObj.columnName);
		}

		var aCatObj = this;  // closure
		
		// ajax call
		ajax({
			url: url,				
			type: "GET",
			data: "text",
			debug: this.debug,
			synchronous: false,
			
			onSuccess: function(jsonContent) {  // To do: split this off in a separate function. 
				try {	
					var jsonObj;					
					try {						
						jsonObj = jsonParse(jsonContent);
					} catch(err) {   // if jsonParse fails, fall back on eval (which is slower)
						jsonObj = eval('(' + jsonContent + ')');
					}
											
					if (!jsonObj.success || jsonObj.success == "0") {
						alert('Cannot get values ' + jsonObj.error.code + ' : '+  jsonObj.error.message);
					} else {
						for (var column2=0; column2 < nColumns; column2++) {
							var columnObj2 = listObj.columns[column2];
							columnObj2.values = jsonObj.columns[column2].values;							
						}
						listObj.combinations = jsonObj.combinations;						
					}	


					if (list != null) {
					var DivId= hpSectionId + 'DivList_'+listIdx;
					document.getElementById(DivId).innerHTML = aCatObj.displayCombinationsInTree(list,hpSectionId);
					// For back end lists this call has to be done here for the tree and image panel to be restored properly
					aCatObj.initExpandTree(hpSectionId);
					}
															

				} catch (err) {
					alert("Error processing return of ajax call to GetRowsFromList.jsp in loadTreeOnTheFly: " + err ); 
					
				}
			},
			
			
			onError: function(status, objHdl, errorMsg){
				aCatObj.handleAjaxError(status,objHdl, errorMsg); 				
			}
		});		
				
				
		
		
	} catch (err) {
		alert('Problem in attributeCategory.loadTreeOnTheFly on line: ' + err.lineNumber + ' : ' + err.message + ' \nlogger' + logger);	
		
	}
}

attributeCategory.prototype.changeTreeDisplay = function (elem, hpSectionId) {
	try {
		if (!elem) return;
		if (hpSectionId == null) hpSectionId = '';
		
		var leftPaneId = hpSectionId + "leftTreeMainPane";
		var leftPaneElem = document.getElementById(leftPaneId);
		if (leftPaneElem) {
			if (elem.value == 'ImagePanelOnly' ) {
				document.getElementById(leftPaneId).style.display = 'none';
			} else {
				document.getElementById(leftPaneId).style.display = '';
				
			}
		}
		// update the currentSelection location when coming back from ImagePanelOnly
		if (elem.value != 'ImagePanelOnly' ) {
			var currentSelectionElem = document.getElementById(hpSectionId+'CurrentSelectionTopLeft');			
			var currentSelectionElem2 = document.getElementById(hpSectionId+'CurrentSelectionImageArea');						
			
			if (currentSelectionElem && currentSelectionElem.innerHTML == '' && currentSelectionElem2) {				
				currentSelectionElem.innerHTML = currentSelectionElem2.innerHTML;				
				var divElem = document.getElementById(hpSectionId+'CurrentSelectionDiv');
				if (divElem) divElem.style.width = leftPaneElem.style.width ;  // if you want to subtract something, you need parseInt because width is a string like 300px
			}
		
		}
		
		if (document.getElementById(hpSectionId+"ImageArea_tr1")) {
			if (elem.value == 'LeftPanelOnly' ) {
				document.getElementById(hpSectionId+"ImageArea_tr1").style.display = 'none';
				document.getElementById(hpSectionId+"ImageArea_tr2").style.display = 'none';
			} else {
				document.getElementById(hpSectionId+"ImageArea_tr1").style.display = '';
				
				this.showHideTree(this.currentSelectedTreeId,hpSectionId,true);
				document.getElementById(hpSectionId+"ImageArea_tr2").style.display = '';
			}
		}
		
		
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.displayCascadingTree' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}

};


attributeCategory.prototype.showHideTree = function (elemId,hpSectionId,show,noUpdateImages) {
	try {
		if (hpSectionId == null) hpSectionId = '';
			
		if (elemId != 'allLists') {
			var elem = document.getElementById(elemId);		    
			var offset = hpSectionId == '' ? 0 : 17;
			var postFix = elemId.substring(4+offset,elemId.length);
			var imgElem =  document.getElementById(hpSectionId+'Img'+postFix);
			
			if (elem) {
				
				if (elem.style.display == 'none' || show) {
					elem.style.display=''; 
					if (imgElem) imgElem.src = cExpandIcon;
					
				} else {
					elem.style.display='none';
					if (imgElem) imgElem.src = cCollapseIcon;
				}
				// select this element
				//this.currentSelectedTreeNode = elemId;
				
			}
		}

		// reset background color of element no longer selected
		var labelElemId;
		if (this.currentSelectedTreeId && this.currentSelectedTreeId != 'allLists') {
			var depth = hpSectionId == '' ? this.currentSelectedTreeId.split('_').length-3 : this.currentSelectedTreeId.split('_').length-5; 
			var offset = hpSectionId == '' ? 0 : 17;
			
			if (depth>=0) {
				labelElemId = hpSectionId+'td' + depth + this.currentSelectedTreeId.substring(4+offset,this.currentSelectedTreeId.length);  // td1_2_1_3
			} else {
				labelElemId = hpSectionId+'td'  + this.currentSelectedTreeId.substring(4+offset,this.currentSelectedTreeId.length);  // td_1
			}
			var labelElem = document.getElementById(labelElemId);
			if (labelElem) labelElem.style.background = '';
		}
		
		this.currentSelectedTreeId = elemId;
		
		var treeDisplaySelectElem = document.getElementById(hpSectionId+'treeDisplaySelect');

		if (treeDisplaySelectElem &&  treeDisplaySelectElem.value != 'LeftPanelOnly') {
			// change Background color of selected element
			// elemId looks like List_0_1
			// we need to find the element td0_0_1
			var depth = hpSectionId == '' ? elemId.split('_').length-3 : elemId.split('_').length-5; 			
			var offset = hpSectionId == '' ? 0 : 17;
			if (depth>=0) {
				labelElemId = hpSectionId+'td' + depth + elemId.substring(4+offset,elemId.length);  // td1_2_1_3
			} else {
				labelElemId = hpSectionId+'td'  + elemId.substring(4+offset,elemId.length);  // td_1
			}
			var labelElem = document.getElementById(labelElemId);
			if (labelElem) labelElem.style.background = '#d0d0d0';
		}
		
		if (!noUpdateImages) this.updateImageArea(elemId, hpSectionId);
		
		
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.showHideTree' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}
	
};

/**
* @param elemId the id of the currently selected element in the tree
*/
attributeCategory.prototype.updateImageArea = function (elemId, hpSectionId) {
	try {
		if (hpSectionId == null) hpSectionId = '';
		var offset = hpSectionId == '' ? 0 : 17;
		var htmlcode ='';
		var imageAreaElem = document.getElementById(hpSectionId+"ImageArea");				
		var imageAreaTr = document.getElementById(hpSectionId+"ImageArea_tr1");
		if (imageAreaTr && imageAreaTr.style.display == 'none') return;	
		var functionPrefix = this.functionPrefix;
		var onloadFunction = 'WCHTML.scaleImageProportionally(this,100,100);';
		var listName = 'Pepsi';
		var onerrorFunction =  'WCHTML.alternativeImage(this,null);';
		var attributeValue,selector;
		var column = null;
		
		var curWidth = $('#'+hpSectionId+'ImageArea').width() - 230;		
		htmlcode+='<table><tr>';
		
		// Current Selection: copy from the top left current selection if that one is hidden
		// updateImageArea is done multiple times so after the first copy, the section is gone, so you need to copy from the image area before whiping it
		var currentSelectionElem = document.getElementById(hpSectionId+'CurrentSelectionTopLeft');
		var treeDisplaySelectElem = document.getElementById(hpSectionId+'treeDisplaySelect');
		var currentSelectionElem2 = document.getElementById(hpSectionId+'CurrentSelectionImageArea');
		var currentSelectionTableElem = document.getElementById(hpSectionId+'CurrentSelectionTable');
		if (currentSelectionElem && treeDisplaySelectElem && currentSelectionTableElem && treeDisplaySelectElem.value == 'ImagePanelOnly' ) {
			//<div style="width:' + (panelWidth + 20) +';overflow-x:auto"><table id="CurrentSelectionTable">
			//var htmlcode100 = currentSelectionElem.innerHTML;
			//if (!htmlcode100 && currentSelectionElem2) htmlcode100 = currentSelectionElem2.innerHTML;
			var htmlcode100 =''; 
			htmlcode100 += '<div style="width:150px;overflow-x:auto" id="' + hpSectionId+'CurrentSelectionDiv"><table id="'+hpSectionId+'CurrentSelectionTable">';
			htmlcode100 += currentSelectionTableElem.innerHTML;
			htmlcode100 += '</table></div>';
			htmlcode += '<td valign="top" id="'+hpSectionId+'CurrentSelectionImageArea">' + htmlcode100 + '</td>';			
			curWidth -= 152;
			currentSelectionElem.innerHTML = '';			
		}
		
		if (elemId != 'allLists') {
			// find the currently selected node. this is something like 'List_0_5' 
			var elemArr = elemId.split('_');
			var list = hpSectionId == '' ? elemArr[1] : elemArr[3];			
			column = hpSectionId == '' ? elemArr.length-2 : elemArr.length-4;
			listName = this.lists[list].name; 
			onerrorFunction = 'WCHTML.alternativeImage(this,\'./custom/AttributeValueImages/'+listName+'.jpg\');';
				
			// go back icon
			htmlcode+='<td style="vertical-align:top;"><table>';
			if (column>0) {
				var parentId = hpSectionId + 'List_' + list;
				for (var col=0; col< column-1; col++) { 
					if (hpSectionId == '') parentId += '_' + elemArr[col+2];
					else parentId += '_' + elemArr[col+4];						
				}
			} else {
				parentId =  'allLists';
			}

			var clickMethod = functionPrefix + '.showHideTree(\''+parentId+'\',\''+hpSectionId+'\',true); ';
			htmlcode+='<tr height="100px" valign="top"><td style="height:100px;vertical-align:middle;"><img src="./images/arrow-left-icon-48.png" onclick="'+clickMethod+'"/></td></tr>';
			htmlcode+='</table></td>';
			
			// first the currently selected element
			if (column>0) {
				selector = hpSectionId+'td'+(column-1)+ elemId.substring(4+offset,elemId.length);	
			} else {
				selector = hpSectionId+'td_'+ list;
			}
			attributeValue = WCXML.xmlencode($('#'+selector+'').attr("value"));		
			htmlcode += '<td valign="top"><table>';
			htmlcode+='<tr><td style="height:100px;vertical-align:middle;"><img src="./custom/AttributeValueImages/'+attributeValue+'.jpg" height="100" onload="'+onloadFunction+'" onerror="'+onerrorFunction+'"></td></tr>';
			htmlcode += '<tr align="center" valign="top"><td class="darktext_greenlink_11"  valign="top">'+attributeValue+'</td></tr>';
			htmlcode += '</table></td>';
			htmlcode+='<td><img src="./images/clearpixel.gif" width="70px"/></td>';
		}	
		
			
		
		// scrolling area with thumbnails
		htmlcode += '<td><div style="width:'+curWidth+';overflow-x:auto;" id="CTree_MainImageDiv"><table id="CTree_MainImageTable"><tr>';
				
		if (elemId != 'allLists') {
			selector = hpSectionId+'td'+column+ elemId.substring(4+offset,elemId.length)+'_';			
		} else {
			selector = hpSectionId+'td_';	
		}	
		
		$tdElems = $('td[id^="'+selector+'"]');	
		
		$tdElems.each( function(index) {
            var attributeValue = WCXML.xmlencode($(this).attr("value"));
            var tdId = $(this).attr("id");
            tdId = tdId.substring(offset);
            var startPos = tdId.indexOf("_");
			var suffix = tdId.substring(startPos,tdId.length);

			var targetId = hpSectionId + 'List' + suffix;  // convert from td12_0_1 to List_0_1
			htmlcode += '<td valign="top"><table>';
			
			// determine clickmethod
			var clickMethod = functionPrefix + '.showHideTree(\''+targetId+'\',\''+hpSectionId+'\',true); ';	// Normal situation
			var expandImgId = hpSectionId + 'Img' + suffix;   // trick: look whether this element has an expand icon in the base tree. If not: don't allow expanding
			if (!document.getElementById(expandImgId)) {
				clickMethod = 'document.getElementById(\'Chkbox'+suffix+'\').click()';
			}
			
			htmlcode += '<tr align="center"  valign="top"><td style="height:100px;vertical-align:middle;"><img src="./custom/AttributeValueImages/'+attributeValue+'.jpg" height="100" onclick="'+clickMethod+'" onload="'+onloadFunction+'" onerror="'+onerrorFunction+'"></td></tr>';
			
			if (elemId != 'allLists') {
				var checked = document.getElementById(hpSectionId+'Chkbox'+suffix).checked;
				var clickMethod = 'document.getElementById(\''+hpSectionId+'Chkbox'+suffix+ '\').click()';
				htmlcode += '<tr align="center"><td><input type="checkbox" id="'+hpSectionId+'ImgCB'+suffix+'" onclick="'+clickMethod+'"';
				if (checked) htmlcode += ' checked ';
				htmlcode += '/></td></tr>';
			}
			htmlcode += '<tr align="center" valign="top"><td class="darktext_greenlink_11"  valign="top">'+attributeValue+'</td></tr>';
			htmlcode += '</table></td>';
			// add spacer
			htmlcode += '<td><img src="./images/clearpixel.gif" width="30px"></td>';
		});
		htmlcode += '</tr></table></div></td>';
		// end scrolling area with thumbnails
		
		// close the main table
		htmlcode += '</tr></table>';
		
		imageAreaElem.innerHTML = htmlcode;
		
		// fix up Internet Explorer: when there is a scrollbar, the bottom text covered if there are 3 lines --> calculate the right height and set it the hard way
		// Other browsers do this by themselves.
		if ( detectIfIE() ) {
			var divElem = document.getElementById('CTree_MainImageDiv');
			var tableElem = document.getElementById('CTree_MainImageTable');
			if (divElem && tableElem && divElem.clientWidth < tableElem.clientWidth) {
				// there is a scrollbar
				divElem.style.height = tableElem.clientHeight + 13 ;												
			}
		}
		
			
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.updateImageArea' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}
};

/**
* Each element looks as follows:
* <tr><td><img triangle></td><td><chkbx></td><td>label</td></tr>
* <tr><td/><td/><td><table>  INCLUSING OF THE SAME </table></td></tr>
* Algorithm:
* At the end of each combination, we make sure we stand in the HTML where the next thing needs to come
* This will obviously demand to look at the next one...
* @param list  Index of the list in this.lists
*/

attributeCategory.prototype.displayCombinationsInTree = function (list,hpSectionId) {
	try {
		var listIdx =  this.cascadingLists[list].listIdx;  // index in this.lists while list is the index in cascadingLists
		var listObj = this.lists[listIdx];
		if (!listObj) throw new Error("listObj " + this.langArr['ACatListIndexProblem'] + listIdx);
		if (hpSectionId == null) hpSectionId = '';
				
		var nColumns = listObj.columns.length;  
		var depth = this.cascadingLists[list].depth;
		if (depth>0 && depth<nColumns) nColumns = depth;
		
		var col,curCombiNext,valueArrNext;
		var firstDiffCol = 0, nextFirstDiffCol=0;
		var htmlcode = '<table>';
		var htmlcode2,htmlcode3;
		
		var nCombinations= listObj.combinations.length;  // when depth is changed or restrictions, you should probably relaunch a quest for the combinations...
		
		var htmlArr = new Array();
		var combi,combiNext;
		for (combiNext=0,combi=0;combiNext<nCombinations; ) {
			htmlcode2= '';
			
			combi = combiNext;  // so initializes to 0
			var curCombi = listObj.combinations[combi];
			var valueArr = curCombi.split("~");

			var labelArr = new Array();
			for (col=0; col<nColumns;col++) {
				labelArr[col] =  listObj.columns[col].values[valueArr[col]];
			}
			
			// figure out how many identicals in the first depth columns
			
			combiNext=combi+1;
			while (combiNext<nCombinations) {
				valueArrNext = listObj.combinations[combiNext].split("~");
				for (col=0; col<nColumns;col++) {
					if (valueArrNext[col]!=valueArr[col]) break;
				}
				if (col != nColumns) break;
				combiNext++;
			}
			// combiNext is now the next combination which is different from combi in the first depth columns and valueArrNext is its valueArr
					
			
			
			if (combiNext == nCombinations) {
				valueArrNext = new Array();
				for ( col=0; col<nColumns; col++) valueArrNext[col]=-1;
			}
			
			
			var postFix = listIdx;
			// determine how deep to paint (don't paint empty columns)
			for (col=nColumns-1; col>=0; col--) {
				if (labelArr[col]!='' && labelArr[col]!= ' ') break;
			}
			lastNonEmptyColumn = col;
			
			// Now look at the next one and see how much we must close	
			firstDiffCol = nextFirstDiffCol;
			
			nextFirstDiffCol = nColumns;			
			for ( col=0; col<nColumns; col++) {
				if (parseInt(valueArr[col])!= parseInt(valueArrNext[col])) {
					nextFirstDiffCol = col; break;
				}			
			}		
			if (firstDiffCol == nColumns)  {
				// this combination is identical to the previous until the requested depth
				// so don't paint anything
				htmlArr[combi] = '';
				
			} else {
				if (nextFirstDiffCol > lastNonEmptyColumn) lastNonEmptyColumn = nextFirstDiffCol;  // DT121408

				for ( col=0; col<firstDiffCol; col++) postFix += '_' + valueArr[col];
				// paint this combination. This might be several rows	
				//for (col=firstDiffCol; col<nColumns; col++) {			
				for (col=firstDiffCol; col<=lastNonEmptyColumn; col++) {
					htmlcode3 = '';
					// paint this column -- We assume we are at the right position now
					htmlcode3 += '<tr>';
					postFix += '_' + valueArr[col];

					var clickMethod = this.functionPrefix + '.showHideTree(\''+hpSectionId+'List_'+postFix+'\','+'\''+hpSectionId+'\'); ';
					var canExpand = (col!= lastNonEmptyColumn || col<nextFirstDiffCol);
					
					if (canExpand) {
						htmlcode3 += '<td><img src="'+cCollapseIcon+'" id="'+hpSectionId+'Img_'+postFix+'"   onclick="'+clickMethod+'"/></td>';  // triangle icon
					} else {
						htmlcode3 += '<td></td>';
					}
					
					// checkbox
					clickMethod = this.functionPrefix + '.clickTreeViewChkbox(this,\'' + hpSectionId + '\'); ';
					
					//var label =  listObj.columns[col].values[valueIdx];
					var label =  WCXML.xmlencode(labelArr[col]);
					
					htmlcode3 += '<td><input type="checkbox" id="'+hpSectionId+'Chkbox_'+postFix+'" value="' + label +'" onclick="'+clickMethod+'"/></td>';  // checkbox				
					
					clickMethod = this.functionPrefix + '.showHideTree(\''+hpSectionId+'List_'+postFix+'\','+'\''+hpSectionId+'\',true); ';
					if (!canExpand) {
						clickMethod = 'document.getElementById(\'Chkbox_'+postFix+'\').click()';
					}
					
					htmlcode3 += '<td  class="darktext_darklink_11" value="' + label +'" id="'+hpSectionId+'td'+col+'_'+postFix+'" >';
					htmlcode3 += '<span style="cursor:default; class="darktext_darklink_bold_12" onclick="'+clickMethod+'"><nobr>' + label +'</nobr></span>';    
					htmlcode3 += '</td>';
					//this.preLoadImage(label);
					// consider putting the structure in listObj.tree, which looks like [{value:col1Val,children:[{value: col2Val},...]},{},...]
					// this would make it very easy to implement images.
					
					htmlcode3 += '</tr>';
					//if (col<nColumns-1) {
					if (col<lastNonEmptyColumn) {
						htmlcode3 += '<tr style="display:none" id="'+hpSectionId+'List_'+postFix+'"><td/><td/><td><table>';
					}
					htmlcode2 += htmlcode3;
				}
				


				//for ( col=nColumns-2; col>=firstDiffCol; col--) {
				// close tables if you need to move to the left
				htmlcode3 = '';
				for ( col=lastNonEmptyColumn-1; col>=nextFirstDiffCol; col--) {
						htmlcode3 += '</table></td></tr>';
				}
				// but it's also possible you need to move to the right
				if (lastNonEmptyColumn>=0 && nextFirstDiffCol > lastNonEmptyColumn) {
					htmlcode3 += '<tr style="display:none" id="'+hpSectionId+'List_'+postFix+'"><td/><td/><td><table>';
				}
				htmlcode2 += htmlcode3;
				
				
				htmlArr[combi] = htmlcode2;
				if (htmlcode2 == ''  && lastNonEmptyColumn == -1) nextFirstDiffCol = 0;  // DT121408 nothing painted, all the way to the left, typically a totally empty line. Must paint from beginning thereafter.
				//htmlcode += htmlcode2;			
			}
		}
		htmlcode += htmlArr.join('');
		htmlcode += '</table>';		
		
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.displayCombinationsInTree' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
		htmlcode='';
	}
	return htmlcode;
};

attributeCategory.prototype.clickTreeViewChkbox = function (chkBoxElem, hpSectionId) {
	try {
		if (hpSectionId == null) hpSectionId = '';
		var chkBoxId = chkBoxElem.id;
		var idItemArr = chkBoxId.split("_");
		// Offsets are different depending on whether we're on a normal search results page or a My Work page. 
		var offset = 0; var offset2 = 0; var offset3 = 0;
		if (hpSectionId != '') {
		  offset = 2;
		  offset2 = 16;
		  offset3 = 17;
		}		
		var listIdx = parseInt(idItemArr[1+offset],10);
		
		var col;
		var listObj = this.lists[listIdx];

		if (!listObj.selectedCombinations) listObj.selectedCombinations = new Array();
		// find the current one and update selectedCombinations for this list
		
		for (var idx=0; idx<listObj.selectedCombinations.length; idx++) {
			if (chkBoxId == listObj.selectedCombinations[idx]) break;
		}
		if (chkBoxElem.checked) {
			if (idx>= listObj.selectedCombinations.length) listObj.selectedCombinations.push(chkBoxId);
			// DT107466: Wrong search results when selecting in the tree view 2 items where 1 is a parent of the other 
			// uncheck all checkboxes of child elements and checkbox of parent element
			// also remove them from selectedCombinations and image panel
			
			var selectedCombinationsCopy = listObj.selectedCombinations;
			for (var idx=0; idx<selectedCombinationsCopy.length; idx++) {
				selectID = selectedCombinationsCopy[idx];
				// if name of iterated checkbox starts with name of selected checkbox, it's a child
				// if name of selected checkbox starts with name of iterated checkbox, it's a parent
				if((chkBoxId.indexOf(selectID + '_') == 0 || selectID.indexOf(chkBoxId + '_') == 0) && chkBoxId != selectID){
					chkbxElem = $("#"+selectID);
					if(chkbxElem.attr('checked')){
						chkbxElem.removeAttr('checked');
						var oldSelected = listObj.selectedCombinations;
						listObj.selectedCombinations = new Array();
						for (var j=0; j<oldSelected.length; j++) {
							if(oldSelected[j] != selectID){
								listObj.selectedCombinations.push(oldSelected[j]);
							}
						}
						// if this element is visible in the image area: update its checked value
						var imageChkBoxId = hpSectionId+'ImgCB'+selectID.substring(6+offset3,selectID.length);
						if (document.getElementById(imageChkBoxId)) {
							document.getElementById(imageChkBoxId).checked = false;
						}
					}
				}
			}
			// end DT107466
			
		} else {
			if (idx< listObj.selectedCombinations.length) listObj.selectedCombinations.splice(idx,1);
		}
			
		// Update all search fields for this search
		for (col=0;col<listObj.columns.length; col++) {
			// find the attribute
			var colObj = listObj.columns[col];
			if (!colObj) continue;
			var attObj = colObj.attObj;
			if (!attObj) continue;			
			
			// calculate new search string
			var searchString = '';
			for (var idx=0; idx<listObj.selectedCombinations.length; idx++) {
				var combiArr = listObj.selectedCombinations[idx].split("_");
				if (col+offset+2< combiArr.length && combiArr[col+offset+2]!='') {					
					var value =  '\"' + WCXML.QuoteEncode(colObj.values[combiArr[col+offset+2]],true) + '\"'; 
					if (searchString.indexOf(value) == -1) {
						// OR replaced by ||. Makes the construction of the lucene query
						// much easier for use with Un-Tokenized attributes - DT108230.
						if (searchString) searchString += ' || ';					
						searchString += value ;  
					}
				} 
			}	
			if (searchString) {
				// Fix for DT105899
				// If this column/attribute has a search string, we need to add all combinations for those selected combinations where this column had no selection (which is to be interpreted as "all"). 
				try {
					for (var idx=0; idx<listObj.selectedCombinations.length; idx++) {
						var combiString = listObj.selectedCombinations[idx];  // something like Chkbox_0_7  where the first index is the list
						var combiArr = combiString.split("_");		
						
						if (col+offset+2 >= combiArr.length || combiArr[col+offset+2]=='') {
							// this column had no value for this combination
							// find all values for this combination
							var combiString2 = combiString.substring(6+offset3);  // looks like _0_7
							var selector = hpSectionId+'td'+ col + combiString2 + "_";			
							var $tdElems = $('td[id^="'+selector+'"]');
							
							$tdElems.each( function(index) {
								var value =  '\"' + WCXML.QuoteEncode($(this).attr("value"),true) + '\"';  
								if (searchString.indexOf(value) == -1) {
									// OR replaced by ||. Makes the construction of the lucene query
									// much easier for use with Un-Tokenized attributes - DT108230.
									if (searchString) searchString += ' || ';					
									searchString += value ;  
								}
								
							});
							
							
						} 
					}
				} catch(err) {
					alert(this.langArr['ACatProblemIn'] + ' attributeCategory.clickTreeViewChkbox' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	// To do: remove this alert.
				}
				
			
			}
						
			// update the (typically hidden) search fields
			var searchInputFieldName = hpSectionId + 'db_attr_input_'+ attObj.id;
			/*
			var searchInputFieldArr = document.getElementsByName(searchInputFieldName);
			if (searchInputFieldArr && searchInputFieldArr.length>0) {
				var searchInputField = searchInputFieldArr[0];
				searchInputField.value = searchString;
			}
			*/
	
			var searchInputField = document.getElementById(searchInputFieldName);			
			if (!searchInputField && searchString) {  // try creating it				
				var htmlcode = '<tr style="display:none;">';
				var tdId = hpSectionId? (hpSectionId + 'db_attr_td_'+ attObj.id) : ('db_attr_td_'+ attObj.id);
				htmlcode += '<td id="' + tdId + '">';
				htmlcode += '<input type="hidden" value="" id="'+searchInputFieldName+'" name="'+searchInputFieldName+'"/>';
				
				var theName = 'db_attr_' + attObj.id + '_rerun'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="1" id="'+theName+'" name="'+theName+'"/>';
				
				var theName = 'db_attr_hidden_attr_id'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="'+ attObj.id +'" id="'+theName+'" name="'+theName+'"/>';
				
				// in the below, att versus attr is NOT a typo.  This is an inconsisteny in the API
				var theName = 'db_attr_hidden_att_name_' + attObj.id; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="'+ WCXML.xmlencode(attObj.name)+'" id="'+theName+'" name="'+theName+'"/>';
				
				var theName = 'db_attr_hidden_units_' + attObj.id; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="TEXT" id="'+theName+'" name="'+theName+'"/>';

				var theName = 'db_attr_' + attObj.id + '_temp_attr'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="1" id="'+theName+'" name="'+theName+'"/>';
				
				var theName = 'db_attr_' + attObj.id + '_selected_from_list'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="0" id="'+theName+'" name="'+theName+'"/>';
				
				var theName = 'db_attr_' + attObj.id + '_sequence'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="" id="'+theName+'" name="'+theName+'"/>';

				var theName = 'db_attr_db_attr_' + attObj.id + '_sort_order'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="" id="'+theName+'" name="'+theName+'"/>';
				
				var theName = 'db_attr_db_attr_' + attObj.id + '_sort_number'; if (hpSectionId) theName = hpSectionId + theName;
				htmlcode += '<input type="hidden" value="" id="'+theName+'" name="'+theName+'"/>';
			
				htmlcode += '</td></tr>';					
					
				// append 				
				$("#" + hpSectionId+"parmsTableBody").append(htmlcode);
				searchInputField = document.getElementById(searchInputFieldName);
			} 
			if (searchInputField) {
				searchInputField.value = searchString;
			} else {
				if (searchString) alert(this.langArr['ACatSearchPanelFieldProblem']  + attObj.name);
				
			}
			
		}
		
		
		
		// update overview of current selection (top left)
		var currentSelectionHTML = '<table>';  // the HTML which will be displayed
		var currentSelectionValue = '';  // the hidden values which will be submitted (the fields are in the parameter form)

		for (var listCt=0; listCt<this.lists.length; listCt++) {
			var selListObj = this.lists[listCt];
			if (selListObj.selectedCombinations) {
				for (var idx=0; idx<selListObj.selectedCombinations.length; idx++) {
		
					var chkbxId = selListObj.selectedCombinations[idx];
					var combiArr = chkbxId.split("_");
					var curString = selListObj.treeListObj.label? selListObj.treeListObj.label: selListObj.name;
					var curString2 = listCt;
					
					for (col=0;col<selListObj.columns.length; col++) {				
		
						if (col+offset+2<combiArr.length) {
							
							curString += ' ~ ' + selListObj.columns[col].values[combiArr[col+offset+2]]  ;
							curString2 += ' ~ ' + combiArr[col+offset+2];
						}
					}
					if (listCt == listIdx) {
						var clickMethod = this.functionPrefix + '.deleteCurrentTreeSelection(\''+ chkbxId+'\',\'' + hpSectionId + '\'' + ');';
						currentSelectionHTML += '<tr><td><img src="./images/delete_icon_15.png" onclick="'+clickMethod+'"></td>';
						currentSelectionHTML += '<td class="darktext_greenlink_11"><nobr>' +WCXML.xmlencode(curString) + '</nobr></td></tr>';
					}
					currentSelectionValue += '<input type="hidden" value="'+WCXML.xmlencode(curString2)+'" name="'+hpSectionId+'SelectedTreeAttributes">';  
				}
			}
		
		}
		currentSelectionHTML += '</table>';
		currentSelectionElem = document.getElementById(hpSectionId+"currentSelection_" + listIdx);
		if (currentSelectionElem) currentSelectionElem.innerHTML = currentSelectionHTML;   // this thing demands time on IE because every DOM operation does once the big tree is loaded...
		var selectedTreeAttributeDiv = document.getElementById(hpSectionId+'selectedTreeAttributeDiv');
		if (selectedTreeAttributeDiv) {
			selectedTreeAttributeDiv.innerHTML = currentSelectionValue;
		}
		
		// if this element is visible in the image area: update its checked value
		var imageChkBoxId = hpSectionId+'ImgCB'+chkBoxId.substring(6+offset3,chkBoxId.length);
		
		if (document.getElementById(imageChkBoxId)) {
			document.getElementById(imageChkBoxId).checked = chkBoxElem.checked;
		}

		// Only try to auto re-run the search if we're on a search results screen.
		if (hpSectionId != null && (typeof searchAutoReRun  == 'function')) 
		  searchAutoReRun(hpSectionId);

	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.clickTreeViewChkbox' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}


};

/**
* called when you click on a button defined in the category
* @param {Obj} buttonObj - Object of the button to be executed. If null, elem will be checked to find the button
* @param {DOMObj} elem - the button DOM element. Required if buttonObj == null. Its id looks like 000002_0003233423ACatButton~1~0~0. First part is an ID which never contains a ~
* @param {String} clickType - on of 'onLoad', 'onSave', 'click', 'onChange'
*/

attributeCategory.prototype.clickACatButton = function (buttonObj, elem, clickType) {
	"use strict";
	var errorObj=null;
	try {
		var row,col, cellObj, buttonObj,  button;
	
		if (!buttonObj) {
			var arr = elem.id.split("~");
			row = arr[1];
			col = arr[2];
			button = arr[3];
			
			cellObj = this.cells[row][col];
			buttonObj = cellObj.buttons[button];
		}
		
		if (buttonObj) {
			var listObj = buttonObj.listObj;
			switch (buttonObj.func) {  
				case 'LookUpAttributes':
					errorObj = this.clickACatButtonLookUpAttributes(buttonObj, clickType=='onSave');
					break;
				case 'JavaScript':
					errorObj = this.clickACatButtonJavaScript(buttonObj, clickType);
					break;				
				case 'CheckUniqueValue':
					// find the attObj for which uniqueness needs to be tested
					var attObj = this.findAttObjToCheck(buttonObj);
					
					if (attObj) this.clickACatButtonCheckUniqueValue(attObj);  // currently not foreseen on error handling since treated separately in validation
					break;
				default:
					alert('This function is not implemented:' + buttonObj.func);
			}
			buttonObj.clicked++;
		}
	
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.clickACatButton' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	} 
	return errorObj;

}

/**
* returns the attObj of the cell in which a button resides. If this cell does not have an attibute, returns null.
* typically used for finding the attribute to check uniqueness on when its button is clicked (the buttonObj does not know itself where it is)
*/
attributeCategory.prototype.findAttObjToCheck = function(buttonObj) {
	"use strict";
	var attObj=null;
	try {
		var found = false;
		for (var row=0; row < this.nRows  && !found; row++) {
			if (!this.cells[row]) continue;
            for (var col=0;col < this.nCols  && !found; col++) {
                var cellObj = this.cells[row][col];
				if (cellObj) {		
					if (cellObj.buttons && cellObj.buttons.length > 0) {

						for (var button = 0; button < cellObj.buttons.length; button++) {
							if (buttonObj == cellObj.buttons[button]) {
								found = true;
								break;
							}
							
						}
						attObj = cellObj.attObj;
					}				
				}
			}
		}
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.findAttObjToCheck' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	} 
	return attObj;
}

/**
* called when you click on a button of the type LookUpAttributes
* @param {Object.<buttonObj>} buttonObj: The buttonObj for the clicked button
* @param {Boolean} synchronous - Whether the look up call will freeze the UI waiting until the answer comes back, only advised when validating the form
* @param {Obj} errorObj - validation error object returned only when the call is synchronous
*/

attributeCategory.prototype.clickACatButtonLookUpAttributes = function (buttonObj, synchronous) {
	"use strict";
	var errorObj=null;
	try {

		var listObj = buttonObj.listObj;			
		var arr, att;
		var url = 'GetRowsFromList.jsp?';
		var inputsGiven = false;
		if (synchronous !== true) synchronous = false;  // false, undefined, 0 (even 1...) all go to false
		
		// add list
		if (!listObj) throw new Error ('list is not or no longer defined');
		url += 'list=' + WCXML.urlencode(listObj.name);
		url += '&listType=WCR_DB_List';  
		url += '&maxrows=1';   // no maxcolrows needed when not indexed
		url += '&outputType=XML';
		
		arr= buttonObj.inputAtts;
		if (arr && arr.length >0) {						
			for (att=0; att< arr.length; att++) {
				if (arr[att].attObj && arr[att].attObj.value !== '' && arr[att].columnObj) {
					var attValue = arr[att].attObj.value;
					url += '&input=' + WCXML.urlencode(arr[att].columnObj.columnName) + ':' + (arr[att].exact? '1' : '0') + ':'  + WCXML.urlencode(attValue); 
					inputsGiven = true;
				}
			}
		}
		var arr= buttonObj.outputAtts;
		
		if (arr && arr.length >0) {								
			for (att=0; att< arr.length; att++) {							
				// url += '&output=' + WCXML.urlencode(arr[att].columnName);
				// To do: bring this back when output works correctly on the JSP call
			}
		}
		
		if (!inputsGiven) {alert(this.langArr['ACatNoInputValues']); return;}
		
		var aCatObj = this;  // closure and also buttonObj is a closure
		
		ajax({
			url: url,				
			type: "GET",
			data: "xml",
			debug: this.debug,
			synchronous: synchronous,   // synchronous only when called from validation 
			
			
			/* Ajax call returns an XML like following:
				<result>
					<header>
						<column>trademark</column>
						<column>brand</column>
						<column>flavor</column>
						<column>bfn1</column>
						<column>brand_formula_number</column>
						<column>atc_brand_code</column>
						<column>brand_nickname</column>
					</header>
					<row>
						<value>Retired Tropicana Juice Drinks</value>
						<value>Sugar Free Fruit Punch</value>
						<value>Acai Fruit Punch</value>
						<value>2723</value>
						<value/>
						<value>42</value>
						<value>Tampico</value>
					</row>
				</result>

				*/
		
			onSuccess: function(returnXML) {	
				try {
					var column;
					var mainElem = WCXML.GetFirstFirstLevelChild(returnXML,"result");
										
					var headerElem =  WCXML.GetFirstFirstLevelChild(mainElem,"header");
					var columnNameArr =  WCXML.GetFirstLevelChilds(headerElem,"column");
					var columnNames = [];  // list of columnnames as they returned from the call
					
					var rowElem = WCXML.GetFirstFirstLevelChild(mainElem,"row");	// only first row is read						
					var columnArr = WCXML.GetFirstLevelChilds(rowElem,"value");
					
					// read the column names first
					if (columnNameArr && columnNameArr.length>0) {
						for (column=0; column <	columnNameArr.length; column++) {
							columnNames.push(WCXML.GetElementSelfTextContent(columnNameArr[column]));
						}
					}
					
					// read the only row we read...
					if (columnArr && columnArr.length>0) {
						var arr = buttonObj.outputAtts;
						for (var column=0; column < columnArr.length; column++) {
							var columnName = columnNames[column];
							for (var att=0; att < arr.length; att++) {
								if (arr[att].columnObj && arr[att].columnObj.columnName == columnName) {
									var newValue = WCXML.GetElementSelfTextContent(columnArr[column])
									if (newValue != arr[att].attObj.value) {
										arr[att].attObj.value = newValue;							
										aCatObj.onChangeAttValue( arr[att].attObj );										
									}
									break;
								}
							}
						}
					} else {						
						if (synchronous) {
							errorObj = {errorMsg : langArr['ACatNoFittingRow'], errorCode : 'ACatNoFittingRow'};
						} else {
							alert(aCatObj.langArr['ACatNoFittingRow']);
						}
					}
					aCatObj.displayAttributeCategory();
				} catch (err) {
					alert("Error processing return of ajax call to GetRowsFromList.jsp: " + err ); 
				}
			},
			
			
			onError: function(errorMsg){
				alert("Error in Ajax call to GetRowsFromList.jsp " + errorMsg);
				if (synchronous) errorObj = {errorMsg : 'Error in Ajax call to GetRowsFromList.jsp', errorCode : 'Error in Ajax call to GetRowsFromList.jsp'};
			},				
			objHdl: this
		});
		 
	
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.clickACatButtonLookUpAttributes' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	} 
	return errorObj;

}

/**
* called when you click on a button of the type JavaScript
* @param {Object} buttonObj: The buttonObj for the clicked button
* @param {Boolean} clickType - determines whether the call is done at Save time, typically to suppress alerts
*/

attributeCategory.prototype.clickACatButtonJavaScript = function (buttonObj, clickType) {
	"use strict";
	var errorObj=null;
	var docatt, projatt, bomdocatt;
	try {
		
		var inProjectCreation = false;
		var projectAttElem = document.getElementById('projectAttributesBlockId');
		if (projectAttElem) inProjectCreation = true;  // To do: evaluate whether a more robust check can be done, f.e. via this.type		
		
		// construct the att Object
		var att = {};
		var attOld = {};
		for (var attIdx = 0;  attIdx < this.attributes.length; attIdx++) {
			var attObj = this.attributes[attIdx];
			if ( attObj && attObj.placed) {
				//att[attObj.name] = attObj.value;  // almost all strings, except dates which are of Date type
				att[attObj.name] = this.getCalculationValue(attObj);
				attOld[attObj.name] = this.getCalculationValue(attObj);
				
			}
		}
				
		// construct projatt
		if (buttonObj.javaScript.indexOf('projatt') != -1 ) {
		
			if (inProjectCreation) {			
				// try constructing a projatt object				
				if (projectAttElem && projectAttElem.ACatObj && projectAttElem.ACatObj.attributes) {
					projatt = {};
					var projAttArr = projectAttElem.ACatObj.attributes;
					for (var attIdx = 0;  attIdx < this.attributes.length; attIdx++) {
						var attObj = projAttArr[attIdx];
						if ( attObj && attObj.placed) {
							projatt[attObj.name] = this.getCalculationValue(attObj);
						}
					}
				}				
			
			} else {
				// always go reading it, even if it was loaded already for copy from project att to task specifications. Reason: not the same attributes! You need all project attributes, and really the latest.
				projatt = this.getProjectAttributeValuesForButton();  // synchronous ajax call			
			}
		}
		
		// construct docatt
		docatt = [];
		bomdocatt = [];
		if (buttonObj.javaScript.indexOf('docatt') != -1 ) this.createDocAttForButton(docatt, bomdocatt);   // the if will also detect bomdocatt since docatt is part of it
	
		
		var dummyObj = { 
			shiftDate : function(date, days, hours) { return new Date(date.getTime() + days*86400000 + hours * 3600000);} 		
		};  // dummy object created  to have executeJavaScriptButton point the keyword 'this' to this dummy object instead of the ACatObj.  Used to provide utility functions
		if (gCustomFormExtension) dummyObj = gCustomFormExtension;
		errorObj = this.executeJavaScriptButton.call(dummyObj,buttonObj.javaScript, att, projatt, buttonObj.label, clickType, docatt, bomdocatt);  // 'call' takes 1 extra parameter, which is the definition of this. 
		
		// Validate and update the values - To Do: think of moving this in a separate function
		var msg = '';
		for (var attIdx = 0;  attIdx < this.attributes.length; attIdx++) {
			var attObj = this.attributes[attIdx];
			var validChange = true;  // will be set to false if the new value is not of the right type 
			var isChanged = true; 
			var newValue = att[attObj.name];	
			var oldValue = attOld[attObj.name];
						
			if ( attObj && attObj.placed  && attObj.type ) {  // mind !== instead of != otherwise you cannot set via JavaScript from '' to 0 because it will think it's the same
			
				// validate
				switch ( attObj.type.toLowerCase() ) {
					case 'datetime':
						if ( newValue !== '' && newValue.constructor != Date ) {
							validChange = false;
							msg += '\nNew Value for attribute ' + attObj.name + ' is not a Date type';							
						} 
						if (newValue === oldValue) isChanged = false;   
						break;	
					case 'text':
						// check maximum length and type
						if ( newValue.constructor == Number) newValue = newValue.toString();
						if ( newValue.constructor != String ) {
							validChange = false;
							msg += '\nNew Value for attribute ' + attObj.name + ' is not a string';
						} 
						if (newValue === oldValue) isChanged = false;
						break;
					case 'richtext':
						isChanged = false;
						if(newValue !== oldValue){
							validChange = false;
							msg += '\nYou cannot set a new value for rich text attributes. Attribute: ' + attObj.name ;
						}
						break;
					
					case 'float':  
					case 'integer':					
					case 'area':
					case 'area1':
					case 'area2':
					case 'volume':
					case 'volume1':
					case 'volume2':
					case 'weight':
					case 'weight1':
					case 'distance': 
					case 'distance1':
				
					
						if (newValue === oldValue) isChanged = false;
						if ( newValue !== '' && newValue.constructor != Number || isNaN(newValue) )  {
							validChange = false;
							msg += '\nNew Value for attribute ' + attObj.name + ' is not a number';
						} else {					
							if (newValue !== '' && attObj.type.toLowerCase() == 'integer') {
								newValue = parseInt(newValue);  // cut off to integer								
							}
						}
						break;
				}  // end switch
				
				if (validChange && isChanged) {
					attObj.value = newValue;  
					this.onChangeAttValue(attObj);	
					
					// list values need to be propagated
					if(attObj.listObj){
						if(attObj.listObj.loadingType == 'OnTheFly'){
							this.checkAutoSelectLazyLoading(attObj,false);
						} else {
							this.changeListSelection(attObj,newValue,null,null,null,this.prefix);
						}
					}
				} 
			}
		}
		if (msg) alert('Problem executing button ' + buttonObj.label + msg);
	
		this.displayAttributeCategory();  // anything can have changed so full repaint.
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.clickACatButtonJavaScript' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	} 
	return errorObj;
}

attributeCategory.prototype.createDocAttForButton = function(docatt, bomdocatt) {

	"use strict";
	try {
	
		// try constructing a docatt Object - will only work in project creation
		// documentDivIdBOM1Attributes
		
		var docIdx = 0, docIdxBOM = 0;
		var aCatObj= this;
		$('[id^="documentDivId"]').each( function(index) {
			var curId = $(this).attr('id');  
			var isBOM = false; 
			if (curId.indexOf('BOM') != -1) isBOM = true;
			if (curId.indexOf('Attributes') != -1) { 
				
				var docACatObj = $(this)[0].ACatObj;  // get this document's attribute category object
				if (docACatObj) {
					var assoArray;
					
					if (isBOM) {
						assoArray = bomdocatt[docIdx] = {};
						docIdxBOM++;
					} else {
						assoArray = docatt[docIdx] = {};  // make an associative array
						docIdx++;
					}
					var docAttArr = docACatObj.attributes;
					for (var attIdx = 0;  attIdx < docAttArr.length; attIdx++) {
						var attObj = docAttArr[attIdx];
						if ( attObj && attObj.placed) {
							assoArray[attObj.name] = aCatObj.getCalculationValue(attObj);
						}
					}
					
				}
			}
		});
		
	
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.createDocAttForButton' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	} 
	return docatt;
}

attributeCategory.prototype.getProjectAttributeValuesForButton = function() {
	"use strict";
	var projatt = [];
	try {
		var url = 'GetAttributes.jsp?projectid=' + this.projectID;
		var attCatObj = this;
		ajax({
            url: url,
            type: "GET",
            data: "xml",
            debug: this.debug,
            synchronous: true,
            onSuccess: function(valuesXML) {
				try {
					if (!valuesXML) throw(attCatObj.langArr['ACatAttributeProblem']);
					var errorElem = WCXML.GetFirstFirstLevelChild(valuesXML,"error");
					if (errorElem) {
						var errorCode = WCXML.GetElementText(errorElem,"code");
						// if error code is 043 (no attributes), just do nothing
						if (errorCode != "043") {
							var errorMsg = WCXML.GetElementText(errorElem,"message");
							alert(attCatObj.langArr['ACatErrorReadingAttributes'] + ":\n" + errorMsg);							
						}
						return;
					}

					var attributeCategoryElem=WCXML.GetFirstFirstLevelChild(valuesXML,"attribute_category");
					if (!attributeCategoryElem) throw(attCatObj.langArr['ACatServerXMLProblem3']);

					var attributesElem=WCXML.GetFirstFirstLevelChild(attributeCategoryElem,"atts");
					if (attributesElem) {
						
						// read the attribute values
						var attArr = WCXML.GetFirstLevelChilds(attributesElem,"att");
						if (attArr && attArr.length>0) {
							for (var attribute = 0; attribute < attArr.length; attribute++) {
								var attElem = attArr[attribute];
								var attName = WCXML.GetElementText(attElem,"an");
								if (attName) {			
									var value =  WCXML.GetElementText(attElem,"av");									
									var type = WCXML.GetElementText(attElem,"at");
									var rsId = WCXML.GetElementText(attElem,"rs");	
									
									value = attCatObj.getCalculationValue({type: type, value:value});  // convert strings to float/integer for numbers. Does nothing with date
									
									// convert date to Date format
									if (type && type.toLowerCase() == 'datetime' && !rsId && value ) {		
										value = attCatObj.correctForDaylightSaving(new Date(parseInt(value)));
									}
									projatt[attName] = value;
							
								}
							}
						}
						
					}
				} catch (err) {
					alert(attCatObj.langArr['ACatProblemIn'] + ' attributeCategory.getProjectAttributeValuesForButton read Ajax return' + attCatObj.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
				}		 
			},
            onError: this.readError,
            timeout: 120000,
            objHdl: this
        });
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.getProjectAttributeValuesForButton' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	} 
	return projatt;
};
		
/**
* Converts the typical string format of attribute values (strings even for numeric types!) into numbers and leaves the rest untouched
* @param attObj - attribute object containing the value in attObj.value, must have attObj.type filled out correctly
* @return The value as will be used in JavaScript calculations
*/
attributeCategory.prototype.getCalculationValue = function (attObj) {
	"use strict";
	var returnValue = attObj.value;
	try {		
		
		if (attObj.type && attObj.value) { 
			switch ( attObj.type.toLowerCase() ) {
				case 'integer': 				
					returnValue = parseInt( returnValue );  // also converts "0"  to 0. 
					break;
				
				case 'float': case 'area': case 'area1': case 'area2':
				case 'volume': case 'volume1': case 'volume2': case 'weight': case 'weight1':
				case 'distance': case 'distance1':
					returnValue = parseFloat( returnValue ); 
					
					break;
				// default: do nothing
			}
		}
		// else it remains what it was, which might be 0 or ''. So for one of the number types '' is NOT converted to 0 (otherwise there is no way to see the difference anymore which has business value)
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.getCalculationValue' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}
	return returnValue;
}

/**
* Executes the code of a JavaScript Button
* @param {String} code - the code
* @param {Array} att - associative array of current attribute values (att['AttName'] = attValue)
* @param {Array} projatt - associative array of current project attribute values, only available for document attributes and task specifications, null when editing project attributes
* @param {String} buttonLabel - label of the button (used to give feedback)
* @param {String} clickType - event which triggered the button (same as in clickACatButton)
* @param {Array.Array} docatt - array of associative arrays of document attributes. docatt[n]['AttName']  is the value of document attribute with name AttName in document n in project creation's documents section
* @param {Array.Array} bomdocatt - same as docatt but for the documents in the BOM section of project creation
*/
attributeCategory.prototype.executeJavaScriptButton = function(code,att, projatt, buttonLabel, clickType, docatt, bomdocatt) {
	
	'use strict';
	var errorObj = null;
			
	try	 {
		
		// first make a couple of globals unavailable by masking them with a local variable 
		var codePrefix = 'var document = null, storedObj = null, jQuery=null, $ = null, window = null';
		codePrefix += ', keywordsearchform=null, detailslinksform=null, docactionform= null, actionform = null, actionform2 = null, inputform = null';
		codePrefix += ', attributeCategory=null, tinyMceMgr = null, ajax = null, setTimeout = null, setInterval = null;';

		var args = 'att, projatt, event, docatt, bomdocatt'; 
		
		var myFunc = new Function(args,codePrefix + code);   // no longer uses eval to allow returning an errorObj
		errorObj = myFunc.call(this,att,projatt, clickType, docatt, bomdocatt);  // clickType is given through as 'event'
		if (errorObj && clickType == 'click') alert(errorObj.errorMsg);  // only chat when people have clicked the button
	} catch (err) {
		alert('Problem executing button ' + buttonLabel + '\nCheck your script!\n' + err);
	}
	return errorObj;

}

/**
* Called when all values (also from project when in TEP) are loaded
*/
attributeCategory.prototype.onValuesLoaded = function() {
	try	 {
		
		this.executeButtonsOnLoad();  // execute all buttons which are marked to run 'on load'
		
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.onValuesLoaded' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);
	}

}

/**
* execute all buttons which are marked to run 'on load'
*/
attributeCategory.prototype.executeButtonsOnLoad = function() {
	'use strict';
	try	 {
		for (var row=0; row < this.nRows; row++) {
            for (var col=0;col < this.nCols; col++) {
                var cellObj = this.cells[row][col];
				if (cellObj) {		
					if (cellObj.buttons && cellObj.buttons.length > 0) {

						for (var button = 0; button < cellObj.buttons.length; button++) {
							var buttonObj = cellObj.buttons[button];
							if (buttonObj.execOnLoad) {
																
								var errorObj = null;
								errorObj = this.clickACatButton(buttonObj, null,'onLoad');	
								if (errorObj && errorObj.errorMsg ) alert(buttonObj.label + ' : ' + errorObj.errorMsg);
							}
						}
					}				
				}
			}
		}
		
		
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.onValuesLoaded' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);
	}

}

/**
* Called when you click on a button of the type CheckUniqueValue.
* @param {attObj} attObj: The attribute to be checked.
*/
attributeCategory.prototype.clickACatButtonCheckUniqueValue = function (attObj) {
	return this.validateAttributeUniqueValue(attObj, true);
};


/**
* to be called when an attribute value is changed programatorically
* Guarantees that all needed additional operations (f.e. for cascading dropdowns) are done
* Does NOT repaint. This is left to the caller
*/
attributeCategory.prototype.onChangeAttValue = function ( attObj ) {
	// To do: this function could be used more than it is today (f.e. after loading project attributes)
	try {
		var Obj = this.findCell(attObj);  
		if ( Obj ) {
			var cellObj = Obj.cellObj;
			var row = Obj.row;
			var col = Obj.col;
			var elem = document.getElementById(this.prefix + attObj.id);
			if ( elem ) {
				
				switch (elem.tagName.toLowerCase()) {   // To do : test with all types of controls including checkbox etc
					case 'input':
						elem.value = attObj.value;
						if (attObj.type && attObj.type.toLowerCase() == 'datetime') {
							this.setHiddenDate(attObj);		
							this.executeButtonDependencies(attObj);	
						} else {
							this.changeText(elem,row,col);
						}
						break;
					case 'select':
						elem.value = attObj.value;
						this.changeSelect(elem,row,col);
						break;
				}
			} else {
				// also execute button dependencies for read-only values
				this.executeButtonDependencies(attObj);	
			}
					
		}
	
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.onChangeAttValue' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}
};

/**
* execute buttons which are dependent on the attribute represented by attObj
*/
attributeCategory.prototype.executeButtonDependencies = function (attObj) {
	'use strict';
	try {
		
		// to avoid endless loop, we keep track of which attObj are being executed. Only checks synchronous loops. For asynchronous loops, there is a counter, see cMaxButtonClick.		
		if (!this.pendingExecuteButtonDependencies) this.pendingExecuteButtonDependencies = [];
		var found=false;
		for (var att=0; att< this.pendingExecuteButtonDependencies.length; att++) {
			if (this.pendingExecuteButtonDependencies[att] == attObj) {
				found=true; break;
			}
		}
		if (!found) {
			this.pendingExecuteButtonDependencies.push(attObj);
			if (attObj.buttonDependency && attObj.buttonDependency.length>0) {
				for (var dep = 0; dep < attObj.buttonDependency.length; dep++) {
					var buttonObj = attObj.buttonDependency[dep];
					//logger += '\nButton ' + buttonObj.label + ' clicked at ' + new Date() + '  count ' + buttonObj.clicked + ' attribute: ' + attObj.name ;
					if (buttonObj && buttonObj.viewName == this.currentView) {
						if (buttonObj.clicked > cMaxButtonClick) {   // if button too many times clicked, this is probably an endless loop. Stop clicking it automatically at least (they can still click by hand)
							var errMsg = '';
							if (buttonObj.style == 'hidden') errMsg += 'Hidden ';
							errMsg += 'Button ' + buttonObj.label + ' clicked more than '+ cMaxButtonClick + ' times. \n Please check for endless loops.';
							throw new Error(errMsg);
						} else {
							var errorObj = null;
							errorObj = this.clickACatButton(buttonObj, null,'onChange');	
							if (errorObj && errorObj.errorMsg ) alert(buttonObj.label + ' : ' + errorObj.errorMsg);				
						}							
					}
				}
			}
			// pull it off again
			for (var att=0; att< this.pendingExecuteButtonDependencies.length; att++) {
				if (this.pendingExecuteButtonDependencies[att] == attObj) {
					this.pendingExecuteButtonDependencies.splice(att,1); break;
				}
			}			
		} 
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.executeButtonDependencies' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
	}
}

attributeCategory.prototype.deleteCurrentTreeSelection = function (chkbxId, hpSectionId) {
	try {
		var chkBxElem = document.getElementById(chkbxId);
		if (chkBxElem) {
			chkBxElem.checked = false;
			this.clickTreeViewChkbox(chkBxElem, hpSectionId);
		}
	
	} catch (err) {
		alert(this.langArr['ACatProblemIn'] + ' attributeCategory.deleteCurrentTreeSelection' + this.langArr['ACatAtLine'] + err.lineNumber + ' : ' + err.message);	
		
	}
};
/**
*
* OBJECT CLASS FOR TINYMCE (RICH TEXT ATTRIBUTE FIELD) MANAGER
*
*/
function tinyMceMgr() {
	this.tmceEditors = new Array();
	this.initialized = false;
	this.rows = new Array();
	this.cols = new Array();
	this.readOnlyIds = new Array();
	this.readOnlyValues = new Array();
}

/**
* Initialize Rich text editor pane
*/
tinyMceMgr.prototype.initTinyMCEInstances = function(){
	if(!this.initialized){
		this.initialized = true;
		try{
			tinyMCE.init({

				selector : "#tinymceeditarea",
				plugins: "textcolor code pagebreak fullscreen print searchreplace charmap insertdatetime table hr link image",
				
				toolbar1 : "bold italic underline strikethrough | subscript superscript | forecolor backcolor | formatselect fontselect fontsizeselect | code pagebreak print",
				toolbar2 : "undo redo | bullist numlist | alignleft aligncenter alignright alignjustify | outdent indent blockquote | charmap inserttime | table | hr removeformat | link unlink image | searchreplace",
				toolbar_items_size: 'small',

				paste_retain_style_properties : "margin, padding, width, height, font-size, font-weight, font-family, color, text-align, ul, ol, li, text-decoration, border, background, float, display",
			
				height: 300,
				resize: false,
				menubar: false,
				statusbar: false
			});

		}catch(err){
			this.initialized = false;
		}
	}
};

tinyMceMgr.prototype.addRichTextFields = function(catId){
	if(this.tmceEditors[catId]){
		if(this.tmceEditors[catId].length>0){
			name = "";
			for(var i=0; i<this.tmceEditors[catId].length;i++ ){
				if(i != 0){
					name += ",";
				}
				name += "#" + this.tmceEditors[catId][i];
			}
			try{	
				tinyMCE.init({
					// General options

					selector : name,
					plugins: "textcolor code pagebreak fullscreen print searchreplace charmap insertdatetime table hr link image",
			
					setup : function(ed) {

			          	ed.on("change", function() {
			          		try{
			          			var elemid = ed.id;
			          			if (elemid && tinyMCE.get(elemid)){
			          				var elem = document.getElementById(elemid);
			          				if (elem) elem.onchange();
			          			}
			          			
			        		}catch(err){}
			         	});
			  		},

			  		resize: false,
			  		menubar: false,
			  		toolbar: false,
			  		statusbar: false,
					
					paste_retain_style_properties : "margin, padding, width, height, font-size, font-weight, font-family, color, text-align, ul, ol, li, text-decoration, border, background, float, display"
					
				});
			}catch(err){
			}
		}
	}
	if(!this.tmceEditorsActive) this.tmceEditorsActive = new Array();
	if(this.tmceEditors[catId]) this.tmceEditorsActive[catId] = this.tmceEditors[catId].slice(); 
	this.tmceEditors[catId] = [];
};

tinyMceMgr.prototype.hideRichTextFields = function(catId){
	if(this.tmceEditorsActive && this.tmceEditorsActive[catId]){
		for(var i=0; i<this.tmceEditorsActive[catId].length;i++ ){
			try {
				tinymce.remove("#" + this.tmceEditorsActive[catId][i]);
    		} catch(err) {
			}
		}
	}
};

///////////////////////////////////////////////
//Takes out leading and trailing spaces
function strtrim(string){

    try {
        if (string == null || string == '') return ('');
        // Find beginning and end of the final string
        var start = 0;
        while (start < string.length && string.charAt(start) == " ") {
			start++;
		}

        end = string.length - 1;
        while (end > 0 && string.charAt(end) == " ") {
			end--;
		}

        if (start <= end) {
            string = string.substring(start, end + 1);
        } else {
            string = '';
        }


    } catch (err) {
        alert("Problem in strtrim: " + err);
    }

    return string;
}

function alertSessionLost() {
	alert(sessionLostMsg);
}

////////////////////////////////////////////////////////////////////////////////////////////////
// DOCUMENT LINKS
////////////////////////////////////////////////////////////////////////////////////////////////
/**
* Document Link Details:
* cellObj.docLinkSources {Array of docLinkSourceObj}
* docLinkSourceObj {
*		sourceType {one of 'Browse','ProjectFolder','Search'}
*		label {String}
*		documentType {String} Document type to restrict to 
*		operation {one of 'Link', 'Copy', 'CopyLink', 'Move'}
*		targetFolder {String}
*		sourceFolder {String} only when sourceType == ProjectFolder
*		sourceProject {String} only when sourceType == ProjectFolder. Smartnamed with following smartnames : [CurrentProject], [Att:...]
* }
*/

/**
* Loads document information (name, id, version id and attributes) from all documents in a folder of the current project
* The project is found in ACatObj.projectID
* @param folder {String} name of the folder
* @param target {String}  name of the attribute category property which will be filled with the results
* @param attributes {Boolean} if true, attribute information is loaded for each of the documents
* Uses SDK Call GetDocumentList.jsp
*/
attributeCategory.prototype.loadFolderDocuments = function(projectName, folderName, parentObj ,attributes) {
    try {
        var readURL = 'GetDocumentList.jsp?';
		
		// project
		if (!projectName || projectName == '[CurrentProject]') {   // assumes the projectID is given... 
			readURL += 'projectID=' + this.projectID;
		} else {
			readURL += 'projectName=' + WCXML.urlencode(projectName);
		}
		
		// folder		
		if (folderName) {  // if no foldername is given, the param will not be supplied and all documents of the entire project are loaded
			if (folderName.toLowerCase() == '[root]') {
				readURL += '&folderName=' ;
			} else {
				readURL += '&folderName=' +WCXML.urlencode(folderName);
			}
		}
		var aCatObj=this;
		
		// go fetching it        
		ajax({
			url: readURL,				
			type: "GET",
			data: "xml",
			debug: this.debug,
			synchronous: false,
			
			
			//<document_list projectid="00002_0000005883">
				//<document>
					//<document_version_id>00002_0000046212</document_version_id>
					//<document_id>00002_0000045152</document_id>					
					//<document_name>Label_master.pdf</document_name>
					
			
			
			onSuccess: function(returnXML,objHdl2) {					
				var docListElem = WCXML.GetFirstFirstLevelChild(returnXML,"document_list");
				var docArr = WCXML.GetFirstLevelChilds(docListElem,"document");
				var foundDocuments = new Array();
				if (docArr && docArr.length>0) {
					for (var md=0; md<docArr.length;  md++) {
						var mDoc = {};
						mDoc.name =  WCXML.GetElementText(docArr[md],"document_name");
						mDoc.id =  WCXML.GetElementText(docArr[md],"document_id");
						mDoc.versionId =  WCXML.GetElementText(docArr[md],"document_version_id");					
						foundDocuments.push(mDoc);
						if (attributes) aCatObj.getAttributes(mDoc.id,mDoc);
					}
					if (!parentObj.docList) parentObj.docList = new Array();
					parentObj.docList = foundDocuments;
				}
				aCatObj.displayAttributeCategory();
			},
			
			//onSuccess: this.readReturnXML,
			onError: this.readError,				
			objHdl: this
		});
				


    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " loadFolderDocuments: " + err);
    }
};


attributeCategory.prototype.getAttributes = function(documentId,docObj) {
    try {
		var readURL = 'GetAttributes.jsp?projectID=';
		readURL += this.projectID;
		readURL += '&documentId=' + documentId;
		var aCatObj=this;
		
		// go fetching it        
		ajax({
			url: readURL,				
			type: "GET",
			data: "xml",
			debug: this.debug,
			synchronous: false,
			
			
			/* Ajax call returns an XML like following:
				<attribute_category>
					<id>00002_0000000002</id>
					<name>Person Information</name>
					<description>Person Information about Person</description>
					<atts>
						<att>
							<id>00002_0000000006</id> <! Attribute Id ?
							<an>Person Name</an> <! Attribute Name ?
							<av>John Walsh</av> <! Attribute Value ?
							<at>Text</at> <! Attribute Type ?
						</att>
					</atts>
				</attribute_category>
				*/
					
			
			
			onSuccess: function(returnXML,objHdl2) {					
				var aCatElem = WCXML.GetFirstFirstLevelChild(returnXML,"attribute_category");
				var attsElem = WCXML.GetFirstFirstLevelChild(aCatElem,"atts");		
				var attArr = WCXML.GetFirstLevelChilds(attsElem,"att");
				
				if (attArr && attArr.length>0) {
					docObj.attributes = new Array();
					
					for (var att=0; att<attArr.length;  att++) {
						var attElem = attArr[att];					
						var attObj = {};
						attObj.name =  WCXML.GetElementText(attElem,"an");
						attObj.type =  WCXML.GetElementText(attElem,"at");
						attObj.id =  WCXML.GetElementText(attElem,"id");
						attObj.value =  WCXML.GetElementText(attElem,"av");
										
						docObj.attributes.push(attObj);
					}					
				}				
			},
			
			//onSuccess: this.readReturnXML,
			onError: this.readError,				
			objHdl: this
		});
				


	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " getAttributes: " + err);
    }

}


attributeCategory.prototype.renderDocumentSelector = function(cellObj,row,col) {
    
    var htmlcode = '';
    try {
        var mDocs = this.masterDocuments;
		if (!mDocs) return;
        
        var changeMethod = this.functionPrefix + '.changeDocumentLink(this,' + row + ',' + col+')';
        var theName = this.prefix  + cellObj.attObj.id;


        htmlcode += '<select onchange="' + changeMethod + '" id="' + theName + '" name="' + theName + '" ';
        htmlcode += this.renderStyle(cellObj.styleObj,"Dropdown");
        
        htmlcode += '>';
        htmlcode += '<option value="">---' + this.langArr['ACatPleaseSelect'] + '---</option>';
		var found = false; // did we find the selected value in the list of options
        for (var option=0; option < mDocs.length; option++) {
            var value = mDocs[option].name;
			var valueDisplay = value;
			
            htmlcode += '<option value="'+ mDocs[option].versionId +'"';
            if (cellObj.attObj.value == mDocs[option].versionId) {htmlcode += ' selected '; found=true;};
            htmlcode += '>' + WCXML.xmlencode(valueDisplay) + '</option>';
        }
		if (cellObj.attObj.value && !found) {
			var value = cellObj.attObj.value;
			var valueDisplay = value;
			
			htmlcode += '<option  value="NOT FOUND" selected ';            
            htmlcode += '>' + WCXML.xmlencode(valueDisplay) + ' (' + this.langArr['ACatDeleted'] + ')</option>';
		}
        htmlcode += '</select>';
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderDocumentSelector:\n' + err);
    }
    return htmlcode;
};

/**
*
*/
attributeCategory.prototype.changeDocumentLink = function(elem,row,col) {
	var attObj = null;
	var idx;
	
    try { 
        if (!elem) throw("elem "+ this.langArr['ACatNotDefinedProblem']);
		var cellObj = this.cells[row][col];
        attObj = cellObj.attObj;
				
		var newValue;	
		//if (!attObj.docList) attObj.docList = [];
		
		
		attObj.docList = [];  // Always start from an empty array. TAKE CARE!!!!! This only works for a single doc source...
		
		checkArr = document.getElementsByName('MultiValueSelect');   // also when single select, then they are radio buttons
	
	
		if (checkArr && checkArr.length>0) {				
			for (idx=0; idx<checkArr.length; idx++) {
				if (checkArr[idx].checked) {
					var docLinkObj = cellObj.docList[idx];
					// add to attObj.docList if it's not there yet
					
					
					
					for (var idx2=0; idx2< attObj.docList.length; idx2++ ) {
						if (attObj.docList[idx2] == docLinkObj) break;
					}
					if (idx2 == attObj.docList.length) {
						docLinkObj = {}
						
						//To do : replace with a deep copy (for which you have a library)
						docLinkObj.id = cellObj.docList[idx].id;
						docLinkObj.versionId = cellObj.docList[idx].versionId;
						docLinkObj.name = cellObj.docList[idx].name;
						// To do: run over the allowed document properties 
						docLinkObj.linkAttributes = [];
						for (var lAtt=0; lAtt< cellObj.linkAttributes; lAtt++) {
							docLinkObj.linkAttributes[cellObj.linkAttributes[lAtt].name]='';
						}
						attObj.docList.push(docLinkObj);
					}
				}
			}
		}
		// construct new attribute value
		this.updateDocLinkAttValue(attObj);
		
		// repaint the element
		if (cellObj.renderOption == 'Table') {
			document.getElementById(this.prefix + 'td_'+row+'_'+col).innerHTML =  this.renderHiddenAttributeInfo(attObj) + this.renderDocumentLinkTable(cellObj,row,col,true);
		} else {
			document.getElementById(this.prefix + 'td_'+row+'_'+col).innerHTML =  this.renderHiddenAttributeInfo(attObj) + this.renderDocumentLinkSimple(cellObj,row,col,true);
		}
				
		this.setAttributesChanged();		
		this.executeButtonDependencies(attObj);

		
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.changeDocumentLink: " +err);
    }
    
	this.checkRepaintAttributeCategory(attObj);   // only repaint if a rule is applied to this cell
	
	
};

attributeCategory.prototype.updateDocLinkAttValue = function(attObj) {
	
	
    try { 
        
		// construct new attribute value
		var value= '';
		var valueXML = '';
		for (idx=0; idx< attObj.docList.length; idx++) {
			if (value) value += this.multiSep;
			var docLinkObj = attObj.docList[idx];
			value += docLinkObj.versionId;
			valueXML += '<dLink>';
			valueXML += '<vid>'+docLinkObj.versionId+'</vid>';
			valueXML += '<id>'+docLinkObj.id+'</id>';
			valueXML += '<name>'+docLinkObj.name+'</name>';
			valueXML += '</dLink>';
		}
		if (attObj.value != value) {
			attObj.changed = true;		
			attObj.value = value;
			attObj.valueXML = valueXML;
			this.setAttributesChanged();
			// this.checkTaskSyncWarning(attObj);	
		}
		
		this.setAttributesChanged();
		
    } catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.updateDocLinkAttValue: " +err);
    }

	
};



/**
* Called when a document link detail is edited
* @param elem:  The element which has been changed
*/
attributeCategory.prototype.changeDocLinkDetail = function(elem) {
	
	
    try { 
		var attObj = null;
		if (!elem) throw("elem "+ this.langArr['ACatNotDefinedProblem']);
		
		
		// figure out which element has changed
		// elem.id looks like 'LinkAttribute~(row)~(col)~(linkidx)~SomeId', f.e. 'LinkAttribute~9~0~1~00001_000000006'

		var elemArr = elem.id.split('~');
		var type = elemArr[0];
		var row = elemArr[1];
		var col = elemArr[2];
		var linkIdx = elemArr[3]
		var typeDetail = elemArr[4];
		var cellObj = this.cells[row][col];
		attObj = cellObj.attObj;
		
		var docLinkObj = attObj.docList[linkIdx];
		switch (type) {
			case 'DocumentProperty':
				if (!docLinkObj.documentProperties) docLinkObj.documentProperties = {};
				docLinkObj.documentProperties[typeDetail] = elem.value;
				break;
			case 'LinkAttribute':				
				if (!docLinkObj.linkAttributes) docLinkObj.linkAttributes = {};			
				docLinkObj.linkAttributes[typeDetail] = elem.value;
				break;
			case 'DocumentAttribute':
			if (!docLinkObj.docAttributes) docLinkObj.docAttributes = {};		
				docLinkObj.docAttributes[typeDetail].value = elem.value;
				break;
			default:
				throw new Error('type unknown:' + type);
		
		}
		this.updateDocLinkAttValue(attObj);
		this.setAttributesChanged();

	} catch (err) {
        alert(this.langArr['ACatProblemIn'] + " attributeCategory.changeDocLinkDetail: " +err);
    }
    	
	
};
/**
* Renders a document link more or less as a multi value attribute
*/
attributeCategory.prototype.renderDocumentLinkSimple = function(cellObj,row,col,editable) {
    var htmlcode = '';
    try {
		
		var attObj = cellObj.attObj;    
				
		if (this.currentView != 'Header') {
		
			var currentValueArr = attObj.value.split(this.multiSep);  // splits the docVersionId's
			
			htmlcode += '<table valign="top" padding="0">';
			
			htmlcode += '<tr padding="0">';
			htmlcode += '<td style="padding:0px; vertical-align:top">';
			
			htmlcode += '<div style="vertical-align:top; padding: 0 0 0 0;';
			if (editable || currentValueArr.length>4) htmlcode += 'border:1px solid grey;min-width:160px;background-color:white;';
			if (currentValueArr.length>4 && !this.printLayout) htmlcode+='height:80px; overflow-y:auto;';
			htmlcode+= '" ><table padding="0">';
			if (attObj.value != ""  && cellObj.docList) {
				var docList = cellObj.docList;
				for (var idx=0; idx< currentValueArr.length; idx++) {		
					// get its name
					// TODO Make more efficient
					var versionId = currentValueArr[idx];
					for (var statIdx=0; statIdx<docList.length; statIdx++) {
						if (versionId == docList[statIdx].versionId) break;
					}
					if (statIdx <docList.length) {
						clickMethod = this.functionPrefix + '.showDocumentLinkDetails('+statIdx+',\''+attObj.id+'_Details\','+row+','+col+'); ';
						htmlcode += '<tr><td class="darktext_greenlink_11" onclick="'+clickMethod+'" ' + this.renderStyle(cellObj.styleObj,"MultiValue") + '><nobr>'+WCXML.xmlencode(docList[statIdx].name)+'&nbsp;';					
						htmlcode += '</nobr></td></tr>';  // the &nbsp; is needed for the case of empty (attObj.value = " "). 
					}
				}
			} else if (editable) {
				htmlcode += '<tr><td class="darktext_greenlink_11"  ' + this.renderStyle(cellObj.styleObj,"MultiValue") + '><nobr>---'+this. langArr['ACatPleaseSelect']+'---</nobr></td></tr>';	
			}
			htmlcode += '</table></div></td>';
			
			
				
			if (editable) {
				var theName = this.prefix  + attObj.id;
				// drop icon
				var clickMethod = this.functionPrefix + '.showMultiValueDropdown(this,event,'+row+','+col+'); ';
				htmlcode += '<td style="padding:0 0 0 0; vertical-align:top"><img src="./images/ar_tp.gif"  onclick="'+clickMethod+'"></td>';
				
				
				// hidden element
				htmlcode += '<input type="hidden" id="' + theName +'" name="' + theName +'" value="'+WCXML.xmlencode(attObj.value)+'">';
			}
			// Details cell - to be filled dynamically when clicking on a text statement
			htmlcode += '<td id="' + attObj.id +'_Details"></td>';
			
			
			htmlcode += '</tr>';
			
			
			htmlcode += '</table>';
		} else {
			// not implemented. Puts nothing in the header.
		}
		
		
		
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderDocumentLinkSimple:\n' + err);
    }
    return htmlcode;
};

/**
* Renders a document link in a table 
*/
attributeCategory.prototype.renderDocumentLinkTable = function(cellObj,row,col,editable) {
    var htmlcode = '';
    try {
		
		var attObj = cellObj.attObj;    
		var docLinkTableOptions = cellObj.docLinkTableOptions;
		var changeMethod = this.functionPrefix + '.changeDocLinkDetail(this)';
		
				
		if (this.currentView != 'Header' && docLinkTableOptions) {
		
			var currentValueArr = attObj.value.split(this.multiSep);  // splits the docVersionId's
						
			
			htmlcode += '<table class="ordtable" width="100%" valign="top">';
						
			// put the labels 
			htmlcode += '<thead><tr class="darktext_greenlink_11">';
			for (var col2=0; col2 < docLinkTableOptions.length; col2++) {
				var dltObj = docLinkTableOptions[col2];
				htmlcode += '<th>' + dltObj.label + '</th>';						
			}
			htmlcode += '</tr></thead>';
			
			// put 1 row for each current document link
			if (attObj.value != ""  && cellObj.docList) {
				var docList = attObj.docList;
				for (var idx=0; idx< docList.length; idx++) {	

					htmlcode += '<tr class="darktext_greenlink_11">';				
					
					var docLinkObj = docList[idx];																
					
					// put the values
					for (var col2=0; col2 < docLinkTableOptions.length; col2++) {
						var dltObj = docLinkTableOptions[col2];
						var value = '';
						var colEditable = editable;
						var controlType = 'InputField';
						
						if (dltObj.visibility == 'ReadOnly' ) colEditable = false;
						
						switch (dltObj.type) {
							case 'DocumentProperty':
								switch (dltObj.typeDetail) {
									case 'Name':
										value = docLinkObj.name; break;											
									case 'Description':
										value = docLinkObj.description; break;
									default: 	
										
								}
								break;
							case 'LinkAttribute':	
								value = docLinkObj.linkAttributes[dltObj.typeDetail];						
							case 'DocumentAttribute':
							default:	
								
						
						}
						htmlcode += '<td>';
						if (colEditable) {
							var id = dltObj.type + '~' + row + '~' + col + '~' + idx + '~' + dltObj.typeDetail;
							if (controlType == 'InputField') {
								htmlcode += '<input type="text" value="'+WCXML.xmlencode(value)+'" onchange ="'+changeMethod+'" id="'+id+'" >';						
							} else {
								// assume select
								htmlcode += '<select onchange ="'+changeMethod+'" id="'+id+'" >';	
								if (options) {
									for (var opt=0; opt<options.length;opt++) {
										htmlcode += '<option value="'+WCXML.xmlencode(options[opt])+'">'  +  WCXML.xmlencode(options[opt]) +  '</option>'; 
									}
								}
								htmlcode += '</select>';
							}
						} else {
							htmlcode += WCXML.xmlencode(value) ;	
						}	
						htmlcode += '</td>';
						
					}
						
					
					htmlcode += '</tr>';
				}
			}
			
			
			// put a row of links and the hidden element with the value - not used in WCR 14, will be updated for document references epic in 14.1
			var docLinkSources = cellObj.docLinkSources;
			if (editable && docLinkSources) {
					
				var options = { hasDocumentTemplates: true, clickMethodArr: [] }
				options.clickMethodArr['Browse'] = this.functionPrefix + '.showMultiValueDropdown(this,event,'+row+','+col+'); ';
				options.clickMethodArr['ProjectFolder'] = this.functionPrefix + '.showMultiValueDropdown(this,event,'+row+','+col+'); ';
				options.clickMethodArr['DocumentTemplate'] = this.functionPrefix + '.showMultiValueDropdown(this,event,'+row+','+col+'); ';
				options.clickMethodArr['Search'] = this.functionPrefix + '.showSearch(this,event,'+row+','+col+'); ';
				htmlcode += docLinkSources.displayLinks(options);
			}			
			
			

			
			
			// hidden element with
			var theName = this.prefix  + attObj.id;
			htmlcode += '<tr style="display:none;"><td><input type="hidden" id="' + theName +'" name="' + theName +'" value="'+WCXML.xmlencode(attObj.valueXml)+'"></td></tr>';
			
			htmlcode += '</table>';
		} else {
			// not implemented. Puts nothing in the header.
		}
		
		
		
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderDocumentLinkTable:\n' + err);
    }
    return htmlcode;
};

attributeCategory.prototype.showDocumentLinkDetails = function(statIdx,detailsId,row,col) {
    var htmlcode = '';
    try {
		var cellObj = this.cells[row][col];
		var docList = cellObj.docList;
		var attributes = docList[statIdx].attributes;
		var title =  docList[statIdx].name;
		if (attributes) {
			
			htmlcode +=  '<table><tr>';
			htmlcode += '<tr class="darktext_greenlink_11" valign="top" ><td colspan="2"><b>' +title + ':</b></td></tr>';
			for (var att=0; att<attributes.length; att++) {
				var value = attributes[att].value;			
				if (value != '') {
					htmlcode += '<tr class="darktext_greenlink_11" valign="top" ><td>' + attributes[att].name + ':</td>';
					htmlcode += '<td>' + attributes[att].value + '</td></tr>';
				}
			}
			document.getElementById(detailsId).innerHTML = htmlcode;
		}
		if (document.getElementById("dropmenudiv")) document.getElementById("dropmenudiv").style.width = '800px';
		
		
		
		
    } catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.showDocumentLinkDetails:\n' + err);
    }
    
};

attributeCategory.prototype.loadDocLinkSources  = function(cellObj) {
	try {
		var dlsArr = cellObj.docLinkSources;
		for (var dls=0; dls<dlsArr.length; dls++) {
			var dlsObj = dlsArr[dls];
			switch (dlsObj.sourceType) {
				case 'ProjectFolder':
					// see whether you can load information right now
					if (dlsObj.sourceProject) {
						this.loadFolderDocuments(dlsObj.sourceProject, dlsObj.sourceFolder, cellObj, true);
					
					}
				
				default:
					// not implemented
			
			}
		}
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.loadDocLinkSources:\n' + err);
    }
}



/**
* Gets values for a dropdown from the WebCenter database
* @param {DOMObject} elem - the input field OR the dropdown arrow OR a link in the pop-up, whichever has triggered this function to be called
* @param {DOMEvent} event - the DOM event (click, keyup, ...)
* @param {int} row - row of the current cell 
* @param {int} col - column of the current cell 
* @param {Boolean} showAll - if true, values are shown irrespective of whether they are allowed or not (meaning no filtering). Don't confuse with showing really all values in the DB, which is not supported, but see maxResults
* @param {Boolean} forSearch - if true, this is called from a search params panel
* @param {Number} maxResults - maximum number of results shown. If omitted, 200 results are shown. maxResults cannot be larger than 2000 (to avoid overload). 
* @param {String} hpSectionId - Id of the home page section (only used for search)
*/
attributeCategory.prototype.getDBValues = function(elem,event,row,col,showAll,forSearch, maxResults, hpSectionId) {

	"use strict";
	try {
				
		var cellObj = this.cells[row][col];
		var attObj = cellObj.attObj;
		var columnObj = attObj.columnObj;
		var columnName = columnObj.columnName;
		var listObj = attObj.listObj;
		var listName = listObj.name;
		var patternUsed = true; // remembers whether you got here through typing or by clicking on the dropdown icon
		var pattern = elem.value;
		var url,url2;
		var multiValue = false;  // whether multivalue checkboxes will be shown
		
		if (!forSearch) forSearch = false;  // normalize forSearch
		
		if (forSearch) {
			multiValue =  columnObj.searchMultiple;  // both single as multivalue attributes can be searched for with OR clauses
		} else {
			multiValue =  attObj.allowMultipleValues;
		}
		
		// replace elem with the input field if the user clicked on the dropdown icon or on 'show all' or in the filter of the multi-value pop-up
		// If you don't do that, the pop-up appears under the image instead of under the input field, which is a worse user experience. In case of the show all anchor it even goes top left since the thing is gone by then
		if (elem.tagName.toLowerCase() == 'img' || elem.tagName.toLowerCase() == 'a' || elem.id == 'MultiValueFilterInput') {
			var elem2;
			if (attObj.allowMultipleValues && !forSearch) {
				elem2 = document.getElementById('mvdiv_' + this.prefix + attObj.id);
			} else {
				elem2 = document.getElementById(this.prefix + attObj.id);
			}
			
			if (elem.id != 'MultiValueFilterInput') {
				patternUsed = false;
			}
			if (elem2) elem = elem2;  // so if we fail finding the right input field, continue using the image, resulting in the pop-up to come under the image, which still works
		}
		
		

		
		var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
		menuItems.push('<img src="./images/ajax_loader.gif">');			
		var menuWidth = '350px', menuHeight = '300px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=false;
		
					
		dropdownmenu(elem,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;	
		if (forSearch) document.getElementById('dropmenudiv').style.zIndex = 1200;   // to put it above a jQuery dialog which sits on 1002.

		// construct the url
		url = 'GetRowsFromList.jsp?listType=WCR_DB_List&outputType=JSON&indexed=1';
		url += '&list=' + WCXML.urlencode(listName);
		url += '&output=' + WCXML.urlencode(columnName);
		if (!maxResults) maxResults = 200;
		if (maxResults > cMaxResultsDBValues)  maxResults = cMaxResultsDBValues;
		url += '&maxrows=' + maxResults;  
		url += '&maxcolrows=200';

		
		if (pattern && patternUsed) url += '&input='+ WCXML.urlencode(columnName) + ':0:' + WCXML.urlencode(pattern);  // TO DO: encoding of columnName should also escape the : ?
		
		url2 = url; // the part needed to load all 
		
		// check for additional input values by checking other columns of the same list
		if (listObj && !showAll) {
			for (var column=0; column < listObj.columns.length; column++) {
				var columnObj2 = listObj.columns[column];
				if (columnObj2 && columnObj2.attObj && (columnObj2.attObj != attObj) && columnObj2.attObj.value && !columnObj2.autoSelected) {
					url += '&input='+ WCXML.urlencode(columnObj2.columnName) + ':1:' + WCXML.urlencode(columnObj2.attObj.value);
				}
			}			
		
		}
		
		var aCatObj = this;  // closure
		
		// ajax call
		ajax({
			url: url,				
			type: "GET",
			data: "text",
			debug: this.debug,
			synchronous: false,
			onSuccess: function(jsonContent) {  
				aCatObj.displayDBValues(jsonContent, event, elem, row, col, multiValue, attObj, forSearch, dropmenuobj,url2,showAll,pattern, hpSectionId);				
			},		
			onError: function(status, objHdl, errorMsg){
				aCatObj.handleAjaxError(status,objHdl, errorMsg); 
				dropmenuobj.style.visibility = 'hidden';
			}
		});		
			

		
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.getDBValues:\n' + err);
    }	
}

/**
* Fills dropmenudiv with values provided via jsonContent
* @param {String} jsonContent - jsonContent returned by the Ajax call
* @param {DOMEvent} event - event which triggers the dropdown to appear (click, keyup)
* @param {DOMObj} elem - DOM object of the element under which the dropdown will appear
* @param {int} row - row of the cell
* @param {int} col - col of the cell
* @param {Boolean} multiValue - indicates whether the dialog allows multivalue selection
* @param {Object} attObj - attribute object of the current attribute
* @param {Boolean} forSearch - true if this is called from the search panels
* @param {DOMObj} dropmenuobj - DOM object to be filled with the created information
* @param {String} urlOtherValues - url to get other values (values not fitting to current selection). If not provided, other values will not be loaded
* @param {Boolean} showAll - if true, all values are shown irrespective of whether they are allowed or not
* @param {String} pattern - text typed in the filter. null is allowed and is seen as the same as ''
* @param {String} hpSectionId - Id of the home page section (only used for search)
*/
attributeCategory.prototype.displayDBValues = function(jsonContent, event, elem, row, col, multiValue, attObj, forSearch, dropmenuobj,urlOtherValues,showAll, pattern, hpSectionId) {
    "use strict";	

	
	try {

		var jsonObj;					
		var listObj = attObj.listObj;
		var listName = listObj.name;
		var columnObj = attObj.columnObj;
		var isMobileMultiValue = WCHTML.isMobile() && multiValue;	// when mobile and multi-value, we need to add an OK button because there is no other intuitive way to close the pop-up
		if (!pattern) pattern = '';
		
		try {						
			jsonObj = jsonParse(jsonContent);
		} catch(err) {   // if jsonParse fails, fall back on eval (which is slower)
			jsonObj = eval('(' + jsonContent + ')');


		}
								
		if (!jsonObj.success || jsonObj.success == "0") {
			alert('Cannot get values ' + jsonObj.error.code + ' : '+  jsonObj.error.message);
		} else {
		
			var column;
			var rows_returned = parseInt(jsonObj.rows_returned,10);
			var rows_total = parseInt(jsonObj.rows_total,10);
			var curVal = attObj.value;
			var curValArr;
			if (multiValue) {
				curValArr = curVal.split(this.multiSep);  // the set of actual values				
			}
			
			var htmlcode = '';
			var clickMethod;
			var selected;
			var hashArr = [];
			
			// read the column names first
			var rowArr = jsonObj.columns[0].values;
			if (isMobileMultiValue) htmlcode += '<div id="dbValuesDivAround" style = "height:250px; overflow-y:auto;">';
			
			htmlcode += '<table id="dbValues" style="margin-right:20px">';

			
			// top links for advanced functions
			clickMethod = this.functionPrefix + '.showAdvancedDBFunctions(event); return false;';
			htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctionsHide" ><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBShowAdvancedFunctions"  onclick = "'+clickMethod+'">' + this.langArr['ACatShowAdvancedDBFunctions']+'</a></td></tr>';
			
			clickMethod = this.functionPrefix + '.clearListValues(\''+WCXML.xmlencode(listName)+'\',\''+ listObj.type +'\','+ listObj.counter +',true); return false';
			htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBResetValues" onclick = "'+clickMethod+'">' + this.langArr['ACatResetListValues']+'</a></td></tr>';

			
			clickMethod = this.functionPrefix + '.highlightRelatedValues(\''+WCXML.xmlencode(listName)+'\',\''+ listObj.type +'\','+ listObj.counter +',1); return false';
			htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBHighlightRelatedValues" onclick = "'+clickMethod+'">' + this.langArr['ACatHighlightRelatedValues']+'</a>' ;

			
			clickMethod = this.functionPrefix + '.highlightRelatedValues(\''+WCXML.xmlencode(listName)+'\',\''+ listObj.type +'\','+ listObj.counter +',0); return false';
			htmlcode += '&nbsp;-&nbsp;<a href="#" id="getDBResetHighlight" onclick = "'+clickMethod+'">' + this.langArr['ACatResetHighlight']+'</a></td></tr>';

			
			clickMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+ col + ',true, '+forSearch+'); return false';
			htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBShowAll"  onclick = "'+clickMethod+'">' + this.langArr['ACatShowUnfiltered']+'</a></td></tr>';

			
			if (forSearch) {
				htmlcode += '<tr class="darktext_greenlink_11 getDBAdvancedFunctions" style="display:none;"><td style="cursor:default;" class="nobr" colspan="2">';
				if (columnObj.searchMultiple) {
					clickMethod = this.functionPrefix + '.allowSearchMultiple(this,event,'+row+','+ col + ',true,\''+attObj.id+'\',false, \''+hpSectionId+'\' ); return false';
					htmlcode += '<a href="#" id="getDBSearchMultiple"  onclick = "'+clickMethod+'">' + this.langArr['ACatSearchSingle']+'</a>';
				} else {
					clickMethod = this.functionPrefix + '.allowSearchMultiple(this,event,'+row+','+ col + ',true,\''+attObj.id+'\',true, \''+hpSectionId+'\'); return false';
					htmlcode += '<a href="#" id="getDBSearchMultiple"  onclick = "'+clickMethod+'">' + this.langArr['ACatSearchMultiple']+'</a>';
				}
				htmlcode += '</td></tr>';
			}
			
			// select all, only for multivalue
			if (multiValue) {							
				clickMethod = this.functionPrefix + '.selectAllMultiValue(); return false; ';
				htmlcode += '<tr class="darktext_greenlink_11"><td style="cursor:default;" class="nobr" colspan="2"><a href="#" id="getDBSelectAll"  onclick = "'+clickMethod+'">' + this.langArr['ACatSelectAll']+'</a>';
				clickMethod = this.functionPrefix + '.deselectAllMultiValue(); return false; ';
				htmlcode += '&nbsp;&nbsp;&nbsp;<a href="#" onclick="'+clickMethod+'">'+this.langArr['ACatDeselectAll']+'</a>';
				htmlcode += '</td></tr>';

			}
			

			if (multiValue) {							
				var keyUpMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+ col + ',true, '+forSearch+');';
				htmlcode += '<tr class="darktext_greenlink_11"><td colspan="2">';
				//htmlcode += '<input type="text" id="MultiValueFilterInput" autocomplete = "off" onkeyup="'+keyUpMethod+'" value="'+ pattern +'"/>';
				htmlcode += '<input type="text" id="MultiValueFilterInput" autocomplete = "off" onkeyup="'+keyUpMethod+'" />';  // value is set later to have the cursor at the end
				htmlcode += '</td></tr>';
			}

			
			dropmenuobj.style.visibility = 'hidden';

			
			// first put the valid set
			if (rowArr && rowArr.length >0) {	

				
				for (var row2=0; row2<rowArr.length; row2++) {		
					var value =  rowArr[row2];

					
					if (multiValue) {									
						// figure out whether this one is selected
						selected = false;
						for (var idx=0; idx < curValArr.length; idx++) {
							if (curValArr[idx] == value && value !='') {
								selected=true;								
							}

						}
						htmlcode += '<tr class="darktext_greenlink_11">';
						htmlcode += '<td class="checkboxWidth"><input  type="checkbox" id="mvchk_' + row2 + '" value="'+WCXML.xmlencode(value)+ '" name="MultiValueSelect" ' ;
						if (selected) htmlcode += 'checked';
						htmlcode += '/></td>';
						htmlcode += '<td style="cursor:default;" class="nobr">' + WCXML.xmlencode(value)+'</td>';
						htmlcode += '</tr>';
					} else {
						clickMethod = this.functionPrefix + '.selectDBValue(\''+attObj.id+'\', \''+WCXML.xmlencode(WCHTML.encodeForJScript(value))+'\','+showAll+',\''+elem.id+'\'); return false';
						htmlcode += '<tr class="darktext_greenlink_11"><td onclick="'+clickMethod+'" style="cursor:default;" class="nobr">' + WCXML.xmlencode(value)+'</td></tr>';

					}
					hashArr[value]=true;

				}
			} else {
				htmlcode += '<tr class="darktext_greenlink_11"><td>'+ this.langArr['ACatNothingValid']+'</td></tr>';  
			}
			
		
			// the remaining selected one(s) must always be displayed too
			if (multiValue) {	
				for (var idx=0; idx < curValArr.length; idx++) {
										
					if (!hashArr[curValArr[idx]] && (curValArr[idx] != '' || curValArr.length > 1)) {  // show not yet shown values but not the empty value unless the empty value is clearly wanted - seen from the fact that there are multiple things selected	
						value = WCXML.xmlencode(curValArr[idx]);
						htmlcode += '<tr class="darktext_greenlink_11">';
						htmlcode += '<td class="checkboxWidth"><input  type="checkbox" id="mvchk_' + row2 + '" value="'+WCXML.xmlencode(value)+ '" name="MultiValueSelect" checked /></td>' ;					
						htmlcode += '<td style="cursor:default;" class="nobr">' + value+'</td>';
						htmlcode += '</tr>';
						row2++;
						hashArr[curValArr[idx]] = true;
					}

				}
			}
			
			// indicate that something is missing
			if (rows_returned < rows_total) {
				htmlcode += '<tr class="darktext_greenlink_11 orange_medium_bold"><td class="nobr" colspan="2">' + this.langArr['ACatOnly'] + rows_returned  + this.langArr['ACatValuesShownOf'] + rows_total +'</td></tr>';
				htmlcode += '<tr class="darktext_greenlink_11 orange_medium_bold"><td class="nobr" colspan="2">' + this.langArr['ACatTypePartInInputField'] +'</td></tr>';
				var rows_to_get = rows_total;
				
				var labelShowAll = this.langArr['ACatShowAll'];
				if ( rows_total > cMaxResultsDBValues ) {
					labelShowAll += ' (max. ' + cMaxResultsDBValues + ')';  // no translation needed
					rows_to_get = cMaxResultsDBValues;
				}
				clickMethod = this.functionPrefix + '.getDBValues(this,event,'+row+','+ col + ',true, '+forSearch+','+ rows_to_get + '); return false';
				htmlcode += '<tr class="darktext_greenlink_11 orange_medium_bold"><td class="nobr" colspan="2"><a href="#" id="getDBShowAllBottom"  onclick = "'+clickMethod+'">' +  labelShowAll +'</a></td></tr>';
				
			}
			
			var aCatObj = this;  // closure
			// now put the rest, but only when there no or only 1 valid one
			if (rowArr.length <= 1 && urlOtherValues) {
				// try immediately load the rest?
				ajax({url: urlOtherValues, type: "GET", data: "text",	debug: this.debug,synchronous: true,

					onSuccess: function(jsonContent2) {
						var jsonObj2;					

						try {						
							jsonObj2 = jsonParse(jsonContent2);
						} catch(err) {   // if jsonParse fails, fall back on eval (which is slower)
							jsonObj2 = eval('(' + jsonContent2 + ')');
						}
							
						if (jsonObj2.success) {
							var rowArr2 = jsonObj2.columns[0].values;
							if (rowArr2 && rowArr2.length >0) {	
								htmlcode += '<tr class="darktext_greenlink_11"><td colspan="2"><hr></td></tr>';

								htmlcode += '<tr class="darktext_greenlink_11"><td colspan="2">'+ aCatObj.langArr['ACatValuesRequiringChanges'] + '</td></tr>';
								for (var row3=0; row3<rowArr2.length; row3++) {		
									var value =  rowArr2[row3];
									if (hashArr[value]) continue;  // don't display values which are already displayed in the first set
									
									if (multiValue) {									
										// figure out whether this one is selected
										selected = false;
										for (var idx=0; idx < curValArr.length; idx++) {
											if (curValArr[idx] == value && value !='') selected=true;
										}

										htmlcode += '<tr class="darktext_greenlink_11">';
										htmlcode += '<td><input type="checkbox" id="mvchk_' + (rowArr.length + row2) + '" value="'+WCXML.xmlencode(value)+ '" name="MultiValueSelect" ' ;  // row2 is correct, not row3!
										if (selected) htmlcode += 'checked';
										htmlcode += '/></td>';
										htmlcode += '<td style="cursor:default;" class="nobr">' + WCXML.xmlencode(value)+'</td>';
										htmlcode += '</tr>';
									} else {											
										clickMethod = aCatObj.functionPrefix + '.selectDBValue(\''+attObj.id+'\', \''+WCXML.xmlencode(WCHTML.encodeForJScript(value))+'\',true,\''+elem.id+'\'); return false';
										htmlcode += '<tr class="darktext_greenlink_11"><td onclick="'+clickMethod+'" style="cursor:default;" class="nobr"><i>' + WCXML.xmlencode(value)+'</i></td></tr>';
									}
									row2++;
								}

							}
							
						}
					}
				});  // end ajax
			}
			
			htmlcode += '</table>';
			if (isMobileMultiValue) { // add an OK button
				htmlcode += '</div>';
				htmlcode += '<div style="width: 95%; text-align: center; padding: 6px;"><input type="button" class="buttonsecondary" value="OK" onclick="hidemenu()"></div>';
			}

			var menuItems = [];  // dropdownmenu expects and array of html code snippets, so we give it an array of 1 element
			menuItems.push(htmlcode);	
			
			var menuWidth = '350px', menuHeight = '300px',scrollHor = false, nohide=false, bgcolor = "#ffffff",nohideonclick=false;
			if (multiValue) nohideonclick = true;
		
			dropdownmenu(elem,event,menuItems,menuWidth,menuHeight,scrollHor,nohide,bgcolor,nohideonclick) ;
			dropmenuobj.style.visibility = 'visible';   // gets hidden in dropdownmenu because it was visible before 
			
			// set focus on the input field if it exists
			var filterElem = document.getElementById('MultiValueFilterInput');
			if (multiValue && filterElem ) {
				filterElem.focus();
				// now set (again) the value
				filterElem.value = pattern;
			}

			// decide what to do when the content of the dropdownmenu changes (only interesting for checkbox selection in multivalue)
			
			if (attObj.allowMultipleValues && !forSearch) {
				dropmenuobj.onchange = function(event) {aCatObj.changeSelectMultiple(row,col); }
			} else if (forSearch && columnObj.searchMultiple) {
				dropmenuobj.onchange = function(event) {aCatObj.searchSelectMultiple(elem,attObj.id); }
			} else {
				dropmenuobj.onchange = function(event) {}
			}
			
		}					
												

	} catch (err) {
		alert(this.langArr['ACatProblemIn']+ " attributeCategory.displayDBValues" + this.langArr['ACatAtLine'] + err.lineNumber + '  : ' + err.message);
		dropmenuobj.style.visibility = 'hidden';
	}
	
}

/**
* show the advanced functions for the cascading dropdown pop-up and hide the Advanced... text
* @param {Event} event - the click event
*/
attributeCategory.prototype.showAdvancedDBFunctions = function(event) {
    "use strict";
	try {
		$(".getDBAdvancedFunctions").show(); 
		$(".getDBAdvancedFunctionsHide").hide();
		
		 // avoid the click to close the hover
		if (event.stopPropagation) {
			event.stopPropagation();   // W3C model
		} else {
			event.cancelBubble = true; // IE model
		}

	
	} catch (err) {
		alert(this.langArr['ACatProblemIn']+ " attributeCategory.showAdvancedDBFunctions" + this.langArr['ACatAtLine'] + err.lineNumber + '  : ' + err.message);
	}
}

/**
* Called when any action is done on the multiple select in search dropdowns
* @param {DOMObject} searchInputField - input field for this search criterion
* @param {String} attId - attribute id for the attribute linked to this search criterion
* @description uses global dropmenuobj: the div used for the pop-up
*/
attributeCategory.prototype.searchSelectMultiple = function(searchInputField,attId) {

    "use strict";
	try {
		
		if (dropmenuobj.style.visibility == 'hidden') {  // only do something when the menu gets Hidden
			// find what is selected and construct a search string from it
			var checkArr = document.getElementsByName('MultiValueSelect');
			var searchString = '';
			var attValue = '';
			
			if (checkArr && checkArr.length>0) {				
				for (var idx=0; idx < checkArr.length; idx++) {
					if (checkArr[idx].checked) {
						if (searchString) searchString += ' || ';
						searchString += '\"' + WCXML.QuoteEncode(checkArr[idx].value, true) + '\"';  // encode " but not '
						if (attValue) attValue += this.multiSep;
						attValue += checkArr[idx].value;
					}
				}
				var attObj = this.findAttribute(attId);
				attObj.value = attValue;
				this.onChangeAttValue(attObj);
				
				searchInputField.value = searchString;				
				
				
				if (searchInputField.onblur && typeof searchInputField.onblur == 'function') searchInputField.onblur();
			}
			
		}

		
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.searchSelectMultiple:\n' + err);
    }	
}

/**
* resets all the values of a list
* @param {Boolean} closeDropMenuDiv - if true an attempt will be made to close dropmenudiv and avoid its closure onchange to be executed
*/
attributeCategory.prototype.clearListValues = function(listName, listType, listCounter, closeDropMenuDiv) {
	try {
		var listObj = this.findList(listName, listType, listCounter);
		if (listObj) this.deselectListAttributeValues(listObj);
		dropmenuobj.onchange = '';
		dropmenuobj.style.visibility = 'hidden';
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.clearListValues:\n' + err);
    }	
}

/**
* Called typically from the search panels to allow multiple select in a cascading dropdown. Only supported for back-end lists
* first set of parameters (elem --> showAll): see definiion in getDBvalues
* @param {String} attId - attribute Id of the attribute for which you allow multiple search values.
* @param {Boolean} searchMultiple - true if you want to enable mutliple search, false if you want to disable it
*/
attributeCategory.prototype.allowSearchMultiple = function(elem,event,row,col,showAll, attId, searchMultiple, hpSectionId) {
	"use strict";
	try {
		var attObj = this.findAttribute(attId);
		if (attObj && attObj.columnObj) attObj.columnObj.searchMultiple = searchMultiple;
		dropmenuobj.style.visibility = 'hidden';
		this.getDBValues(elem,event,row,col,showAll,true);
		
		if (searchMultiple === true) {
			$('#' + hpSectionId + 'db_attr_' + attId + '_temp_attr').val('2');  
		} else {
			$('#' + hpSectionId + 'db_attr_' + attId + '_temp_attr').val('0');   
		}
		
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.allowSearchMultiple:\n' + err);
    }	
}


/**
* Select a value from a database list cascading dropdown
* @param {String} attid - id of the current attribute (for which a value was chosen)
* @param {String} value - the chosen value
* @param {Boolean} showAll - whether the current dropdown is in showAll mode
*/
attributeCategory.prototype.selectDBValue = function(attid,value,showAll,elemId) {

	try {
		var attObj = this.findAttribute(attid);
		if (attObj) {
			attObj.value = value;
			this.onChangeAttValue(attObj);
			attObj.columnObj.autoSelected = false;
			// look whether autoselect of other items is needed
			this.checkAutoSelectLazyLoading(attObj,showAll);
		}
		
		var elem = document.getElementById(elemId);
		if (elem && elem.onblur && typeof elem.onblur == 'function') elem.onblur();
	
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.selectDBValue:\n' + err);
    }	
}


/**
* Called when an attribute's value has changed when the attribute is part of a cascading dropdown of list type 'database'
* Checks whether the other attributes of the same cascading dropdown only have 1 possible valid value, in that case, selects that value automatically
* @param {Obj} attObj - attribute object of the cell in which a value was just selected
* @param {Boolean} showAll - indicates whether the current dropdown is in show all mode (allowing to select invalid values, this way forcing more repaint than otherwise)
*/
attributeCategory.prototype.checkAutoSelectLazyLoading = function(attObj,showAll) {

	try {
		var listObj = attObj.listObj;
		
		// construct a URL with the known input values and ask for all distinct column values.  maxrows/maxcolrows put on 2, as soon as you have 2 lines , you know what you need to know.
		var url = 'GetRowsFromList.jsp?listType = WCR_DB_List&outputType=JSON&indexed=1&maxrows=2&maxcolrows=2&list=' + WCXML.urlencode(listObj.name);  
			
		
		for (var column=0;column <listObj.columns.length; column++) {
			var columnObj = listObj.columns[column];
			var attObj2 = columnObj.attObj;
			var colName = columnObj.columnName;
			if (attObj2) {
				if ( attObj2.value && !columnObj.autoSelected) {			// autoselected values are never used to exclude valid values for other columns (since the user hasn't made this choice explicitly) 		
					url += '&input=' + WCXML.urlencode(colName) + ':1:' + WCXML.urlencode(attObj2.value);
				} else {
					url += '&output=' + WCXML.urlencode(colName);	// whatever does not have a value explicitly set is ready to be replaced automatically. We want to know how many different values there are for this
					
				}
			}
		}
		var aCatObj = this;
				

		// ajax call to get the valid values of the other columns
		ajax({
			url: url,				
			type: "GET",
			data: "text",
			debug: this.debug,
			synchronous: false,
			
			onSuccess: function(jsonContent) {
				try {	
					var jsonObj;
					var somethingChanged = false;
					try {						
						jsonObj = jsonParse(jsonContent);
					} catch(err) {   // if jsonParse fails, fall back on eval (which is slower)
						jsonObj = eval('(' + jsonContent + ')');
					}
											
					if (!jsonObj.success || jsonObj.success == "0") {
						alert('Cannot check on autocomplete: checkAutoSelectLazyLoading code ' + jsonObj.error.code + ' : '+  jsonObj.error.message);
					} else {
						var rowsReturned = parseInt(jsonObj.rows_returned);
						if (rowsReturned > 0) {
							for (var column=0;column <listObj.columns.length; column++) {	
								
								var columnObj2 = listObj.columns[column];
								var colName = columnObj2.columnName;
								var attObj2 = columnObj2.attObj;
								if (attObj2 && (!attObj2.value || columnObj2.autoSelected)) {
									// this should appear in the output columns
									for (outcol=0; outcol< jsonObj.columns.length; outcol++) {
										if (jsonObj.columns[outcol].name == colName) {
											if (jsonObj.columns[outcol].values && jsonObj.columns[outcol].values.length == 1) {
												// then we can autoselect												
												var newValue = WCXML.QuoteDecode(jsonObj.columns[outcol].values[0]);  // TODO: Test this with values having quotes
												if (newValue != attObj2.value) {
													attObj2.value = newValue;
													columnObj2.autoSelected = true;
													aCatObj.onChangeAttValue(attObj2);												
													somethingChanged = true;
												}
											}
										}
									}
								}
							}
						} else {
							// an invalid selection was made/typed: no row in the list has this set of input values
							if (showAll) {
								// this was done on purpose --> de-assign the rest and try again auto-selecting
								var column2;
								for (var column2=0; column2<listObj.columns.length; column2++) {
									var columnObj2 = listObj.columns[column2];
									var attObj2 = columnObj2.attObj;
									if (attObj2 && attObj2 != attObj && attObj2.value != '' && attObj2.value != null) {  
										attObj2.value = '';  
										//aCatObj.onChangeAttValue(attObj2);  // CR: why not???  Probably becaus a full repaint is done anyway
										columnObj2.autoSelected = false;
										somethingChanged = true;
									}
								}
								aCatObj.checkAutoSelectLazyLoading(attObj);
								
							} else {
								alert('invalid selection made');	// ToDo: this is probably not a good UX. When can this happen? 
							}
						}
					}	
					if (somethingChanged) aCatObj.displayAttributeCategory();  // TODO: do a less expensive call - if you do so, you probably need again onChangeAttValue above

				} catch (err) {
					alert("Error processing return of ajax call to GetRowsFromList.jsp: " + err ); 
				}
			},
			
			
			onError: function(status, objHdl, errorMsg){
				this.handleAjaxError(status,objHdl, errorMsg); 
			}
		});		

	
	} catch(err) {
        alert(this.langArr['ACatProblemIn']+ ' attributeCategory.selectDBValue:\n' + err);
    }	
}





attributeCategory.prototype.handleAjaxError = function(status,objHdl, description) {

	var loginForm = document.getElementById('sessionexpiredform');
	
	if ( description == 'SESSION_EXP' && loginForm!= null ) {
		// redirect to the login.jsp for login in again. 
 		loginForm.submit();  		
	} else {
		alert("Ajax call unsuccessful. Status: " + status + "\n" + description);
	}
};

	  
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Experiment day EXTENDED Forms
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**
* Creates htmlcode for a property field. Like each renderer, it returns a list of <td>'s adding up to the needed column span (as defined in cellObj.colSpan)
* @param cellObj cell object containing the attribute
* @param row col : row and col where the field is
* @param visibility: any of Editable, Required, Hidden (typically not),  HiddenKeepValues, ReadOnly
* @param forPrint: if true, will be optimized for printing
*/
attributeCategory.prototype.renderProperty = function(cellObj,row,col, visibility, forPrint) {
	var attObj = null;
    try {
        var htmlcode = '';		
        var changeMethod;
        var editable = false;
        if (this.currentUser && this.currentUser.canEditAttributes == true) editable=true;
        if (this.showFakedReadOnly) editable = false;  // header preview in admin page
		if (visibility == "ReadOnly") editable = false;
		var hasButton = false;
		if (cellObj.buttons && cellObj.buttons.length>0) hasButton = true;

        attObj = cellObj.attObj;
        if (!attObj) throw(this.langArr['ACatRenderAttrProblem'] + " attObj");
        var isSection = false;
		if (attObj.name == 'CheckList' || attObj.name == 'Documents'  || attObj.name == 'BillOfMaterials' || attObj.name =='Members' ) isSection = true;
		
		// determine colspan
        var colSpan = 1;
        if (cellObj.colSpan >1) colSpan =  cellObj.colSpan * 3 - 2 ;
		if (cellObj.labelPosition == 'Top' || isSection) colSpan += 2;
		
		if (cellObj.labelPosition == 'Top') htmlcode += '<td colspan="'+colSpan+'" ' + this.renderStyle(cellObj.styleObj,"Value") + '><table margin="0" width="100%"><tr>';
		var sourceField = document.getElementById(attObj.sourceTd);
		
		
		if (attObj.name != 'buttonRow'  && !isSection) {
		
			// RENDER THE LABEL 		
			//////////////////////////////////////////////////////////////////////////////////////		
			 htmlcode += this.renderAttributeLabel(cellObj, row, col, visibility,editable,false);   // always renders a <td>...</td>
			
			if (cellObj.labelPosition == 'Top') htmlcode += '</tr><tr>';
		}


		// RENDER THE VALUE     Looks like  <td id="doc1_td_5_5" style="..." colspan="...">...content of a render function...</td>
		////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		htmlcode += '<td ';
		if (this.currentView != 'Header') htmlcode += ' id="' + this.prefix + 'td_' + row + '_' + col + '"';  
		htmlcode += this.renderStyle(cellObj.styleObj,"Value");
   
		if (colSpan >1 && cellObj.labelPosition != 'Top') {
			htmlcode += ' colspan="' + colSpan +  '" ';
		}
		htmlcode += '>';

		switch (attObj.name) {
			case 'Description':
			case 'ButtonRow':
			case 'Status':
			case 'Due':
			case 'Estimated':
				$("#mainWcTaskInfoRowId").hide(); // as soon as 1 element is placed in the specifications section, the rest must be placed there too. 
				$("#upperButtonRow").hide();
				// no break, we still want to execute what is under default
				
				$("#mainProjectDetailsRowId").hide();
			default:

				if (sourceField) {
					htmlcode += sourceField.innerHTML;
					//sourceField.style.visibility = 'hidden';
					$('#' + attObj.sourceSection).remove();
					
					
				} 

		}
		
		htmlcode += '</td>';
		if (attObj.name != 'buttonRow'  && !isSection) htmlcode += '<td/> ';
		
		
	} catch (err) {
		alert(this.langArr['ACatProblemIn']+ ' attributeCategory.renderProperty:\n' + err);
	}
	return htmlcode;
}

attributeCategory.prototype.addOtherProperties = function(elem) {

	try {
		this.otherProperties = [];
		if (this.isTaskTypePage) {
			this.otherProperties.push({name:'Description', sourceTd: 'descriptionValueTd', sourceSection: 'descriptionRow'});
			this.otherProperties.push({name:'ButtonRow', sourceTd: 'buttonRowValueTd', sourceSection: 'upperButtonRow'});
			this.otherProperties.push({name:'Status', sourceTd: 'statusValueTd', sourceSection: 'statusRow'});
			this.otherProperties.push({name:'AssignedTo', sourceTd: 'assigneeValueTd',sourceSection: 'assigneeRow' });
			this.otherProperties.push({name:'Due' , sourceTd: 'dueValueTd' ,sourceSection: 'dueDateRow'});
			this.otherProperties.push({name:'Estimated', sourceTd: 'estimatedValueTd'  ,sourceSection: 'estimatedDateRow'});
			this.otherProperties.push({name:'CheckList', sourceTd: 'checklistDetailsBlockId', sourceSection: 'mainChecklistDetailsRowId'});
			this.otherProperties.push({name:'Discussions', sourceTd: 'discussionDetailsBlockId', sourceSection: 'mainDiscussionDetailsRowId'});
			this.otherProperties.push({name:'Documents', sourceTd: 'documentsDetailsBlockId', sourceSection: 'mainDocumentsDetailsRowId'});
		} else {
			this.otherProperties.push({name:'ProjectName', sourceTd: 'projectNameValueTd', sourceSection: 'projectNameRowId'});
			this.otherProperties.push({name:'Requestor'});
			this.otherProperties.push({name:'ProjectManager'});
			this.otherProperties.push({name:'Status', sourceTd: 'PROJ_CREATION_STATUS', sourceSection: ''});
			this.otherProperties.push({name:'Customer', sourceTd: 'customerValueTd', sourceSection: ''});
			this.otherProperties.push({name:'Location', sourceTd: 'locationValueTd', sourceSection: ''});
			this.otherProperties.push({name:'Thumbnail', sourceTd: 'thumbnailValueTd', sourceSection: ''});
			this.otherProperties.push({name:'Description', sourceTd: 'descriptionValueTd', sourceSection: ''});
			this.otherProperties.push({name:'DueDate', sourceTd: 'dueDateValueTd', sourceSection: ''});		
			this.otherProperties.push({name:'Planning', sourceTd: 'planningValueTd', sourceSection: ''});
			this.otherProperties.push({name:'LastTaskEnds', sourceTd: 'lastTaskEndsValueTd', sourceSection: ''});  // also used for FirstTaskStarts
			this.otherProperties.push({name:'Members',sourceTd: 'projectMembersBlockTd', sourceSection: 'mainMembersRowId'});
			this.otherProperties.push({name:'BillOfMaterials',sourceTd: 'projectBOMBlockTd', sourceSection: 'mainBomDetailsRowId'});
			this.otherProperties.push({name:'Documents',sourceTd: 'projectDocumentsBlockId', sourceSection: 'mainDocumentDetailsRowId'});
		}
		// set placedArr
		for (var prop=0; prop<this.otherProperties.length; prop++) this.otherProperties[prop].placedArr = [];
		
		if (!this.galleryType) this.galleryType = 'Attributes';
		
    } catch(err) {
        alert("Problem in attributeCategory.addOtherProperties: " + err);
        return '';
    }
};

attributeCategory.prototype.initManageTEP = function() {

	try {
		//if (this.easterEgg) $("#mainWcTaskInfoRowId").hide();
		
		
    } catch(err) {
        alert("Problem in attributeCategory.initManageTEP: " + err);
        return '';
    }
};

function detectIfIE() {
      return /msie/i.test(navigator.userAgent) && !/opera/i.test(navigator.userAgent);
}

attributeCategory.prototype.fixStrictModeColor = function(color) {
	try {
		if(color && color.match(/^[0-9A-Fa-f]{6}$/g)){
			color = "#" + color;
		}
		return color;
	} catch(err) {
        alert("Problem in attributeCategory.fixStrictModeColor: " + err);
        return '';
    }
};


